!----------------------------------------------------------------------!
!                                                                      !
!                      Global Atmospheric Model                        !
!                                                                      !
!                          Zavisa I. Janjic                            !
!                                                                      !
!        The serial i,j,l F90 version coded by Zavisa I. Janjic        !
!           in January 2005 in Bethesda and Camp Springs, MD           !
!----------------------------------------------------------------------!
!                  Previous most significant upgrades                  !
!                                                                      !
!               Pressure-sigma hybrid coordinate version,              !
!                       July 2000, Bethesda, MD                        !
!                                                                      !
!         Direct solver for implicit vertical pressure equation,       !
!              NH module moved, no recomputation of fluxes,            !
!         Conservation only once, d2/dxdy in passive advection,        !
!                     December 2000, Bethesda, MD                      !
!                                                                      !
!                    Reformulation of surface layer,                   !
!                  iterative marine viscous sublayer,                  !
!             April-July 2001, Belgrade, YU, Bethesda, MD              !
!                                                                      !
!                   Moist turbulence closure scheme                    !
!              November 2001-January 2002, Bethesda, MD                !
!                                                                      !
!   Holtslag & de Bruin stability functions in stable surface layer    !
!                     January 2002, Bethesda, MD                       !
!                                                                      !
!              New land surface thermal parameterization               !
!              February 2002, Bethesda, MD, Belgrade, YU               !
!                                                                      !
!              New land surface hydrology parameterization             !
!           June-September 2002, Offenbach, DE, Bethesda, MD           !
!                                                                      !
!                          Global version                              !
!           September 2004, Bethesda, MD, Camp Springs, MD             !
!----------------------------------------------------------------------!
!                      This is the global version                      !
!            January 2005, Bethesda, MD, Camp Springs, MD              !
!                                                                      !
!        +1 301 657 3128; +1 301 657 2912; zjanjic@starpower.net       !
!----------------------------------------------------------------------!
!                                                                      !
!  Janjic, Z. I., J. P. Gerrity, Jr. and S. Nickovic, 2001: An         !
!  Alternative Approach to Nonhydrostatic Modeling.  Monthly Weather   !
!  Review, Vol. 129, 1164-1178.                                        !
!                                                                      !
!  Janjic, Z. I., 2003:  A Nonhydrostatic Model Based on a New         !
!  Approach.  Meteorology and Atmospheric Physics, Vol. 82, 271-285.   !
!                                                                      !
!----------------------------------------------------------------------!
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        program umo
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
use module_control
use module_dynamics
use module_fltbnds
use module_radiation
use module_sfcpbl
use module_moist
use module_surface,only:setsfc
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok0'
      call allocall
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok1'
      call consts
!---initialize filters--------------------------------------------------
      if(global) then
        call prefft &
        (im,jm &
        ,dlmd,dphd,sbd &
        ,khfilt,kvfilt &
        ,hfilt,vfilt &
        ,wfftrh,nfftrh,wfftrw,nfftrw)
        call presmud &
        (jm &
        ,dlmd,dphd,sbd &
        ,nhsmud)
      endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok2'
      call init
 1000 format(' *** Month=',i2,' Day=',i2,' Year=',i4,' Cycle=',i2)
      write(*,1000) idat(2),idat(1),idat(3),ihrst
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok3'
      call setsfc
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok3'
      call output
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      first=.true.
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok4'
      call pgforce &
      (first &
      ,ids,ide,jds,jde,jm,lm &
      ,dt,r,rdyv &
      ,dsg2,pdsg1 &
      ,rdxv,wpdar &
      ,fis,pd,pdo &
      ,t,q,cw,dwdt &
      ,pint &
      ,rtop &
      !temporary argument passing
      ,div,pcne,pcnw,pcx,pcy,tcu,tcv)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok5'
      call dht &
      (global &
      ,ids,ide,jds,jde,lm &
      ,cp,dyv &
      ,dsg2,pdsg1 &
      ,dxv,fcp,fdiv &
      ,pd,pdo &
      ,u,v,q,cw &
      ,omgalf &
      !temporary argument passing
      ,div,pcne,pcnw,pcx,pcy,pfne,pfnw,pfx,pfy)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
        if(global) then
          call fftfhn &
          (im,jm,lm &
          ,khfilt &
          ,hfilt &
          ,div &
          ,wfftrh,nfftrh)
          call swaphn(im,jm,lm,div)
          call swaphn(im,jm,lm,omgalf)
          call polehn(im,jm,lm,div)
          call polehn(im,jm,lm,omgalf)
        endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      first=.false.
!-----------------------------------------------------------------------
      do ntsd=ntsti,ntstm
!-----------------------------------------------------------------------
        print*,'ntsd=',ntsd,' hour=',(ntsd-1)*dt/3600.
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok6'
        call pdtsdt &
        (ids,ide,jds,jde,lm &
        ,dt &
        ,sg2 &
        ,pd &
        ,pdo,psdt &
        ,psgdt &
        !temporary argument passing
        ,div)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call swaph2(im,jm,pd)
            call swaph2(im,jm,psdt)
            call poleh2(im,jm,pd)
            call poleh2(im,jm,psdt)
            call swaphn(im,jm,lm-1,psgdt)
            call polehn(im,jm,lm-1,psgdt)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok7'
        call adv1 &
        (global,secadv &
        ,ids,ide,im,jds,jde,jm,lm,lnsad &
        ,dt,dyv,rdyh,rdyv &
        ,dsg2,pdsg1 &
        ,curv,dxv,fad,fah,rdxh,rdxv &
        ,f,pd,pdo &
        ,omgalf,psgdt &
        ,t,u,v &
        ,tp,up,vp &
        !temporary argument passing
        ,pfne,pfnw,pfx,pfy,tct,tcu,tcv)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok8'
        call vtoa &
        (ids,ide,jds,jde,lm &
        ,cp,dt,ef4t,pt &
        ,sg2 &
        ,psdt &
        ,dwdt,rtop,q,cw &
        ,omgalf &
        ,pint &
        !temporary argument passing
        ,div,tct)
!print*,'ok8.1'
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call swaphn(im,jm,lm,omgalf)
            call swaphn(im,jm,lm+1,pint)
            call polehn(im,jm,lm,omgalf)
            call polehn(im,jm,lm+1,pint)
          endif
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call fftfhn &
            (im,jm,lm &
            ,khfilt &
            ,hfilt &
            ,tct &
            ,wfftrh,nfftrh)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        call updatet &
        (ids,ide,jds,jde,lm &
        ,t &
        !temporary argument passing
        ,tct)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call swaphn(im,jm,lm,t)
            call polehn(im,jm,lm,t)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok9'
        call cdzdt &
        (global,hydro &
        ,ids,ide,im,jds,jde,jm,lm &
        ,dt,g,r &
        ,dsg2,pdsg1 &
        ,fah &
        ,fis,pd,pdo &
        ,psgdt &
        ,cw,q,rtop,t &
        ,pint &
        ,dwdt,pdwdt,w &
        ,z &
        !temporary argument passing
        ,pfne,pfnw,pfx,pfy)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call poavhn &
            (im,jm,lm &
            ,w)
            call swaphn(im,jm,lm,w)
            call polehn(im,jm,lm,w)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok10'
        call cdwdt &
        (global,hydro &
        ,ids,ide,im,jds,jde,jm,lm,ntsd &
        ,dt,g &
        ,dsg2,pdsg1,psgml1 &
        ,fah &
        ,hdacx,hdacy,pd,pdo &
        ,psgdt &
        ,dwdt,pdwdt,w &
        ,pint &
        !external scratch area
        ,def,pfne,pfnw,pfx,pfy)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call nhpoav &
            (im,jm,lm &
            ,dwdt)
            call fftfhn &
            (im,jm,lm &
            ,khfilt &
            ,hfilt &
            ,dwdt &
            ,wfftrh,nfftrh)
            call swaphn(im,jm,lm,dwdt)
            call polehn(im,jm,lm,dwdt)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok11'
        call vsound &
        (global,hydro &
        ,ids,ide,im,jds,jde,jm,lm,ntsd &
        ,cp,dt,g,pt,r &
        ,dsg2,pdsg1 &
        ,pd &
        ,cw,q,rtop &
        ,dwdt,t,w &
        ,pint)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call poavhn &
            (im,jm,lm &
            ,dwdt)
            call poavhn &
            (im,jm,lm &
            ,t)
            call poavhn &
            (im,jm,lm &
            ,w)
            call poavhn &
            (im,jm,lm+1 &
            ,pint)
            call swaphn(im,jm,lm,dwdt)
            call swaphn(im,jm,lm,t)
            call swaphn(im,jm,lm,w)
            call swaphn(im,jm,lm+1,pint)
            call polehn(im,jm,lm,dwdt)
            call polehn(im,jm,lm,t)
            call polehn(im,jm,lm,w)
            call polehn(im,jm,lm+1,pint)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        if(mod(ntsd-1,idtad).eq.0) then
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

          call adv2 &
          (idtad,ids,ide,jds,jde,lm,kss,kse &
          ,dt &
          ,dsg2,epsq2,pdsg1 &
          ,fah &
          ,pd,pdo &
          ,psgdt &
          ,q,cw,rrw,q2 &
          ,s,sp &
          !temporary arguments
          ,pfne,pfnw,pfx,pfy,s1,tcs)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok12' 
          call mono &
          (ids,ide,jds,jde,lm,kss,kse &
          ,dsg2,pdsg1 &
          ,dare &
          ,pd &
          ,s &
          !temporary arguments
          ,s1,tcs)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            do ks=kss,kse
              call fftfhn &
              (im,jm,lm &
              ,khfilt &
              ,hfilt &
              ,tcs(:,:,:,ks) &
              ,wfftrh,nfftrh)
            enddo
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          call updates &
          (ids,ide,jds,jde,lm &
          ,q &
          !temporary argument passing
          ,tcs(:,:,:,1))
          call updates &
          (ids,ide,jds,jde,lm &
          ,cw &
          !temporary argument passing
          ,tcs(:,:,:,2))
          call updates &
          (ids,ide,jds,jde,lm &
          ,rrw &
          !temporary argument passing
          ,tcs(:,:,:,3))
          call updates &
          (ids,ide,jds,jde,lm &
          ,q2 &
          !temporary argument passing
          ,tcs(:,:,:,4))
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call swaphn(im,jm,lm,q  )
            call swaphn(im,jm,lm,cw )
            call swaphn(im,jm,lm,rrw)
            call swaphn(im,jm,lm,q2 )
            call polehn(im,jm,lm,q  )
            call polehn(im,jm,lm,cw )
            call polehn(im,jm,lm,rrw)
            call polehn(im,jm,lm,q2 )
          endif
        endif
!---gla radiation-------------------------------------------------------
!print*,'ok13'
        call radtn &
        (ihrst,nradl,nrads,ntsd &
        ,dt &
        ,idat &
        ,dsg2,sg2,sgml2 &
        ,pdsg1,psg1,psgml1 &
        ,lbot,ltop &
        ,albase,albedo,snoalb,cldefi,glat,glon &
        ,pd,rswin,rlwin,sno,th02,ths,vegfrc &
        ,cw,q,t)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
            if(global) then
              call poleh2(im,jm,rswin)
              call poleh2(im,jm,rlwin)
!
              call swaph2(im,jm,rswin)
              call swaph2(im,jm,rlwin)
!
              call swaphn(im,jm,lm,t)
              call polehn(im,jm,lm,t)
            endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        if(mod(ntsd-1,nphs).eq.0) then
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok14'
          call turbl &
          (global &
          ,ids,ide,itb,jds,jde,jtb,kmsc,kztm,kztm2,lm,ntsd,nwetc &
          ,cp,dtq2,dzeta1,dzeta2,elivw,epsl,epsq2,fh01,fh02,r &
          ,rdq,rdth,rdyv,thl &
          ,qs0,sqs &
          ,ztmax1,ztmax2,ztmin1,ztmin2 &
          ,dsg2,pdsg1 &
          ,rdxv &
          ,insoil,inveg &
          ,psih1,psih2,psim1,psim2 &
          ,fis,pd,rlwin,rswin,sice,sm,sst,stdh,vegfrc &
          ,pint &
          ,ptbl &
          ,kmp,lpbl &
          ,akhs,akms,albedo,epsr,ghct,prcu,prra,prsn,q02,q10,qs,qz0 &
          ,roff,sno,th02,th10,ths,thz0,deept &
          ,u10,ustar,uz0,v10,vz0,wliq,z0 &
          ,tg,wg &
          ,cw,q,q2,t,u,v,w)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
            if(global) then
!              call fftfwn &
!              (im,jm,lm &
!              ,kvfilt &
!              ,vfilt &
!              ,u &
!              ,wfftrw,nfftrw)
!              call fftfwn &
!              (im,jm,lm &
!              ,kvfilt &
!              ,vfilt &
!              ,v &
!              ,wfftrw,nfftrw)
!              call fftfhn &
!              (im,jm,lm &
!              ,khfilt &
!              ,hfilt &
!              ,tct &
!              ,wfftrh,nfftrh)
!              call smudhn &
!              (im,jm,lm &
!              ,nhsmud &
!              ,cw)
!              call smudhn &
!              (im,jm,lm &
!              ,nhsmud &
!              ,q)
!              call smudhn &
!              (im,jm,lm &
!              ,nhsmud &
!              ,q2)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-------------latitudinal and polar boundary conditions-----------------
              call poleh2(im,jm,akhs)
              call poleh2(im,jm,akms)
              call poleh2(im,jm,q02)
              call poleh2(im,jm,q10)
              call poleh2(im,jm,qs)
              call poleh2(im,jm,qz0)
              call poleh2(im,jm,roff)
              call poleh2(im,jm,sno)
              call poleh2(im,jm,th02)
              call poleh2(im,jm,th10)
              call poleh2(im,jm,ths)
              call poleh2(im,jm,thz0)
              call poleh2(im,jm,u10)
              call poleh2(im,jm,v10)
              call poleh2(im,jm,ustar)
              call poleh2(im,jm,wliq)
!
              call swaph2(im,jm,akhs)
              call swaph2(im,jm,akms)
              call swaph2(im,jm,q02)
              call swaph2(im,jm,q10)
              call swaph2(im,jm,qs)
              call swaph2(im,jm,qz0)
              call swaph2(im,jm,roff)
              call swaph2(im,jm,sno)
              call swaph2(im,jm,th02)
              call swaph2(im,jm,th10)
              call swaph2(im,jm,ths)
              call swaph2(im,jm,thz0)
              call swaph2(im,jm,u10)
              call swaph2(im,jm,v10)
              call swaph2(im,jm,ustar)
              call swaph2(im,jm,wliq)
!
              call swapw2(im,jm,uz0)
              call swapw2(im,jm,vz0)
              call polew2(im,jm,uz0,vz0)
!
              call polshn(im,jm,kmsc,tg)
              call polshn(im,jm,nwetc,wg)
!
              call swpshn(im,jm,kmsc,tg)
              call swpshn(im,jm,nwetc,wg)
!
              call swaphn(im,jm,lm,t)
              call swaphn(im,jm,lm,q)
              call swaphn(im,jm,lm,cw)
              call swaphn(im,jm,lm,q2)
              call polehn(im,jm,lm,t)
              call polehn(im,jm,lm,q)
              call polehn(im,jm,lm,cw)
              call polehn(im,jm,lm,q2)
              call swapwn(im,jm,lm,u)
              call swapwn(im,jm,lm,v)
              call polewn(im,jm,lm,u,v)
            endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!nphs=99999
        if(mod(ntsd-1,nphs).eq.0) then
!+++ferrier microphysics++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok142'
!          if(.not.global) then
!            call cltend &
!            (-1,nphs &
!            ,t)
            call gsmdrive &
            (global &
            ,ntsd &
            ,dt,dtq2,dyh &
            ,dsg2,pdsg1 &
            ,acprec,pd,prec,prra,prsn &
            ,t &
            ,cw,q &
            ,pint)
!            call cltend &
!            (0,nphs &
!            ,t)
!            call cltend &
!            (1,nphs &
!            ,t)
!+++janjic 1990 MWR "microphysics"++++++++++++++++++++++++++++++++++++++
!          else
!print*,'ok15'
!            call rain &
!            (global &
!            ,im,jm,kmsc,lm &
!            ,cp,g,r &
!            ,psgml1,pdsg1,sgml2,dsg2 &
!            ,pd &
!            ,acprec,prec,prra,prsn &
!            ,tg &
!            ,q,t)
!          endif
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
!            if(global) then
!              call fftfhn &
!              (im,jm,lm &
!              ,khfilt &
!              ,hfilt &
!              ,tct &
!              ,wfftrh,nfftrh)
!            endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
            if(global) then
              call swaph2(im,jm,prra)
              call swaph2(im,jm,prsn)
              call swaph2(im,jm,prec)
              call swaph2(im,jm,acprec)
              call poleh2(im,jm,prra)
              call poleh2(im,jm,prsn)
              call poleh2(im,jm,prec)
              call poleh2(im,jm,acprec)
!
              call swaphn(im,jm,lm,cw)
              call polehn(im,jm,lm,cw)
              call swaphn(im,jm,lm,q)
              call polehn(im,jm,lm,q)
              call swaphn(im,jm,lm,t)
              call polehn(im,jm,lm,t)
            endif
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        if(mod(ntsd-1,ncnvc).eq.0) then
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok141'
          call cucnvc &
          (global &
          ,im,itb,jm,jtb,kmsc,lm,ncnvc &
          ,cp,dt,g,pl,pt,thl,rdp,rdq,rdth,rdthe &
          ,psgml1,pdsg1,sgml2,dsg2 &
          ,qs0,sqs,ptbl &
          ,the0,sthe,ttbl &
          ,pd,sm &
          ,lpbl,ltop,lbot &
          ,acprec,cldefi,cuppt,cuprec,deep &
          ,prcu,prec,prsn &
          ,tg &
          ,q,t)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
!            if(global) then
!              call fftfhn &
!              (im,jm,lm &
!              ,khfilt &
!              ,hfilt &
!              ,tct &
!              ,wfftrh,nfftrh)
!            endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
            if(global) then
              call swaph2(im,jm,prcu)
              call swaph2(im,jm,prsn)
              call swaph2(im,jm,prec)
              call swaph2(im,jm,cuppt)
              call swaph2(im,jm,cuprec)
              call swaph2(im,jm,acprec)
              call swaph2(im,jm,cldefi)
!
              call poleh2(im,jm,prcu)
              call poleh2(im,jm,prsn)
              call poleh2(im,jm,prec)
              call poleh2(im,jm,cuppt)
              call poleh2(im,jm,cuprec)
              call poleh2(im,jm,acprec)
              call poleh2(im,jm,cldefi)
!
              call swaphn(im,jm,lm,t)
              call swaphn(im,jm,lm,q)
!
              call polehn(im,jm,lm,t)
              call polehn(im,jm,lm,q)
            endif
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        call deform &
        (global,hydro &
        ,ids,ide,jds,jde,lm &
        ,rdyh &
        ,epsq2 &
        ,rare,rdxh,rdxv &
        ,q2,u,v,w &
!---temporary arguments-------------------------------------------------
        ,def)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call poavhn &
            (im,jm,lm &
            ,def)
            call swaphn(im,jm,lm,def)
            call polehn(im,jm,lm,def)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok17'
        call hdiff &
        (hydro &
        ,ids,ide,jds,jde,lm,lpt2 &
        ,rdyh &
        ,dsg2,pdsg1 &
        ,rdxh &
        ,sice,sm &
        ,hdacx,hdacy,hdacvx,hdacvy &
        ,pd &
        ,z &
        ,cw,q,q2,t,u,v &
!---temporary arguments-------------------------------------------------
        ,def)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call poavhn &
            (im,jm,lm &
            ,t)
            call poavhn &
            (im,jm,lm &
            ,q)
            call poavhn &
            (im,jm,lm &
            ,cw)
            call poavhn &
            (im,jm,lm &
            ,q2)
            call swaphn(im,jm,lm,t)
            call swaphn(im,jm,lm,q)
            call swaphn(im,jm,lm,cw)
            call swaphn(im,jm,lm,q2)
            call polehn(im,jm,lm,t)
            call polehn(im,jm,lm,q)
            call polehn(im,jm,lm,cw)
            call polehn(im,jm,lm,q2)
            call swapwn(im,jm,lm,u)
            call swapwn(im,jm,lm,v)
            call polewn(im,jm,lm,u,v)
          endif
!-----------------------------------------------------------------------
        if(.not.global) then
          readbc=ntsd.eq.1.or.mod(ntsd,nboco).eq.0
          if(readbc) then
            ihr=ntsd*dt/3600.+0.5
            ihrbc=ihr
            write(infile,'(a,i3.3)')'../output/boco.',ihrbc
            nbc=18
            open(unit=nbc,file=infile,status='old',form='unformatted')
            read (nbc) runbc,idatbc,ihrstbc,tboco
            read (nbc) pdbs,pdbn,pdbw,pdbe
            read (nbc) tbs,tbn,tbw,tbe
            read (nbc) qbs,qbn,qbw,qbe
            read (nbc) wbs,wbn,wbw,wbe
            read (nbc) ubs,ubn,ubw,ube
            read (nbc) vbs,vbn,vbw,vbe
            close(unit=nbc)
            !print*,'*** Read boundary conditions from ',infile
          endif
        endif
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(.not.global) then
            call bocoh &
            (im,jm,lm,lnsh &
            ,dt,pt &
            ,dsg2,pdsg1 &
            ,pd &
            ,pdbe,pdbn,pdbs,pdbw &
            ,tbe,tbn,tbs,tbw,qbe,qbn,qbs,qbw,wbe,wbn,wbs,wbw &
            ,t,q,cw &
            ,pint)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok18'
        call pgforce &
        (first &
        ,ids,ide,jds,jde,jm,lm &
        ,dt,r,rdyv &
        ,dsg2,pdsg1 &
        ,rdxv,wpdar &
        ,fis,pd,pdo &
        ,t,q,cw,dwdt &
        ,pint &
        ,rtop &
        !temporary argument passing
        ,div,pcne,pcnw,pcx,pcy,tcu,tcv)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call fftfwn &
            (im,jm,lm &
            ,kvfilt &
            ,vfilt &
            ,tcu &
            ,wfftrw,nfftrw)
            call fftfwn &
            (im,jm,lm &
            ,kvfilt &
            ,vfilt &
            ,tcv &
            ,wfftrw,nfftrw)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        call updateuv &
        (ids,ide,jds,jde,lm &
        ,u,v &
        !temporary argument passing
        ,tcu,tcv)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call poavhn &
            (im,jm,lm &
            ,rtop)
!            call fftfwn &
!            (im,jm,lm &
!            ,kvfilt &
!            ,vfilt &
!            ,u &
!            ,wfftrw,nfftrw)
!            call fftfwn &
!            (im,jm,lm &
!            ,kvfilt &
!            ,vfilt &
!            ,v &
!            ,wfftrw,nfftrw)
            call swapwn(im,jm,lm,u)
            call swapwn(im,jm,lm,v)
            call polewn(im,jm,lm,u,v)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          if(.not.global) then
            call bocov &
            (im,jm,lm,lnsv &
            ,dt &
            ,ube,ubn,ubs,ubw,vbe,vbn,vbs,vbw &
            ,u,v)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok19'
        call dht &
        (global &
        ,ids,ide,jds,jde,lm &
        ,cp,dyv &
        ,dsg2,pdsg1 &
        ,dxv,fcp,fdiv &
        ,pd,pdo &
        ,u,v,q,cw &
        ,omgalf &
        !temporary argument passing
        ,div,pcne,pcnw,pcx,pcy,pfne,pfnw,pfx,pfy)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call fftfhn &
            (im,jm,lm &
            ,khfilt &
            ,hfilt &
            ,div &
            ,wfftrh,nfftrh)
            call swaphn(im,jm,lm,div)
            call swaphn(im,jm,lm,omgalf)
            call polehn(im,jm,lm,div)
            call polehn(im,jm,lm,omgalf)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!print*,'ok20'
        call ddamp &
        (freerun &
        ,ids,ide,jds,jde,lm &
        ,ddmpv,pdtop &
        ,dsg2,pdsg1 &
        ,sg1,sg2 &
        ,ddmpu &
        ,pd,pdo &
        ,u,v &
        !temporary arguments passing
        ,div)
!+++filtering and boundary conditions+++++++++++++++++++++++++++++++++++
          if(global) then
            call swapwn(im,jm,lm,u)
            call swapwn(im,jm,lm,v)
            call polewn(im,jm,lm,u,v)
          endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        if(mod(ntsd,nout).eq.0) then
          ihr=ntsd*dt/3600.+0.5
          call output
        endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      enddo
!-----------------------------------------------------------------------
                        endprogram umo
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
