!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        module module_control
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
implicit none

include 'kind.inc'
include 'modelgrid.inc'
!---physical constants--------------------------------------------------
!integer(kind=kint),parameter:: &

real(kind=kfpt),parameter:: &
 a=6376000 &                 ! radius of earth
,a2=17.2693882 &             ! saturation spec. humidity formula coeff.
,a3=273.15 &                 ! saturation spec. humidity formula coeff.
,a4=35.86 &                  ! saturation spec. humidity formula coeff.
,cp=1004.6 &                 ! spec. heat for dry air at constant pressure
,eliwv=2.683e6 &             ! latent heat, mix ice/water - vapor
,elivw=2.72e6 &              ! another one
,g=9.8060226 &               ! gravity
,pi=3.141592653589793 &      ! ludolf number
,pihf=0.5*pi &               ! pi/2
,pq0=379.90516 &             ! water vapor pressure for tetens formula
,r=287.04 &                  ! gas constant for dry air
,ti0=271.35 &                ! water temperature below sea ice
,tiw=273.15 &                ! melting point
,twom=.00014584 &            ! 2 x angular velocity of earth
,cappa=r/cp &                ! kappa
,dtr=pi/180.                 ! factor converting degrees to radians
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---look-up tables------------------------------------------------------
integer(kind=kint),parameter:: &
 itb=201 &                   ! convection tables, dimension 1
,jtb=601 &                   ! convection tables, dimension 2
,kexm=10001 &                ! size of exponentiation table
,kztm=10001                  ! size of surface layer stability function table

real(kind=kfpt),parameter:: &
 ph=105000. &                ! upper bound of pressure range
,pl=5000. &                  ! lower bound of pressure range
,thh=350. &                  ! upper bound of potential temperature range
,thl=210.                    ! lower bound of potential temperature range

integer(kind=kint):: &
 kexm2 &                     ! internal points of exponentiation table
,kztm2                       ! internal pts. of the stability function table

real(kind=kfpt):: &
 dex &                       ! exponentiation table step
,dzeta1 &                    ! sea table z/L step
,dzeta2 &                    ! land table z/L step 
,fh01 &                      ! prandtl number for sea stability functions
,fh02 &                      ! prandtl number for land stability functions
!,pl &                        ! lower bound of pressure range
,rdp &                       ! scaling factor for pressure
,rdq &                       ! scaling factor for humidity
,rdth &                      ! scaling factor for potential temperature
,rdthe &                     ! scaling factor for equivalent pot. temperature
,rdex &                      ! exponentiation table scaling factor
,xmax &                      ! upper bound for exponent in the table
,xmin &                      ! lower bound for exponent in the table
,ztmax1 &                    ! upper bound for z/L for sea stab. functions
,ztmin1 &                    ! lower bound for z/L for sea stab. functions
,ztmax2 &                    ! upper bound for z/L for land stab. functions
,ztmin2                      ! lower bound for z/L for land stab. functions

real(kind=kfpt),dimension(1:itb):: &
 sthe &                      ! range for equivalent potential temperature
,the0                        ! base for equivalent potential temperature           

real(kind=kfpt),dimension(1:jtb):: &
 qs0 &                       ! base for saturation specific humidity
,sqs                         ! range for saturation specific humidity

real(kind=kfpt),dimension(1:kexm):: &
 expf                        ! exponentiation table

real(kind=kfpt),dimension(1:kztm):: &
 psih1 &                     ! sea heat stability function
,psim1 &                     ! sea momentum stability function
,psih2 &                     ! land heat stability function
,psim2                       ! land momentum stability function

real(kind=kfpt),dimension(1:itb,1:jtb):: &
 ptbl                        ! saturation pressure table

real(kind=kfpt),dimension(1:jtb,1:itb):: &
 ttbl                        ! temperature table
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---miscelaneous control parameters-------------------------------------
character(64):: &
 infile
 
logical(kind=klog):: &
 first &                     ! if true preparation for the first time step
,freerun &                   ! not a DA run
,global &                    ! global domain
,hydro &                     ! if true hydrostatic dynamics
,readbc &                    ! read regional boundary conditions
,run &                       ! initial data ready, start run
,runbc &                     ! boundary data ready, start run
,secadv                      ! second or fourth order momentum advection
 
integer(kind=kint):: &
 idtad &                     ! time steps between passive substance advection
,ierr &                      ! error code
,ihr &                       ! current forecast hour
,ihrbc &                     ! boundary condition hour
,ihrend &                    ! ending forecast time, hours
,ihrst &                     ! forecast starting time
,ihrstbc &                   ! boundary conditions starting time
,kse &                       ! terminal index for species^M
,ks &                        ! species index^M
,kss &                       ! initial index for species^M
,nbc &                       ! boundary data logical unit
,nboco &                     ! time steps between updating boundary conditions
,ncnvc &                     ! time steps between convection calls
,nfcst &                     ! initial data logical unit
,nout &                      ! time steps between outputs
,nphs &                      ! time steps between physics calls
,nprec &                     ! time steps for precip accumulation
,nradl &                     ! time steps between longwave radiation calls
,nrads &                     ! time steps between shortwave radiation calls
,nrain &                     ! time steps between grid scale precip calls
,nradpp &                    ! time steps for precipitation in radiation
,ntsti &                     ! initial time step
,ntstm &                     ! final time step
,ntsd                        ! current time step

integer(kind=kint),dimension(1:3):: &
 idat(3) &                   ! date of initial data, day, month, year
,idatbc(3)                   ! date of boundary data, day, month, year

integer(kind=kint),dimension(1:15):: & 
 nfftrh &                    ! fft working field, h points
,nfftrw                      ! fft working field, v points
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---grid constants------------------------------------------------------
integer(kind=kint):: &
 kmsc &                      ! max # of LSM layers
,ksnoc &                     ! number of snow layers in LSM scheme
,nosnoc &                    ! # of soil layers without sno
,nwetc                       ! # of soil layers with soil moisture

integer(kind=kint):: &
 icycle &                    ! # of independent points, x direction
,ide &                       ! ending data index, x direction
,ids &                       ! starting data index, x direction
,jde &                       ! ending data index, y direction
,jds &                       ! starting data index, y direction
,lnsad &                     ! # of boundary lines with upstream advection
,lnsbc &                     ! # of boundary lines with enhanced diffusion
,lpt2 &                      ! # of pressure layers
,n2                          ! # starting address of 3d scratch fields

real(kind=kfpt):: &
 ampl &                      ! amplitude of epsq2 variation
,bofac &                     ! amplification of diffusion along bndrs.
,ctph0 &                     ! cos(tph0)
,ddmpv &                     ! divergence damping coefficient, y direction
,dlm &                       ! grid size, delta lambda, radians
,dph &                       ! grid size, delta phi, radians
,dt &                        ! dynamics time step
,dtq2 &                      ! turbulence time step
,dyh &                       ! delta y, h points
,dyv &                       ! delta y, v points 
,ef4t &                      ! grid constant
,equi &                      ! proportionality between sqrt(2tke) & epsl
,f4d &                       ! grid constant
,pdtop &                     ! depth of pressure range in hybrid v. coord.
,rdlm &                      ! 1 / delta lambda
,rdph &                      ! 1 / delta phi
,rdyh &                      ! 1 / delta y, h points
,rdyv &                      ! 1 / delta y, v points
,sb &                        ! radians from center to southern boundary
,stph0 &                     ! sin(tph0)
,tboco &                     ! boundary conditions interval, hours
,tlm0 &                      ! radians grid rotated in lambda direction
,tph0 &                      ! radians grid rotated in phi direction
,wb                          ! radians from center to western boundary

real(kind=kfpt),allocatable,dimension(:):: &      !lm-1
 epsl                        ! floor values of master length scale

real(kind=kfpt),allocatable,dimension(:):: &      !lm
 dsg1 &                      ! thicnesses of sigma layers in pressure range
,dsg2 &                      ! thicnesses of sigma layers in sigma range
,epsq2 &                     ! floor values of 2tke
,pdsg1 &                     ! thicnesses of pressure layers in press. range
,psgml1 &                    ! pressure at midlayers in pressure range
,sgml1 &                     ! sigma at midlayers in pressure range
,sgml2                       ! sigma at midlayers in sigma range

real(kind=kfpt),allocatable,dimension(:):: &      !lm+1
 psg1 &                      ! pressure at interfaces in pressure range
,sg1 &                       ! sigma at interfaces in pressure range
,sg2 &                       ! sigma at interfaces in sigma range
,sgm                         ! sigma at interfaces

integer(kind=kint),allocatable,dimension(:):: &      !jm
 khfilt &                    ! polar filter, truncation wave #, h points
,kvfilt &                    ! polar filter, truncation wave #, v points
,nhsmud                      ! polar smoother for unfiltered variables

real(kind=kfpt),allocatable,dimension(:):: &      !jm
 curv &                      ! curvature term in coriolis force
,dare &                      ! gridbox area
,ddmpu &                     ! divergence damping coefficient, x direction
,ddv &                       ! gridbox diagonal distance 
,dxh &                       ! delta x, h points
,dxv &                       ! delta x, v points
,fad &                       ! momentum advection factor
,fah &                       ! z, w advection factor
,fcp &                       ! temperature advection factor
,fdiv &                      ! divergence factor
,rare &                      ! 1 / gridbox area
,rddv &                      ! 1 / gridbox diagonal distance 
,rdxh &                      ! 1 / delta x, h points
,rdxv &                      ! 1 / delta x, v points
,wpdar                       ! weight of grid separaton filter

real(kind=kfpt),allocatable,dimension(:):: &      !2*(im-3)
 wfftrh &                    ! fft working field, h points
,wfftrw                      ! fft working field, v points

real(kind=kfpt),allocatable,dimension(:,:):: &      !im,jm
 f &                         ! coriolis parameter
,glat &                      ! latitudes of h points
,glon &                      ! longitudes of h points
,hdacx &                     ! lateral diffusion coefficient, h points
,hdacy &                     ! lateral diffusion coefficient, h points
,hdacvx &                    ! lateral diffusion coefficient, v points
,hdacvy &                    ! lateral diffusion coefficient, v points
,hfilt &                     ! polar filter, h points
,vfilt                       ! polar filter, v points
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---fixed surface fields------------------------------------------------
integer(kind=kint),allocatable,dimension(:,:):: &      !im,jm
 insoil &                    ! soil type
,inveg                       ! vegetation type

real(kind=kfpt),allocatable,dimension(:,:):: &      !im,jm
 albase &                    ! base albedo
,albedo &                    ! albedo
,epsr &                      ! emissivity
,fis &                       ! surface geopotential
,sice &                      ! sea ice mask
,sm &                        ! sea masko
,snoalb &                    ! maximum snow albedo
,stdh &                      ! standard deviation of topography height
,sst &                       ! sea surface temperature
,vegfrc &                    ! vegetation fraction
,z0 &                        ! roughness
,z0b                         ! base roughness
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---surface variables---------------------------------------------------
integer(kind=kint),allocatable,dimension(:,:):: &      !im,jm
 kmp                         ! # of active soil layers

real(kind=kfpt),allocatable,dimension(:,:):: &      !im,jm
 akhs &                      ! heat exchange coeff. / sfc. layer depth
,akms &                      ! momentum exchange coeff. / sfc. layer depth
,acprec &                    ! accumulated total precipitation
,cldefi &                    ! convective cloud efficiency
,cuppt &                     ! convective precipitation for radiation
,cuprec &                    ! accumulated convective precipitation
,deep &                      ! deep convection mask, 0. or 1.
,ghct &                      ! countergradient correction
,prcu &                      ! physics time step convective liquid precip
,prec &                      ! total physics time step precipitation
,prra &                      ! physics time step grid scale liquid precip
,prsn &                      ! physics time step grid scale/conv. snow
,q02 &                       ! 2m specific humidity
,q10 &                       ! 10m specific humidity
,qs &                        ! specific humidity at the surface
,qz0 &                       ! spec. humidity at the top of viscous sublayer
,radin &                     ! total incoming radiation at the surface
,rlwin &                     ! downward lw at the surface
,rswin &                     ! downward sw at the surface
,roff &                      ! runoff
,sno &                       ! snow water equivalent
,th02 &                      ! 2m potential temperature
,th10 &                      ! 10m potential temperature
,ths &                       ! potential temperature at the surface
,thz0 &                      ! pot. temperature at the top of viscous sublayer
,deept &                     ! soil temperature lower boundary
,u10 &                       ! 10m u
,ustar &                     ! u star
,uz0 &                       ! u at the top of viscous sublayer
,v10 &                       ! 10m v
,vz0 &                       ! v at the top of viscous sublayer
,wliq                        ! canopy moisture

real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,jm,nwets
 wg                          ! soil moisture

real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,jm,kmsc
 tg                          ! soil temperature
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---regional boundary conditions----------------------------------------
real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,lnsh,2
 pdbn &                      ! pressure difference at northern boundary
,pdbs                        ! pressure difference at southern boundary

real(kind=kfpt),allocatable,dimension(:,:,:):: &      !lnsh,jm,2
 pdbe &                      ! pressure difference at eastern boundary
,pdbw                        ! pressure difference at western boundary

real(kind=kfpt),allocatable,dimension(:,:,:,:):: &    !im,lnsh,lm,2
 tbn &                       ! temperature at northern boundary
,tbs &                       ! temperature at southern boundary
,qbn &                       ! specific humidity at northern boundary
,qbs &                       ! specific humidity at southern boundary
,wbn &                       ! condensate at northern boundary
,wbs                         ! condensate at southern boundary

real(kind=kfpt),allocatable,dimension(:,:,:,:):: &    !lnsh,jm,lm,2
 tbe &                       ! temperature at eastern boundary
,tbw &                       ! temperature at western boundary
,qbe &                       ! specific humidity at eastern boundary
,qbw &                       ! specific humidity at western boundary
,wbe &                       ! condensate at eastern boundary
,wbw                         ! condensate at western boundary

real(kind=kfpt),allocatable,dimension(:,:,:,:):: &    !im,lnsv,lm,2
 ubn &                       ! u wind component at northern boundary
,ubs &                       ! u wind component at southern boundary
,vbn &                       ! v wind component at northern boundary
,vbs                         ! v wind component at southern boundary

real(kind=kfpt),allocatable,dimension(:,:,:,:):: &    !lnsv,im,lm,2
 ube &                       ! u wind component at eastern boundary
,ubw &                       ! u wind component at western boundary
,vbe &                       ! v wind component at eastern boundary
,vbw                         ! v wind component at western boundary
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---atmospheric variables, hydrostatic----------------------------------
real(kind=kfpt),allocatable,dimension(:,:):: &      !im,jm
 pd &                        ! pressure difference, sigma range
,pdo &                       ! previous pressure difference, sigma range
,psdt                        ! hydrostatic surface pressure tendency

real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,jm,lm-1
 psgdt                       ! vertical mass flux

real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,jm,lm
 cw &                        ! condensate
,cwp &                       ! previous condensate
,e2p &                       ! previous 2tke
,omgalf &                    ! omega-alpha
,q &                         ! specific humidity
,qp &                        ! previous specific humidity
,q2 &                        ! 2tke
,rrw &                       ! condensate species mixing ratio
,rrwp &                      ! previous condensate species mixing ratio
,t &                         ! temperature
,tp &                        ! previous temperature
,u &                         ! u wind component
,up &                        ! previous u wind component
,v &                         ! v wind component
,vp                          ! previous v wind component

real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,jm,lm+1
 pint                        ! nonhydrostatic pressure at interfaces

real(kind=kfpt),allocatable,dimension(:,:,:,:):: &    !im,jm,lm,ks^M
 s &                         ! tracers^M
,sp                          ! tracers at previous time level^M
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---atmospheric variables, nonhydrostatic-------------------------------
real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,jm,lm
 dwdt &                      ! vertical acceleration, correction factor
,pdwdt &                     ! previous correction factor
,rtop &                      ! RT/p, specific volume
,w &                         ! vertical velocity
,z                           ! height at midlayers
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---physics variables---------------------------------------------------
integer(kind=kint),allocatable,dimension(:,:):: &      !im,jm
 lbot &                      ! convective cloud base level
,lpbl &                      ! pbl top level
,ltop                        ! convective cloud top level
!---scratch area for passing temporary arguments------------------------
real(kind=kfpt),allocatable,dimension(:,:,:):: &      !im,jm,lm
 def &                       ! deformation
,div &                       ! horizontal mass divergence
,e2 &                        ! 2TKE in the layers
,pcne &                      ! second term of pgf, ne direction
,pcnw &                      ! second term of pgf, nw direction
,pcx &                       ! second term of pgf, x direction
,pcy &                       ! second term of pgf, y direction
,pfne &                      ! mass flux, ne direction
,pfnw &                      ! mass flux, nw direction
,pfx &                       ! mass flux, x direction
,pfy &                       ! mass flux, y direction
,tct &                       ! time change of temperature
,tcu &                       ! time change of u
,tcv                         ! time change of v

real(kind=kfpt),allocatable,dimension(:,:,:,:):: &    !im,jm,lm,ks
 s1 &                        ! intermediate tracers
,tcs                         ! tracer tendencies
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        contains
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine allocall
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---to read from a namelist in the future-------------------------------
      global=im.gt.imi
!-----------------------------------------------------------------------
      if(global) then
!---global branch-------------------------------------------------------
!
        dt=3600./32.
!
        lnsbc=lnsh
        bofac=0.
!
        lnsad=0
!-----------------------------------------------------------------------
      else
!---regional branch-----------------------------------------------------
!
        dt=3600./36./5.
!
        lnsbc=lnsh
        bofac=1.
!
        lnsad=lnsh+2
!-----------------------------------------------------------------------
      endif
!-----------------------------------------------------------------------
      kmsc=6
      ksnoc=1
      nosnoc=kmsc-ksnoc
      nwetc=nosnoc-1
!-----------------------------------------------------------------------
      if(global) then
        icycle=im-3
      else
        icycle=im
      endif
!-----------------------------------------------------------------------
      ids=1
      ide=im
      jds=1
      jde=jm
!
      kss=1
      kse=4
!---grid constants------------------------------------------------------
allocate( &      !lm-1
 epsl  (1:lm-1))             ! floor values of master length scales

allocate( &      !lm
 dsg1  (1:lm) &              ! thicnesses of sigma layers in pressure range
,dsg2  (1:lm) &              ! thicnesses of sigma layers in sigma range
,epsq2 (1:lm) &              ! floor values of 2tke
,pdsg1 (1:lm) &              ! thicnesses of pressure layers in press. range
,psgml1(1:lm) &              ! pressure at midlayers in pressure range
,sgml1 (1:lm) &              ! sigma at midlayers in pressure range
,sgml2 (1:lm))               ! sigma at midlayers in sigma range

allocate( &      !lm+1
 psg1 (1:lm+1) &             ! pressure at interfaces in pressure range
,sg1  (1:lm+1) &             ! sigma at interfaces in pressure range
,sg2  (1:lm+1) &             ! sigma at interfaces in sigma range
,sgm  (1:lm+1))              ! sigma at interfaces

allocate( &      !jm
 khfilt(1:jm) &              ! polar filter, truncation wave #, h points
,kvfilt(1:jm) &              ! polar filter, truncation wave #, v points
,nhsmud(1:jm))               ! polar smoother for unfiltered variables

allocate( &      !jm
 curv (1:jm) &               ! curvature term in coriolis force
,dare (1:jm) &               ! gridbox area
,ddmpu(1:jm) &               ! divergence damping coefficient, x direction
,ddv  (1:jm) &               ! gridbox diagonal distance 
,dxh  (1:jm) &               ! delta x, h points
,dxv  (1:jm) &               ! delta x, v points
,fad  (1:jm) &               ! momentum advection factor
,fah  (1:jm) &               ! z, w advection factor
,fcp  (1:jm) &               ! temperature advection factor
,fdiv (1:jm) &               ! divergence factor
,rare (1:jm) &               ! 1 / gridbox area
,rddv (1:jm) &               ! 1 / gridbox diagonal distance 
,rdxh (1:jm) &               ! 1 / delta x, h points
,rdxv (1:jm) &               ! 1 / delta x, v points
,wpdar(1:jm))                ! weight of grid separaton filter

allocate( &      !2*(icycle)
 wfftrh(1:2*icycle) &        ! fft working field, h points
,wfftrw(1:2*icycle))         ! fft working field, v points

allocate( &      !im,jm
 f     (1:im,1:jm) &         ! coriolis parameter
,glat  (1:im,1:jm) &         ! latitudes of h points
,glon  (1:im,1:jm) &         ! longitudes of h points
,hdacx (1:im,1:jm) &         ! lateral diffusion coefficient, h points
,hdacy (1:im,1:jm) &         ! lateral diffusion coefficient, h points
,hdacvx(1:im,1:jm) &         ! lateral diffusion coefficient, v points
,hdacvy(1:im,1:jm) &         ! lateral diffusion coefficient, v points
,hfilt (1:im,1:jm) &         ! polar filter, h points
,vfilt (1:im,1:jm))          ! polar filter, v points
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---fixed surface fields------------------------------------------------
allocate( &      !im,jm
 insoil(1:im,1:jm) &         ! soil type
,inveg (1:im,1:jm))          ! vegetation type

allocate( &      !im,jm
 albase(1:im,1:jm) &         ! base albedo
,albedo(1:im,1:jm) &         ! albedo
,epsr  (1:im,1:jm) &         ! emissivity
,fis   (1:im,1:jm) &         ! surface geopotential
,sice  (1:im,1:jm) &         ! sea ice mask
,sm    (1:im,1:jm) &         ! sea mask
,snoalb(1:im,1:jm) &         ! maximum snow albedo
,stdh  (1:im,1:jm) &         ! standard deviation of topography height
,sst   (1:im,1:jm) &         ! sea surface temperature
,vegfrc(1:im,1:jm) &         ! vegetation fraction
,z0    (1:im,1:jm) &         ! roughness
,z0b   (1:im,1:jm))          ! base roughness
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---surface variables---------------------------------------------------
allocate( &      !im,jm
 kmp(1:im,1:jm))             ! # of active soil layers

allocate( &      !im,jm
 akhs  (1:im,1:jm) &         ! heat exchange coeff. / sfc. layer depth
,akms  (1:im,1:jm) &         ! momentum exchange coeff. / sfc. layer depth
,acprec(1:im,1:jm) &         ! accumulated total precipitation
,cldefi(1:im,1:jm) &         ! convective cloud efficiency
,cuppt (1:im,1:jm) &         ! convective precipitation for radiation
,cuprec(1:im,1:jm) &         ! accumulated convective precipitation
,deep  (1:im,1:jm) &         ! deep convection mask, 0. or 1.
,ghct  (1:im,1:jm) &         ! countergradient correction
,prcu  (1:im,1:jm) &         ! physics time step convective liquid precip
,prec  (1:im,1:jm) &         ! total physics time step precipitation
,prra  (1:im,1:jm) &         ! physics time step grid scale liquid precip
,prsn  (1:im,1:jm) &         ! physics time step grid scale/conv. snow
,q02   (1:im,1:jm) &         ! 2m specific humidity
,q10   (1:im,1:jm) &         ! 10m specific humidity
,qs    (1:im,1:jm) &         ! specific humidity at the surface
,qz0   (1:im,1:jm) &         ! spec. humidity at the top of viscous sublayer
,radin (1:im,1:jm) &         ! total incoming radiation at the surface
,rlwin (1:im,1:jm) &         ! downward lw at the surface
,rswin (1:im,1:jm) &         ! downward sw at the surface
,roff  (1:im,1:jm) &         ! runoff
,sno   (1:im,1:jm) &         ! snow water equivalent
,th02  (1:im,1:jm) &         ! 2m potential temperature
,th10  (1:im,1:jm) &         ! 10m potential temperature
,ths   (1:im,1:jm) &         ! potential temperature at the surface
,thz0  (1:im,1:jm) &         ! pot. temperature at the top of viscous sublayer
,deept (1:im,1:jm) &         ! soil temperature lower boundary
,u10   (1:im,1:jm) &         ! 10m u
,ustar (1:im,1:jm) &         ! u star
,uz0   (1:im,1:jm) &         ! u at the top of viscous sublayer
,v10   (1:im,1:jm) &         ! 10m v
,vz0   (1:im,1:jm) &         ! v at the top of viscous sublayer
,wliq  (1:im,1:jm))          ! canopy moisture

allocate( &      !im,jm,nwetc
 wg(1:im,1:jm,1:nwetc))      ! soil moisture

allocate( &      !im,jm,kmsc
 tg(1:im,1:jm,1:kmsc))       ! soil temperature
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---regional boundary conditions----------------------------------------
if(.not.global) then
allocate( &      !im,lnsh,2
 pdbn(1:im,1:lnsh,1:2) &     ! pressure difference at northern boundary
,pdbs(1:im,1:lnsh,1:2))      ! pressure difference at southern boundary

allocate( &      !lnsh,jm,2
 pdbe(1:lnsh,1:jm,1:2) &     ! pressure difference at eastern boundary
,pdbw(1:lnsh,1:jm,1:2))      ! pressure difference at western boundary

allocate( &     !im,lnsh,lm,2
 qbn(1:im,1:lnsh,1:lm,1:2) & ! specific humidity at northern boundary
,qbs(1:im,1:lnsh,1:lm,1:2) & ! specific humidity at southern boundary
,tbn(1:im,1:lnsh,1:lm,1:2) & ! temperature at northern boundary
,tbs(1:im,1:lnsh,1:lm,1:2) & ! temperature at southern boundary
,wbn(1:im,1:lnsh,1:lm,1:2) & ! condensate at northern boundary
,wbs(1:im,1:lnsh,1:lm,1:2))  ! condensate at southern boundary

allocate( &     !lnsh,jm,lm,2
 qbe(1:lnsh,1:jm,1:lm,1:2) & ! specific humidity at eastern boundary
,qbw(1:lnsh,1:jm,1:lm,1:2) & ! specific humidity at western boundary
,tbe(1:lnsh,1:jm,1:lm,1:2) & ! temperature at eastern boundary
,tbw(1:lnsh,1:jm,1:lm,1:2) & ! temperature at western boundary
,wbe(1:lnsh,1:jm,1:lm,1:2) & ! condensate at eastern boundary
,wbw(1:lnsh,1:jm,1:lm,1:2))  ! condensate at western boundary

allocate( &     !im,lnsv,lm,2
 ubn(1:im,1:lnsv,1:lm,1:2) & ! u wind component at northern boundary
,ubs(1:im,1:lnsv,1:lm,1:2) & ! u wind component at southern boundary
,vbn(1:im,1:lnsv,1:lm,1:2) & ! v wind component at northern boundary
,vbs(1:im,1:lnsv,1:lm,1:2))  ! v wind component at southern boundary

allocate( &     !lnsv,jm,lm,2
 ube(1:lnsv,1:jm,1:lm,1:2) & ! u wind component at eastern boundary
,ubw(1:lnsv,1:jm,1:lm,1:2) & ! u wind component at western boundary
,vbe(1:lnsv,1:jm,1:lm,1:2) & ! v wind component at eastern boundary
,vbw(1:lnsv,1:jm,1:lm,1:2))  ! v wind component at western boundary
endif
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---atmospheric variables, hydrostatic----------------------------------
allocate( &      !im,jm
 pd  (1:im,1:jm) &           ! pressure difference, sigma range
,pdo (1:im,1:jm) &           ! previous pressure difference, sigma range
,psdt(1:im,1:jm))            ! hydrostatic surface pressure tendency

allocate( &      !im,jm,lm-1
 psgdt (1:im,1:jm,1:lm-1))   ! specific volume

allocate( &      !im,jm,lm
 cw    (1:im,1:jm,1:lm) &    ! condensate
,cwp   (1:im,1:jm,1:lm) &    ! previous condensate
,e2p   (1:im,1:jm,1:lm) &    ! previous 2tke
,omgalf(1:im,1:jm,1:lm) &    ! omega-alpha
,q     (1:im,1:jm,1:lm) &    ! specific humidity
,qp    (1:im,1:jm,1:lm) &    ! previous specific humidity
,q2    (1:im,1:jm,1:lm) &    ! 2tke
,rrw   (1:im,1:jm,1:lm) &    ! condensate species mixing ratio
,rrwp  (1:im,1:jm,1:lm) &    ! previous condensate species mixing ratio
,t     (1:im,1:jm,1:lm) &    ! temperature
,tp    (1:im,1:jm,1:lm) &    ! previous temperature
,u     (1:im,1:jm,1:lm) &    ! u wind component
,up    (1:im,1:jm,1:lm) &    ! previous u wind component
,v     (1:im,1:jm,1:lm) &    ! v wind component
,vp    (1:im,1:jm,1:lm))     ! previous v wind component

allocate( &      !im,jm,lm+1
 pint(1:im,1:jm,1:lm+1))     ! total pressure at interfaces

allocate( &      !im,jm,lm,kss:kse
 s (1:im,1:jm,1:lm,kss:kse) & ! tracers
,sp(1:im,1:jm,1:lm,kss:kse))  ! tracers at previous time level
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---atmospheric variables, nonhydrostatic-------------------------------
allocate( &      !im,jm,lm
 dwdt (1:im,1:jm,1:lm) &     ! vertical acceleration, correction factor
,pdwdt(1:im,1:jm,1:lm) &     ! previous correction factor
,rtop (1:im,1:jm,1:lm) &     ! RT/p, specific volume
,w    (1:im,1:jm,1:lm) &     ! vertical velocity
,z    (1:im,1:jm,1:lm))      ! height at midlayers
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---physics variables---------------------------------------------------
allocate( &      !im,jm
 lbot (1:im,1:jm) &          ! convective cloud base level
,lpbl (1:im,1:jm) &          ! pbl top level
,ltop (1:im,1:jm))           ! convective cloud top level
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!---scratch area for passing temporary arguments------------------------
allocate( &      !im,jm,lm
 def  (1:im,1:jm,1:lm) &     ! deformation
,div  (1:im,1:jm,1:lm) &     ! horizontal mass divergence
,e2   (1:im,1:jm,1:lm) &     ! 2TKE in the layers
,pcne (1:im,1:jm,1:lm) &     ! second term of pgf, ne direction
,pcnw (1:im,1:jm,1:lm) &     ! second term of pgf, nw direction
,pcx  (1:im,1:jm,1:lm) &     ! second term of pgf, x direction
,pcy  (1:im,1:jm,1:lm) &     ! second term of pgf, y direction
,pfne (1:im,1:jm,1:lm) &     ! mass flux, ne direction
,pfnw (1:im,1:jm,1:lm) &     ! mass flux, nw direction
,pfx  (1:im,1:jm,1:lm) &     ! mass flux, x direction
,pfy  (1:im,1:jm,1:lm) &     ! mass flux, y direction
,tct  (1:im,1:jm,1:lm) &     ! time change of temperature
,tcu  (1:im,1:jm,1:lm) &     ! time change of u
,tcv  (1:im,1:jm,1:lm))      ! time change of v

allocate( &      !im,jm,lm,kss:kse
 s1 (1:im,1:jm,1:lm,kss:kse) & ! intermediate species
,tcs(1:im,1:jm,1:lm,kss:kse))  ! time change of s
!-----------------------------------------------------------------------
                        endsubroutine allocall
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine consts
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j                           ! index in y direction

real(kind=kfpt):: &
 acdt &                      ! diffusion coefficient parameter
,alm &                       ! lambda
,anum &                      ! numerator
,aph &                       ! phi
,ave &                       ! average
,cddamp &                    ! divergence damping factor
,coac &                      ! nonlinear diffusion coefficient
,codamp &                    ! divergence damping coefficient
,ctlm &                      ! temporary
,denom &                     ! denominator
,relm &                      ! temporary
,fpole &                     ! factor to modify polar area
,smag2 &                     ! Smagorinsky
,sph &                       ! temporary
,stlm &                      ! temporary
,stph &                      ! temporary
,ctph &                      ! temporary
,tlm &                       ! rotated lambda
,tout &                      ! time in hours between outputs
,tph &                       ! rotated phi
,tpv &                       ! rotated phi, v points
,wcor                        ! grid separation filtering coefficient

real(kind=kfpt),dimension(1:jm):: &
 hdacxj &                    !
,hdacyj &                    !
,hdacvxj &                   !
,hdacvyj                     !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
 1000 format(100a4)
!---to be read from a namelist in the future----------------------------
      hydro=.false.
!      hydro=.true.
!
      idtad=2
      nphs=4
      ncnvc=nphs
      nrads=3600./dt+0.5
      nradl=3600./dt*3.+0.5
      dtq2=nphs*dt
!
      tout=12.
      nout=tout*3600./dt+0.5
      nprec=12.*3600./dt+0.5
!
      if(.not.global) then
        ihrbc=0
        write(infile,'(a,i3.3)')'../output/boco.',ihrbc
        nbc=18
        open(unit=nbc,file=infile,status='old',form='unformatted')
        read (nbc) runbc,idatbc,ihrstbc,tboco
        rewind nbc
        close(unit=nbc)
        print*,'*** Read tboco in consts from ',infile
        nboco=tboco/dt+0.5
      endif
!---to be read from a namelist in the future----------------------------
      freerun=.true.
      secadv=.true.
!
      if(global) then
        smag2=0.3
        codamp=9.0
      else
        smag2=0.3 !regional 2-nd order
        codamp=9.0
      endif
      coac=smag2*smag2
!
!      wcor=.0075
!      wcor=.15
!      wcor=.05
!      wcor=0.
      wcor=.2000
!--------------derived geometrical constants----------------------------
      tph0=tph0d*dtr                                                    
      wb=wbd*dtr
      sb=sbd*dtr
      dlm=dlmd*dtr                                                      
      dph=dphd*dtr                                                      
      rdlm=1./dlm                                                       
      rdph=1./dph                                                       
!                                                                       
      stph0=sin(tph0)                                                   
      ctph0=cos(tph0)                                                   
!---derived horizontal grid constants-----------------------------------
      dyh=a*dph
      dyv=a*dph
      rdyh=1./dyh
      rdyv=1./dyv
!---derived vertical grid constants-------------------------------------
      ef4t=0.5*dt/cp
      f4d=-0.5*dt
!---derived physical constants------------------------------------------
      acdt=coac*dt
      cddamp=codamp*dt
!-----------------------------------------------------------------------
      if(global) then !global
!---global branch-------------------------------------------------------
        fpole=1.0
!----south pole---------------------------------------------------------
        tph=sb
        tpv=tph+dph*0.5                                               
        dxh(2)=0.
        dxv(2)=a*dlm*cos(tpv)
        ddmpv=cddamp*dyv/(2.*dyv)
!
        rdxh(2)=0.
        rdxv(2)=1./dxv(2)
        dare(2)=dxv(2)*dyv*0.5*fpole !for ghost area
        rare(2)=1./dare(2)
        wpdar(2)=-wcor*dyh**2 &
                /(dt*dxv(2)*dyv*0.5*fpole)/100000.00
        curv(2)=tan(tpv)/a
        fad(2)=-dt/(3.   *dxv(2)*dyv*2.*2.)
        fah(2)=-dt/(3.   *dxv(2)*dyv*0.5*fpole)
        fcp(2)= dt/(3.*cp*dxv(2)*dyv*0.5*fpole)
        fdiv(2)=2./(3.   *dxv(2)*dyv*0.5*fpole)
!
        hdacxj(2)=0.
        hdacyj(2)=acdt*dyh**2 &
                  /(4.*sqrt(2.)*dxv(2)*dyv*0.5*fpole)
        hdacvxj(2)=acdt*dxv(2)*dyv &
                  /(4.*sqrt(2.)*dxv(2)*dyv)
        hdacvyj(2)=acdt*dyv   **2 &
                  /(4.*sqrt(2.)*dxv(2)*dyv)
!
!        ddmpu(2)=cddamp*dxv(2)/(2.*dxv(2))
        ddmpu(2)=cddamp*dyv/(2.*dxv(2))
!-------------between the poles----------------------------------------- 
        do j=3,jm-2
          tph=(j-2)*dph+sb
          tpv=tph+dph*0.5
          dxh(j)=a*dlm*cos(tph)
          dxv(j)=a*dlm*cos(tpv)
          rdxh(j)=1./dxh(j)
          rdxv(j)=1./dxv(j)
          dare(j)=dxh(j)*dyh
          rare(j)=1./dare(j)
          wpdar(j)=-wcor*(dyh)**2 &
                  /(dt*dxh(j)*dyh)/100000.00
          curv(j)=tan(tpv)/a
          fad(j)=-dt/(3.   *dxv(j)*dyv*2.*2.)
          fah(j)=-dt/(3.   *dxh(j)*dyh)
          fcp(j)= dt/(3.*cp*dxh(j)*dyh)
          fdiv(j)=2./(3.   *dxh(j)*dyh)
          hdacxj(j)=  acdt*dxh(j)*dyh         /(4.*sqrt(2.)*dxh(j)*dyh)
          hdacyj(j)=  acdt*max(dxh(j),dyh)*dyh/(4.*sqrt(2.)*dxh(j)*dyh)
          hdacvxj(j)= acdt*dxv(j)*dyv         /(4.*sqrt(2.)*dxv(j)*dyv)
          hdacvyj(j)= acdt*max(dxv(j),dyv)*dyv/(4.*sqrt(2.)*dxv(j)*dyv)
!          ddmpu(j)=cddamp*dxv(j)/(2.*dxv(j))
          ddmpu(j)=cddamp*dyv/(2.*dxv(j))
        enddo
!-------------ghost line beyond the south pole---------------------------- 
        dxh(1)=dxh(3)
        dxv(1)=dxv(2)
        rdxh(1)=rdxh(3)
        rdxv(1)=rdxv(2)
        dare(1)=dare(3)
        rare(1)=rare(3)
        wpdar(1)=wpdar(3)
        curv(1)=curv(2)
        fad(1)=fad(2)
        fah(1)=fah(3)
        fcp(1)=fcp(3)
        fdiv(1)=fdiv(3)
        hdacxj(1)=hdacxj(3)
        hdacyj(1)=hdacyj(3)
        hdacvxj(1)=hdacvxj(2)
        hdacvyj(1)=hdacvyj(2)
        ddmpu(1)=ddmpu(2)
!-------------north pole------------------------------------------------
        tph=tph+dph
        tpv=tph+dph*0.5
        dxh(jm-1)=0.
        rdxh(jm-1)=0.
        dare(jm-1)=dxv(jm-2)*dyv*0.5*fpole
        rare(jm-1)=1./dare(jm-1)
        wpdar(jm-1)=-wcor*(dyh)**2 &
                   /(dt*dxv(jm-2)*dyv*0.5*fpole)/100000.00
        fah(jm-1)=-dt/(3.   *dxv(jm-2)*dyv*0.5*fpole)
        fcp(jm-1)= dt/(3.*cp*dxv(jm-2)*dyv*0.5*fpole)
        fdiv(jm-1)=2./(3.   *dxv(jm-2)*dyv*0.5*fpole)
!zjtest        hdacj(jm-1)=acdt*16.*dxh(jm-1) &
!zjtest                   /(4.*sqrt(2.)*dxv(jm-2)*dyv*0.5*fpole)
        hdacxj(jm-1)=0.
        hdacyj(jm-1)=acdt*dyh**2 &
                    /(4.*sqrt(2.)*dxv(jm-2)*dyv*0.5*fpole)
!-------------ghost line beyond north pole------------------------------
        dxh(jm)=dxh(jm-2)
        dxv(jm-1)=dxv(jm-2)
        dxv(jm)=dxv(jm-2)
        rdxh(jm)=rdxh(jm-2)
        rdxv(jm-1)=rdxv(jm-2)
        rdxv(jm)=rdxv(jm-2)
        dare(jm)=dare(jm-2)
        rare(jm)=rare(jm-2)
        wpdar(jm)=wpdar(jm-2)
        curv(jm-1)=curv(jm-2)
        curv(jm-1)=curv(jm-2)
        fad(jm-1)=fad(jm-2)
        fad(jm)=fad(jm-2)
        fah(jm)=fah(jm-2)
        fcp(jm)=fcp(jm-2)
        fdiv(jm)=fdiv(jm-2)
        hdacxj(jm)=hdacxj(jm-2)
        hdacyj(jm)=hdacyj(jm-2)
        hdacvxj(jm-1)=hdacvxj(jm-2)
        hdacvyj(jm-1)=hdacvyj(jm-2)
        hdacvxj(jm)=hdacvxj(jm-2)
        hdacvyj(jm)=hdacvyj(jm-2)
        ddmpu(jm-1)=ddmpu(jm-2)
        ddmpu(jm)=ddmpu(jm-2)
!-------------averaging over height latitudes for accuracy-------------- 
        do j=1,jm/2
          ave=(dxh(j)+dxh(jm+1-j))*0.5
          dxh(j)=ave
          dxh(jm+1-j)=ave
          ave=(rdxh(j)+rdxh(jm+1-j))*0.5
          rdxh(j)=ave
          rdxh(jm+1-j)=ave
          ave=(dare(j)+dare(jm+1-j))*0.5
          dare(j)=ave
          dare(jm+1-j)=ave
          ave=(rare(j)+rare(jm+1-j))*0.5
          rare(j)=ave
          rare(jm+1-j)=ave
          ave=(wpdar(j)+wpdar(jm+1-j))*0.5
          wpdar(j)=ave
          wpdar(jm+1-j)=ave
          ave=(fah(j)+fah(jm+1-j))*0.5
          fah(j)=ave
          fah(jm+1-j)=ave
          ave=(fcp(j)+fcp(jm+1-j))*0.5
          fcp(j)=ave
          fcp(jm+1-j)=ave
          ave=(fdiv(j)+fdiv(jm+1-j))*0.5
          fdiv(j)=ave
          fdiv(jm+1-j)=ave
          ave=(hdacxj(j)+hdacxj(jm+1-j))*0.5
          hdacxj(j)=ave
          hdacxj(jm+1-j)=ave
          ave=(hdacyj(j)+hdacyj(jm+1-j))*0.5
          hdacyj(j)=ave
          hdacyj(jm+1-j)=ave
        enddo
!-------------averaging over wind latitudes for accuracy--------------- 
        do j=1,(jm-1)/2
          ave=(dxv(j)+dxv(jm-j))*0.5
          dxv(j)=ave
          dxv(jm-j)=ave
          ave=(rdxv(j)+rdxv(jm-j))*0.5
          rdxv(j)=ave
          rdxv(jm-j)=ave
          ave=(curv(j)+curv(jm-j))*0.5
          curv(j)=ave
          curv(jm-j)=ave
          ave=(fad(j)+fad(jm-j))*0.5
          fad(j)=ave
          fad(jm-j)=ave
          ave=(hdacvxj(j)+hdacvxj(jm-j))*0.5
          hdacvxj(j)=ave
          hdacvxj(jm-j)=ave
          ave=(hdacvyj(j)+hdacvyj(jm-j))*0.5
          hdacvyj(j)=ave
          hdacvyj(jm-j)=ave
          ave=(ddmpu(j)+ddmpu(jm-j))*0.5
          ddmpu(j)=ave
          ddmpu(jm-j)=ave
        enddo
!-------------diagonal distances at v points----------------------------
        ddv(2)=dyv
        ddv(1)=ddv(2)
        rddv(2)=1./ddv(2)
        rddv(1)=rddv(2)
        do j=3,jm-3
          ddv(j)=sqrt(dxv(j)**2+dyv**2)
          rddv(j)=1./ddv(j)
        enddo
        ddv(jm-2)=dyv
        ddv(jm-1)=ddv(jm-2)
        rddv(jm-2)=1./ddv(jm-2)
        rddv(jm-1)=rddv(jm-2)
!-------------defining the diffusion coefficient inside the domain------
        do j=1,jm
          do i=1,im
            hdacx(i,j)=hdacxj(j)
            hdacy(i,j)=hdacyj(j)
            hdacvx(i,j)=hdacvxj(j)
            hdacvy(i,j)=hdacvyj(j)
          enddo
        enddo
!-------------coriolis parameter in tll system--------------------------
        tph=sb-dph-dph*.5
        do j=1,jm
          tph=tph+dph
          stph=sin(tph)
          ctph=cos(tph)
          tlm=wb-dlm-dlm*0.5
          do i=1,im
            tlm=tlm+dlm
            f(i,j)=twom*(ctph0*stph+stph0*ctph*cos(tlm))
          enddo
        enddo
!--------------latitudes and longitudes of h points in radians----------     
        tph=sb-dph-dph
        do j=1,jm
          tlm=wb-dlm-dlm
          tph=tph+dph
          stph=sin(tph)
          ctph=cos(tph)                                                                       
          do i=1,im
            tlm=tlm+dlm
            stlm=sin(tlm)
            ctlm=cos(tlm)
            sph=ctph0*stph+stph0*ctph*ctlm
            aph=asin(sph)
            glat(i,j)=aph
            anum=ctph*stlm
            denom=(ctlm*ctph-stph0*sph)/ctph0
            relm=atan2(anum,denom)
            alm=relm+tlm0d*dtr
            if(alm.gt. pi) alm=alm-pi-pi
            if(alm.lt.-pi) alm=alm+pi+pi
            glon(i,j)=alm
          enddo
        enddo
!-----------------------------------------------------------------------
      else !regional
!---regional branch-----------------------------------------------------
        tph=sb-dph
        ddmpv=cddamp*dyv/(2.*dyv)
!-------------between the poles----------------------------------------- 
        do j=1,jm
          tph=tph+dph
          tpv=tph+dph*0.5
          dxh(j)=a*dlm*cos(tph)
          dxv(j)=a*dlm*cos(tpv)
          rdxh(j)=1./dxh(j)
          rdxv(j)=1./dxv(j)
          dare(j)=dxh(j)*dyh
          rare(j)=1./dare(j)
          wpdar(j)=-wcor*dyh**2 &
                  /(dt*dxh(j)*dyh)/100000.00
          curv(j)=tan(tpv)/a
          fad(j)=-dt/(3.   *dxv(j)*dyv*2.*2.)
          fah(j)=-dt/(3.   *dxh(j)*dyh)
          fcp(j)= dt/(3.*cp*dxh(j)*dyh)
          fdiv(j)=2./(3.   *dxh(j)*dyh)
          hdacxj(j)=  acdt*dxh(j)*dyh/(4.*sqrt(2.)*dxh(j)*dyh)
          hdacyj(j)=  acdt*dxh(j)*dyh/(4.*sqrt(2.)*dxh(j)*dyh)
          hdacvxj(j)= acdt*dxv(j)*dyv/(4.*sqrt(2.)*dxv(j)*dyv)
          hdacvyj(j)= acdt*dxv(j)*dyv/(4.*sqrt(2.)*dxv(j)*dyv)
          ddmpu(j)=cddamp*dxv(j)/(2.*dxv(j))
        enddo
!-------------diagonal distances at v points----------------------------
        do j=1,jm
          ddv(j)=sqrt(dxv(j)**2+dyv**2)
          rddv(j)=1./ddv(j)
        enddo
!---defining the diffusion coefficient inside the domain----------------
        do j=1,jm
          do i=1,im
            hdacx(i,j)=hdacxj(j)
            hdacy(i,j)=hdacyj(j)
            hdacvx(i,j)=hdacvxj(j)
            hdacvy(i,j)=hdacvyj(j)
          enddo
        enddo
!---enhancing diffusion along boundaries--------------------------------
        do j=2,lnsbc
          do i=2,im-1
            hdacx(i,j)=hdacxj(j)*bofac       
            hdacy(i,j)=hdacyj(j)*bofac       
          enddo
        enddo
        do j=jm-lnsbc+1,jm-1
          do i=2,im-1
            hdacx(i,j)=hdacxj(j)*bofac       
            hdacy(i,j)=hdacyj(j)*bofac       
          enddo
        enddo
        do j=lnsbc+1,jm-lnsbc
          do i=2,lnsbc
            hdacx(i,j)=hdacxj(j)*bofac       
            hdacy(i,j)=hdacyj(j)*bofac       
          enddo
        enddo
        do j=lnsbc+1,jm-lnsbc
          do i=im-lnsbc+1,im-1
            hdacx(i,j)=hdacxj(j)*bofac       
            hdacy(i,j)=hdacyj(j)*bofac       
          enddo
        enddo
!-----------------------------------------------------------------------
        do j=2,lnsbc
          do i=2,im-2
            hdacvx(i,j)=hdacvxj(j)*bofac
            hdacvy(i,j)=hdacvyj(j)*bofac
          enddo
        enddo
        do j=jm-lnsbc,jm-2
          do i=2,im-2
            hdacvx(i,j)=hdacvxj(j)*bofac
            hdacvy(i,j)=hdacvyj(j)*bofac
          enddo
        enddo
        do j=lnsbc+1,jm-lnsbc-1
          do i=2,lnsbc
            hdacvx(i,j)=hdacvxj(j)*bofac
            hdacvy(i,j)=hdacvyj(j)*bofac
          enddo
        enddo
        do j=lnsbc+1,jm-lnsbc-1
          do i=im-lnsbc,im-2
            hdacvx(i,j)=hdacvxj(j)*bofac
            hdacvy(i,j)=hdacvyj(j)*bofac
          enddo
        enddo
!-------------coriolis parameter in tll system--------------------------
        tph=sb-dph*.5
        do j=1,jm
          tph=tph+dph
          stph=sin(tph)
          ctph=cos(tph)
          tlm=wb-dlm*0.5
          do i=1,im
            tlm=tlm+dlm
            f(i,j)=twom*(ctph0*stph+stph0*ctph*cos(tlm))
          enddo
        enddo
!--------------latitudes and longitudes of h points in radians----------     
        tph=sb-dph
        do j=1,jm
          tlm=wb-dlm
          tph=tph+dph
          stph=sin(tph)
          ctph=cos(tph)                                                                       
          do i=1,im
            tlm=tlm+dlm
            stlm=sin(tlm)
            ctlm=cos(tlm)
            sph=ctph0*stph+stph0*ctph*ctlm
            aph=asin(sph)
            glat(i,j)=aph
            anum=ctph*stlm
            denom=(ctlm*ctph-stph0*sph)/ctph0
            relm=atan2(anum,denom)
            alm=relm+tlm0d*dtr
            if(alm.gt. pi) alm=alm-pi-pi
            if(alm.lt.-pi) alm=alm+pi+pi
            glon(i,j)=alm
          enddo
        enddo
!-----------------------------------------------------------------------
      endif ! end of global/regional branch
!-------------look-up tables--------------------------------------------
      call exptbl
!-----------------------------------------------------------------------
      call tablep
      call tablet
!-----------------------------------------------------------------------
      call psitbl
!-----------------------------------------------------------------------
                        endsubroutine consts       
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine init
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,k &                         ! index
,kcfclt &                    ! counter
,l                           ! index in p direction

real(kind=kfpt):: &
 arg &                       !
,tend                        !

real(kind=kfpt),dimension(1:lm+1):: &
 psint                       ! standard interface pressure over ocean

real(kind=kfpt),dimension(1:im,1:jm):: &
 sic 
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      infile='                                                          '
!-----------------------------------------------------------------------
      nfcst=18
      infile='../output/fcst'
      open(unit=nfcst,file=infile,status='old',form='unformatted')
      read (nfcst) run,global,idat,ihrst,ihrend,ntsd
      read (nfcst) pdtop,lpt2,sgm,sg1,dsg1,sgml1,sg2,dsg2,sgml2
      read (nfcst) fis
      read (nfcst) stdh
      read (nfcst) sm
      read (nfcst) pd
      do l=1,lm
        read (nfcst) u (:,:,l)
      enddo
      do l=1,lm
        read (nfcst) v (:,:,l)
      enddo
      do l=1,lm
        read (nfcst) t (:,:,l)
      enddo
      do l=1,lm
        read (nfcst) q (:,:,l)
      enddo
      do l=1,lm
        read (nfcst) cw(:,:,l)
      enddo
!-----------------------------------------------------------------------
      ihrend=240 !zjtest
      ntsti=ntsd+1
!
      tend=real(ihrend)
      ntstm=tend*3600./dt+1.5
!
      ihr=ntsd*dt/3600.+0.5
!-----------------------------------------------------------------------
      do l=1,lm
        pdsg1(l)=dsg1(l)*pdtop
        psgml1(l)=sgml1(l)*pdtop+pt
      enddo
      do l=1,lm+1
        psg1(l)=sg1(l)*pdtop+pt
      enddo
!-----------------------------------------------------------------------
      do j=1,jm
        do i=1,im
          pdo(i,j)=pd(i,j)
        enddo
      enddo
      do l=1,lm
        do j=1,jm
          do i=1,im
            up(i,j,l)=u(i,j,l)
            vp(i,j,l)=v(i,j,l)
            tp(i,j,l)=t(i,j,l)
!
            tct(i,j,l)=0.
            tcu(i,j,l)=0.
            tcv(i,j,l)=0.
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do l=1,lm
        do j=1,jm
          do i=1,im
            cwp(i,j,l)=0.
            qp(i,j,l)=0.
            q2 (i,j,l)=0.02
            e2p(i,j,l)=0.02
            rrw (i,j,l)=0.
            rrwp(i,j,l)=0.
            dwdt(i,j,l)=1.
            w(i,j,l)=0.
          enddo
        enddo
      enddo
      do j=1,jm
        do i=1,im
          pint(i,j,1)=pt
        enddo
      enddo
      do l=1,lm
        do j=1,jm
          do i=1,im
            pint(i,j,l+1)=pint(i,j,l)+dsg2(l)*pd(i,j)+pdsg1(l)
          enddo
        enddo
      enddo
      do l=1,lm
        do j=jds,jde
          do i=ids,ide
            sp(i,j,l,1)=sqrt(max(q  (i,j,l),1.e-20))
            sp(i,j,l,2)=sqrt(max(cw (i,j,l),1.e-20))
            sp(i,j,l,3)=sqrt(max(rrw(i,j,l),1.e-20))
            sp(i,j,l,4)=sqrt(max(q2 (i,j,l),0.02))
          enddo
        enddo
      enddo
      do ks=kss,kse
        do l=1,lm
          do j=jds,jde
            do i=ids,ide
              tcs(i,j,l,ks)=0.
            enddo
          enddo
        enddo
      enddo
!-------------initialize accumulator arrays-----------------------------
      do j=1,jm
        do i=1,im
          prra(i,j)=0.
          prsn(i,j)=0.
          prcu(i,j)=0.
          prec(i,j)=0.
          acprec(i,j)=0.
          cuprec(i,j)=0.
          cuppt(i,j)=0.
        enddo
      enddo
!-----------------------------------------------------------------------
      do j=1,jm
        do i=1,im
          cldefi(i,j)=1.
        enddo
      enddo
!---reading surface data------------------------------------------------
      read (nfcst) albase
!-----------------------------------------------------------------------
      read (nfcst) albedo
!-----------------------------------------------------------------------
      read (nfcst) snoalb
!-----------------------------------------------------------------------
      read (nfcst) epsr
!-----------------------------------------------------------------------
      read (nfcst) sst
!-----------------------------------------------------------------------
      read (nfcst) deept
!-----------------------------------------------------------------------
      read (nfcst) insoil
!-----------------------------------------------------------------------
      read (nfcst) inveg
!-----------------------------------------------------------------------
      read (nfcst) vegfrc
!-----------------------------------------------------------------------
      read (nfcst) sno
!-----------------------------------------------------------------------
      read (nfcst) sic
!-----------------------------------------------------------------------
      read (nfcst) z0b
!-----------------------------------------------------------------------
      read (nfcst) z0
!-----------------------------------------------------------------------
      read (nfcst) wg
      do j=1,jm
        do i=1,im
         do l=1,nwetc
!if(wg(i,j,l).lt.1.) print*,i,j,l,wg(i,j,l)
            tg(i,j,l+1)=wg(i,j,l)
          enddo
!if(sst(i,j).lt.1.) print*,i,j,sst(i,j)
          tg(i,j,1)=sst(i,j)
          kmp(i,j)=nosnoc                                                
        enddo
      enddo
!-----------------------------------------------------------------------
      read (nfcst) wg
      close(nfcst)
!---cross-check/reinitialize surface data-------------------------------     
      do j=1,jm
        do i=1,im
!-----------------------------------------------------------------------
          if(sic(i,j).gt.0.) then
            sice(i,j)=1.
          else
            sice(i,j)=0.
          endif
!-----------------------------------------------------------------------                                                       
          if(sm(i,j).gt.0.5) then
            albase(i,j)=0.06 
            if(sice(i,j).gt.0.) then
              sm(i,j)=0.
              sice(i,j)=1.
              albase(i,j)=0.60                                                    
            endif                                                    
          else
            if(sice(i,j).gt.0.) then
              vegfrc(i,j)=0.
              insoil(i,j)=9
              inveg(i,j)=11
              albase(i,j)=0.60
              do k=1,nwetc
                wg(i,j,k)=0.
              enddo
            endif
          endif
!-----------------------------------------------------------------------                                                       
        enddo
      enddo
!---initialization of fixed land surface variables----------------------
 1010 format(' i,j=',i3,',',i3,' insoil=',i3,' inveg=',i3,' sst=',f6.2)
      kcfclt=0
      do j=1,jm
        do i=1,im
!-----------------------------------------------------------------------
          wliq(i,j)=0.
          roff(i,j)=0.
!          epsr(i,j)=1.
!-----------------------------------------------------------------------
          if(sice(i,j)+sm(i,j).lt.0.5) then ! land
!-------------standard mix----------------------------------------------
            if(insoil(i,j).gt.21.or.insoil(i,j).lt.1) then
              print*,'*** Surface data conflict'
              write(*,1010) i,j,insoil(i,j),inveg(i,j),sst(i,j)
              vegfrc(i,j)=.50
              insoil(i,j)=20
              inveg(i,j)=7
              albase(i,j)=.20
              kcfclt=kcfclt+1
            elseif(insoil(i,j).eq.16) then
              sno(i,j)=0.
            endif
!-----------------------------------------------------------------------
          elseif(sice(i,j).gt.0.5) then ! sea-ice
!-----------------------------------------------------------------------
            sno(i,j)=0.
            tg(i,j,kmsc)=ti0
            epsr(i,j)=1.
            vegfrc(i,j)=0.
            insoil(i,j)=16
            inveg(i,j)=19
!-----------------------------------------------------------------------
          elseif(sm(i,j).gt.0.5) then ! liquid surface
!-----------------------------------------------------------------------
            sno(i,j)=0.
            epsr(i,j)=1.
!
            vegfrc(i,j)=0.
            insoil(i,j)=0
            inveg(i,j)=0
            albase(i,j)=0.06
!-----------------------------------------------------------------------
          endif
!-----------------------------------------------------------------------
        enddo
      enddo
      print*,'*** total conflict pts ',kcfclt 
!-----------------------------------------------------------------------
      do j=1,jm
        do i=1,im
!---initialize sub-surface temperature and moisture---------------------
          if(sm(i,j).lt.0.5) then ! land and sea-ice surface
!-----------------------------------------------------------------------
            if(insoil(i,j).eq.10) then
              do k=1,nwetc
                wg(i,j,k)=0.0
              enddo
            endif
!-----------------------------------------------------------------------
            else ! liquid surface
!-----------------------------------------------------------------------
            do k=1,kmsc
              tg(i,j,k)=sst(i,j)
            enddo
            do k=1,nwetc
              wg(i,j,k)=0.
            enddo
!-----------------------------------------------------------------------
          endif
!-----------------------------------------------------------------------      
        enddo
      enddo ! end of land surface initialization
      do j=1,jm
        do i=1,im
          lpbl(i,j)=lm
          lbot(i,j)=1
          ltop(i,j)=lm
!
          if(sm(i,j).lt.0.5) then
            ths(i,j)=t(i,j,lm) &
                    *(100000./(sgml2(lm)*pd(i,j)+psgml1(lm)))**cappa
          else
            ths(i,j)=sst(i,j)*(100000.0/(pd(i,j)+pt))**cappa
          endif
          thz0(i,j)=ths(i,j)
          th02(i,j)=ths(i,j)
          ustar(i,j)=0.07
          qs(i,j)=0.
          qz0(i,j)=qs(i,j)
          uz0(i,j)=0.
          vz0(i,j)=0.
          u10(i,j)=0.
          v10(i,j)=0.
          akhs(i,j)=0.0001
          akms(i,j)=0.0001
          ghct(i,j)=0.
        enddo
      enddo
!--compute pressure dependent floor values of epsq2 and epsl-----------         
      ampl=0.09
      equi=0.5
      do l=1,lm+1
        psint(l)=sg2(l)*101325.+psg1(l)
      enddo
!                                                                               
      do l=2,lm
        arg=(psint(l)-psint(2))/(psint(lm)-psint(2))*pi
        epsq2(l-1)=(1.+cos(arg))*ampl+0.02
        epsl (l-1)=sqrt(epsq2(l-1)*equi)
      enddo
      epsq2(lm)=epsq2(lm-1)

!write(0,*) 'epsl=',epsl
!write(0,*) 'epsq2=',epsq2
!-----------------------------------------------------------------------
                        endsubroutine init
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine output
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!--local variables------------------------------------------------------
character(60):: &
 cfname &                    ! control file name
,dfname                      ! model data file name
 
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! oindex in y direction
,k &                         !
,l &                         ! vertical index, top down
,lrec &                      ! record length
,nrec                        ! record pointer

real(kind=kfpt),dimension(1:im,1:jm):: &
 buff                        !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      cfname='                                                          '
      write(cfname,'(a,i5.5)')"../output/cmout.",ihr
      open(unit=1,file=cfname,status='unknown' &
          ,access='sequential',form='unformatted')
      dfname='                                                          '
      write(dfname,'(a,i5.5)')"../output/dmout.",ihr
      nrec=0
      lrec=im*jm*4 ! in bytes
      open(unit=2,file=dfname,status='unknown',form='unformatted' &
          ,access='direct',action='write',recl=lrec)
!
      do i=1,im
        th02(i,1 )=th02(i,2   )
        th02(i,jm)=th02(i,jm-1)
      enddo
      do j=1,jm
        th02(1 ,j)=th02(2   ,j)
        th02(im,j)=th02(im-1,j)
      enddo
      th02(1 ,1 )=th02(2   ,2   )
      th02(im,1 )=th02(im-1,2   )
      th02(1 ,jm)=th02(2   ,jm-1)
      th02(im,jm)=th02(im-1,jm-1)
!      
      write(1) run,idat,ihrst,ihr
      write(1) pt,pdtop,sgm,sg1,sg2,dsg1,dsg2,sgml1,sgml2
      close(1)
!
      nrec=nrec+1
      write(2,rec=nrec) fis
      nrec=nrec+1
      write(2,rec=nrec) pd
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) t(:,:,l)
      enddo
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) q(:,:,l)
      enddo
      do l=1,lm+1
        nrec=nrec+1
        write(2,rec=nrec) pint(:,:,l)
      enddo
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) cw(:,:,l)
      enddo
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) q2(:,:,l)
      enddo
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) u(:,:,l)
      enddo
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) v(:,:,l)
      enddo
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) w(:,:,l)
      enddo
      do l=1,lm
        nrec=nrec+1
        write(2,rec=nrec) dwdt(:,:,l)
      enddo
      nrec=nrec+1
      write(2,rec=nrec) ths
      nrec=nrec+1
      write(2,rec=nrec) qs
      nrec=nrec+1
      write(2,rec=nrec) sno
!print*,'MIN MAX ',minval(sno(:,:)),maxval(sno(:,:))
      nrec=nrec+1
      write(2,rec=nrec) roff
      nrec=nrec+1
      write(2,rec=nrec) ustar
      nrec=nrec+1
      write(2,rec=nrec) akhs
      nrec=nrec+1
      write(2,rec=nrec) akms
      nrec=nrec+1
      write(2,rec=nrec) acprec
      nrec=nrec+1
      write(2,rec=nrec) cuprec
      nrec=nrec+1
      write(2,rec=nrec) u10
      nrec=nrec+1
      write(2,rec=nrec) v10
      nrec=nrec+1
      write(2,rec=nrec) th10
      nrec=nrec+1
      write(2,rec=nrec) q10
      nrec=nrec+1
      write(2,rec=nrec) th02
      nrec=nrec+1
      write(2,rec=nrec) q02
      nrec=nrec+1
      write(2,rec=nrec) radin
      nrec=nrec+1
      write(2,rec=nrec) rswin
      nrec=nrec+1
      write(2,rec=nrec) thz0
      nrec=nrec+1
      write(2,rec=nrec) qz0
      nrec=nrec+1
      write(2,rec=nrec) z0b
      nrec=nrec+1
      write(2,rec=nrec) z0
      nrec=nrec+1
      write(2,rec=nrec) sm
      nrec=nrec+1
      write(2,rec=nrec) sice
      nrec=nrec+1
      write(2,rec=nrec) sst
      nrec=nrec+1
      write(2,rec=nrec) insoil
      nrec=nrec+1
      write(2,rec=nrec) inveg
      nrec=nrec+1
      write(2,rec=nrec) vegfrc
      nrec=nrec+1
      write(2,rec=nrec) albase
      nrec=nrec+1
      write(2,rec=nrec) albedo
      nrec=nrec+1
      write(2,rec=nrec) snoalb
      nrec=nrec+1
      write(2,rec=nrec) epsr
      do l=1,nwetc
        do j=1,jm
          do i=1,im
            k=kmp(i,j)-nwetc
            buff(i,j)=tg(i,j,k)
          enddo
        enddo
        nrec=nrec+1
        write(2,rec=nrec) buff
!        write(0,*)'MIN MAX ',minval(buff(:,:)),maxval(buff(:,:)),l
      enddo
      do l=1,nwetc
        nrec=nrec+1
        write(2,rec=nrec) wg(:,:,l)
!        write(0,*)'MIN MAX ',minval(wg(:,:,l)),maxval(wg(:,:,l)),l
      enddo
      close(2)
!-----------------------------------------------------------------------
      if(mod(ntsd,nprec).eq.0) then
        do j=1,jm
          do i=1,im
            acprec(i,j)=0.
            cuprec(i,j)=0.
            roff(i,j)=0.
          enddo
        enddo
      endif
!-----------------------------------------------------------------------
                        endsubroutine output
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine exptbl
!     ******************************************************************
!     *                                                                *
!     *               exponential function table                       *
!     *               responsible person: z.janjic                     *
!     *                                                                *
!     ******************************************************************
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!--local variables------------------------------------------------------
integer(kind=kint):: &
 k                           ! index

real(kind=kfpt):: &
 x &                         ! argument
,xrng                        ! argument range
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      xmax= 30.
      xmin=-30.
!
      kexm2=kexm-2
      xrng=xmax-xmin
!
      dex=xrng/(kexm-1)
      rdex=1./dex
!--------------function definition loop---------------------------------
      x=xmin-dex
      do k=1,kexm
        x=x+dex
        expf(k)=exp(x)
      enddo
!-----------------------------------------------------------------------
                        endsubroutine exptbl
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        function zjexp(x)
!     ******************************************************************
!     *                                                                *
!     *               exponential function table                       *
!     *               responsible person: z.janjic                     *
!     *                                                                *
!     ******************************************************************
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!-----------------------------------------------------------------------
real(kind=kfpt):: &
 zjexp

!--local variables------------------------------------------------------
integer(kind=kint):: &
 k                           ! index

real(kind=kfpt):: &
 ak &                        ! position in table
,x                           ! argument
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      ak=(x-xmin)*rdex
      k=int(ak)
      k=max(k,0)
      k=min(k,kexm2)
!
      zjexp=(expf(k+2)-expf(k+1))*(ak-real(k))+expf(k+1)
!-----------------------------------------------------------------------
                        endfunction zjexp
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine tablep
!     ******************************************************************
!     *                                                                *
!     *    generates the table for finding pressure from               *
!     *    saturation specific humidity and potential temperature      *
!     *                                                                *
!     ******************************************************************
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 eps=1.e-10
!--local variables------------------------------------------------------
integer(kind=kint):: &
 kth &                       ! index
,kp                          ! index

real(kind=kfpt):: &
 ape &                       ! exner function
,dth &                       ! potential temperature step
,dp &                        ! pressure step
,dqs &                       ! saturation specific humidity step
,expon &                     !
,p &                         ! pressure
,qs0k &                      ! base value for saturation humidity
,sqsk &                      ! saturation spec. humidity range
,th                          ! potential temperature

real(kind=kfpt),dimension(1:itb):: &
 app &                       ! temporary
,aqp &                       ! temporary
,pnew &                      ! new pressures
,pold &                      ! old pressure
,qsnew &                     ! new saturation spec. humidity
,qsold &                     ! old saturation spec. humidity
,y2p                         ! temporary
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      dth=(thh-thl)/real(jtb-1)
      dp=(ph-pl)/real(itb-1)
      rdth=1./dth
!-----------------------------------------------------------------------
      th=thl-dth
      do kth=1,jtb
        th=th+dth
        p=pl-dp
        do kp=1,itb

          p=p+dp
          ape=(100000./p)**cappa
          expon=a2*(th-a3*ape)/(th-a4*ape)
!
          qsold(kp)=pq0/p*exp(expon)
          pold(kp)=p
        enddo
!
        qs0k=qsold(1)
        sqsk=qsold(itb)-qsold(1)
        qsold(1)=0.
        qsold(itb)=1.
!
        do kp=2,itb-1
          qsold(kp)=(qsold(kp)-qs0k)/sqsk
!wwwwwwwwwwwwww fix due to 32 bit precision limitation wwwwwwwwwwwwwwwww
          if((qsold(kp)-qsold(kp-1)).lt.eps) then
            qsold(kp)=qsold(kp-1)+eps
          endif
!mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
        enddo
!
        qs0(kth)=qs0k
        sqs(kth)=sqsk
!
        qsnew(1)=0.
        qsnew(itb)=1.
        dqs=1./real(itb-1)
        rdq=1./dqs
!
        do kp=2,itb-1
          qsnew(kp)=qsnew(kp-1)+dqs
        enddo
!
        y2p(1)=0.
        y2p(itb)=0.
!
        call spline(jtb,itb,qsold,pold,y2p,itb,qsnew,pnew,app,aqp)
!
        do kp=1,itb
          ptbl(kp,kth)=max(pnew(kp),1.)
        enddo
!-----------------------------------------------------------------------
      enddo
!-----------------------------------------------------------------------
                        endsubroutine tablep
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine tablet
!     ******************************************************************
!     *                                                                *
!     *    generates the table for finding temperature from            *
!     *    pressure and equivalent potential temperature               *
!     *                                                                *
!     ******************************************************************
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 eps=1.e-10
!--local variables------------------------------------------------------
integer(kind=kint):: &
 kth &                       ! index
,kp                          ! index

real(kind=kfpt):: &
 ape &                       ! exner function
,dth &                       ! potential temperature step
,dp &                        ! pressure step
,dthe &                      ! equivalent pot. temperature step
,expon &                     !
,p &                         ! pressure
,qs &                        ! saturation specific humidity
,the0k &                     ! base value for equivalent pot. temperature
,sthek &                     ! equivalent pot. temperature range
,th                          ! potential temperature

real(kind=kfpt),dimension(1:jtb):: &
 apt &                       ! temporary
,aqt &                       ! temporary
,tnew &                      ! new temperature
,told &                      ! old temperature
,thenew &                    ! new equivalent potential temperature
,theold &                    ! old equivalent potential temperature
,y2t                         ! temporary
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      dth=(thh-thl)/real(jtb-1)
      dp=(ph-pl)/real(itb-1)
      rdp=1./dp
!-----------------------------------------------------------------------
      p=pl-dp
      do kp=1,itb
        p=p+dp
        th=thl-dth
        do kth=1,jtb
          th=th+dth
          ape=(100000./p)**cappa
          expon=a2*(th-a3*ape)/(th-a4*ape)
!
          if(expon.gt.-30.) then
            qs=pq0/p*exp(expon)
          else
            qs=0.
          endif
!
          told(kth)=th/ape
          theold(kth)=th*exp(eliwv*qs/(cp*told(kth)))
        enddo
!
        the0k=theold(1)
        sthek=theold(jtb)-theold(1)
        theold(1)=0.
        theold(jtb)=1.
!
        do kth=2,jtb-1
          theold(kth)=(theold(kth)-the0k)/sthek
!wwwwwwwwwwwwww fix due to 32 bit precision limitation wwwwwwwwwwwwwwwww
          if((theold(kth)-theold(kth-1)).lt.eps) then
            theold(kth)=theold(kth-1)+eps
          endif
!mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
        enddo
!
        the0(kp)=the0k
        sthe(kp)=sthek
!
        thenew(1)=0.
        thenew(jtb)=1.
        dthe=1./real(jtb-1)
        rdthe=1./dthe
!
        do kth=2,jtb-1
          thenew(kth)=thenew(kth-1)+dthe
        enddo
!
        y2t(1)=0.
        y2t(jtb)=0.
!
        call spline(jtb,jtb,theold,told,y2t,jtb,thenew,tnew,apt,aqt)
!
        do kth=1,jtb
          ttbl(kth,kp)=tnew(kth)
        enddo
!-----------------------------------------------------------------------
      enddo
!-----------------------------------------------------------------------
                        endsubroutine tablet
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine spline(jtb,nold,xold,yold,y2,nnew,xnew,ynew,p,q)           
!     ******************************************************************
!     *                                                                *
!     *  this is a one-dimensional cubic spline fitting routine        *
!     *  programed for a small scalar machine.                         *
!     *                                                                *
!     *  programer: z. janjic, yugoslav fed. hydromet. inst., beograd  *
!     *                                                                *
!     *                                                                *
!     *                                                                *
!     *  nold - number of given values of the function.  must be ge 3. *
!     *  xold - locations of the points at which the values of the     *
!     *         function are given.  must be in ascending order.       *
!     *  yold - the given values of the function at the points xold.   *
!     *  y2   - the second derivatives at the points xold.  if natural *
!     *         spline is fitted y2(1)=0. and y2(nold)=0. must be      *
!     *         specified.                                             *
!     *  nnew - number of values of the function to be calculated.     *
!     *  xnew - locations of the points at which the values of the     *
!     *         function are calculated.  xnew(k) must be ge xold(1)   *
!     *         and le xold(nold).                                     *
!     *  ynew - the values of the function to be calculated.           *
!     *  p, q - auxiliary vectors of the length nold-2.                *
!     *                                                                *
!     ******************************************************************
      implicit real(a-h,o-z),integer(i-n)
!-----------------------------------------------------------------------
      dimension &                      
       xold(jtb),yold(jtb),y2(jtb),p(jtb),q(jtb) &                        
      ,xnew(jtb),ynew(jtb)                                              
!-----------------------------------------------------------------------
      noldm1=nold-1                                                     
!                                                                       
      dxl=xold(2)-xold(1)                                               
      dxr=xold(3)-xold(2)                                               
      dydxl=(yold(2)-yold(1))/dxl                                       
      dydxr=(yold(3)-yold(2))/dxr                                       
      rtdxc=.5/(dxl+dxr)                                                
!                                                                       
      p(1)= rtdxc*(6.*(dydxr-dydxl)-dxl*y2(1))                          
      q(1)=-rtdxc*dxr                                                   
!                                                                       
      if(nold.eq.3) go to 700                                           
!-----------------------------------------------------------------------
      k=3                                                               
!                                                                       
 100  dxl=dxr                                                           
      dydxl=dydxr                                                       
      dxr=xold(k+1)-xold(k)                                             
      dydxr=(yold(k+1)-yold(k))/dxr                                     
      dxc=dxl+dxr                                                       
      den=1./(dxl*q(k-2)+dxc+dxc)                                       
!                                                                       
      p(k-1)= den*(6.*(dydxr-dydxl)-dxl*p(k-2))                         
      q(k-1)=-den*dxr                                                   
!                                                                       
      k=k+1                                                             
      if(k.lt.nold) go to 100                                           
!-----------------------------------------------------------------------
 700  k=noldm1                                                          
!                                                                       
 200  y2(k)=p(k-1)+q(k-1)*y2(k+1)                                       
!                                                                       
      k=k-1                                                             
      if(k.gt.1) go to 200                                              
!-----------------------------------------------------------------------
      k1=1                                                              
!                                                                       
 300  xk=xnew(k1)                                                       
!                                                                       
      do 400 k2=2,nold                                                  
      if(xold(k2).le.xk) go to 400                                      
      kold=k2-1                                                         
      go to 450                                                         
 400  continue                                                          
      ynew(k1)=yold(nold)                                               
      go to 600                                                         
!                                                                       
 450  if(k1.eq.1)   go to 500                                           
      if(k.eq.kold) go to 550                                           
!                                                                       
 500  k=kold                                                            
!                                                                       
      y2k=y2(k)                                                         
      y2kp1=y2(k+1)                                                     
      dx=xold(k+1)-xold(k)                                              
      rdx=1./dx                                                         
!                                                                       
      ak=.1666667*rdx*(y2kp1-y2k)                                       
      bk=.5*y2k                                                         
      ck=rdx*(yold(k+1)-yold(k))-.1666667*dx*(y2kp1+y2k+y2k)            
!                                                                       
 550  x=xk-xold(k)                                                      
      xsq=x*x                                                           
!                                                                       
      ynew(k1)=ak*xsq*x+bk*xsq+ck*x+yold(k)                             
!                                                                       
 600  k1=k1+1                                                           
      if(k1.le.nnew) go to 300                                          
!-----------------------------------------------------------------------
                        endsubroutine spline    
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine psitbl
!     ******************************************************************
!     *                                                                *
!     *               surface layer integral functions                 *
!     *               responsible person: z.janjic                     *
!     *                                                                *
!     ******************************************************************
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 eps=0.000001
!--local variables------------------------------------------------------
integer(kind=kint):: &
 k                           ! index
 
real(kind=kfpt):: &
 x &                         ! temporary
,zeta1 &                     ! z/L, sea
,zeta2 &                     ! z/L, land
,zrng1 &                     ! z/L range, sea
,zrng2                       ! z/L range, land
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      kztm2=kztm-2
!
      fh01=1.
      fh02=1.
!
      ztmin1=-5.0
      ztmax1= 1.0
!
      ztmin2=-5.0
      ztmax2= 1.0
!
      zrng1=ztmax1-ztmin1
      zrng2=ztmax2-ztmin2
!
      dzeta1=zrng1/(kztm-1)
      dzeta2=zrng2/(kztm-1)
!--------------function definition loop---------------------------------
      zeta1=ztmin1
      zeta2=ztmin2
      do k=1,kztm
!--------------unstable range-------------------------------------------
        if(zeta1.lt.0.)then
!--------------paulson 1970 functions-----------------------------------
          x=sqrt(sqrt(1.-16.*zeta1))
          psim1(k)=-2.*log((x+1.)/2.)-log((x*x+1.)/2.)+2.*atan(x)-pihf
          psih1(k)=-2.*log((x*x+1.)/2.)
!--------------stable range---------------------------------------------
        else
!--------------holtslag and de bruin 1988-------------------------------
          psim1(k)=0.7*zeta1+0.75*zeta1*(6.-0.35*zeta1)*exp(-0.35*zeta1)
          psih1(k)=0.7*zeta1+0.75*zeta1*(6.-0.35*zeta1)*exp(-0.35*zeta1)
!-----------------------------------------------------------------------
        endif
!--------------unstable range-------------------------------------------
        if(zeta2.lt.0.)then
!--------------paulson 1970 functions-----------------------------------
          x=sqrt(sqrt(1.-16.*zeta2))
          psim2(k)=-2.*log((x+1.)/2.)-log((x*x+1.)/2.)+2.*atan(x)-pihf
          psih2(k)=-2.*log((x*x+1.)/2.)
!--------------stable range---------------------------------------------
        else
!--------------holtslag and de bruin 1988-------------------------------
          psim2(k)=0.7*zeta2+0.75*zeta2*(6.-0.35*zeta2)*exp(-0.35*zeta2)
          psih2(k)=0.7*zeta2+0.75*zeta2*(6.-0.35*zeta2)*exp(-0.35*zeta2)
!-----------------------------------------------------------------------
        endif
!-----------------------------------------------------------------------
        if(k.eq.kztm)then
          ztmax1=zeta1
          ztmax2=zeta2
        endif
!
        zeta1=zeta1+dzeta1
        zeta2=zeta2+dzeta2
!-----------------------------------------------------------------------
      enddo
!-----------------------------------------------------------------------
      ztmax1=ztmax1-eps
      ztmax2=ztmax2-eps
!-----------------------------------------------------------------------
                        endsubroutine psitbl
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        endmodule module_control
