!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        module module_dynamics
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
use module_fltbnds,only:swapw2,poled2,swaphn,polehn
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

include 'kind.inc'
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        contains
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine pgforce &
(first &
,ids,ide,jds,jde,jm,lm &
,dt,r,rdyv &
,dsg2,pdsg1 &
,rdxv,wpdar &
,fis,pd,pdo &
,t,q,cw,dwdt &
,pint &
,rtop &
!---temporary arguments-------------------------------------------------
,div,pcne,pcnw,pcx,pcy,tcu,tcv)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!include 'kind.inc'
!-----------------------------------------------------------------------
integer(kind=kint),parameter:: &
 jw=01                       ! rows w/o correction next to poles
!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 first

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,jm &                        ! total # of rows
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 dt &                        ! dynamics time step
,r &                         ! gas constant for dry air
,rdyv                        ! 1/deltay

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 rdxv &                      ! 1/deltax
,wpdar                       ! divergence correction weight

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 fis &                       ! surface geopotential
,pd &                        ! sigma range pressure difference
,pdo                         ! old sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 t &                         ! temperature
,q &                         ! specific humidity
,cw &                        ! condensate
,dwdt                        ! nonhydrostatic correction factor

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm+1),intent(in):: &
 pint                        ! pressure at interfaces

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(out):: &
 rtop                        ! RT/p

!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 div &                       ! horizontal mass divergence
,pcne &                      ! second term of pgf, ne direction
,pcnw &                      ! second term of pgf, nw direction
,pcx &                       ! second term of pgf, x direction
,pcy &                       ! second term of pgf, y direction
,tcu &                       ! time change of u
,tcv                         ! time change of v
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,jcl &                       ! lower bound for no divergence correction
,jch &                       ! upper bound for no divergence correction
,l                           ! index in p direction

real(kind=kfpt):: &
 apd &                       ! hydrostatic pressure difference at the point
,apelp &                     ! pressure at the point
,dfdp &                      ! dfi/dp
,dfip &                      !
,dpdp &                      !
,fiup &                      ! geopotential at the upper interface
,pdnep &                     ! hydrostatic pressure difference at the point
,pdnwp &                     ! hydrostatic pressure difference at the point
,pdxp &                      ! hydrostatic pressure difference at the point
,pdyp &                      ! hydrostatic pressure difference at the point
,ppne &                      ! first term of pgf, ne direction
,ppnw &                      ! first term of pgf, nw direction
,ppx &                       ! first term of pgf, x direction
,ppy &                       ! first term of pgf, y direction
,rdu &                       !
,rdv &                       !
,rpdp &                      !
,rt &                        !
,rtpp &                      !
,wprp                        ! divergence modification weight at the point

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 apel &                      ! scratch, pressure in the middle of the layer
,dfi &                       ! geopotential increment
,dpel &                      ! nh pressure increment
,filo &                      ! scratch, geopotential at lower interface
,fim &                       ! scratch, geopotential in the middle of layer
,pdne &                      ! hydrostatic pressure difference at the point
,pdnw &                      ! hydrostatic pressure difference at the point
,pdx &                       ! hydrostatic pressure difference at the point
,pdy &                       ! hydrostatic pressure difference at the point
,pgne &                      ! scratch, pgf, ne direction
,pgnw &                      ! scratch, pgf, nw direction
,pgx &                       ! scratch, pgf, x direction
,pgy                         ! scratch, pgf, y direction
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!do l=1,lm
!do j=1,jm
!  print*,j,l,u(ide-2,j,l),u(ide-1,j,l),u(ide,j,l),u(1,j,l),u(2,j,l)
!  print*,j,l,v(ide-2,j,l),v(ide-1,j,l),v(ide,j,l),v(1,j,l),v(2,j,l)
!enddo
!enddo
!-----------------------------------------------------------------------
      do j=jds,jde
        do i=ids,ide
          filo(i,j)=fis(i,j)
        enddo
      enddo          
!-----------------------------------------------------------------------
      do j=jds,jde
        do i=ids+1,ide
          pdx (i,j)=(pd(i-1,j)+pd(i,j)+pdo(i-1,j)+pdo(i,j))*0.25
        enddo
      enddo
      do j=jds+1,jde
        do i=ids,ide
          pdy (i,j)=(pd(i,j-1)+pd(i,j)+pdo(i,j-1)+pdo(i,j))*0.25
        enddo
      enddo
      do j=jds+1,jde
        do i=ids+1,ide
          pdne(i,j)=(pd(i-1,j-1)+pd(i,j)+pdo(i-1,j-1)+pdo(i,j))*0.25
          pdnw(i,j)=(pd(i,j-1)+pd(i-1,j)+pdo(i,j-1)+pdo(i-1,j))*0.25
        enddo
      enddo
!---vertical grand loop-------------------------------------------------
      do l=lm,1,-1
!-----------------------------------------------------------------------
        do j=jds,jde
          do i=ids,ide
            apel(i,j)=(pint(i,j,l+1)+pint(i,j,l))*0.5
            dpel(i,j)=(pint(i,j,l+1)-pint(i,j,l))*0.5
!
            rt=(q(i,j,l)*0.608+(1.-cw(i,j,l)))*t(i,j,l)*r
            dpdp=dsg2(l)*pd(i,j)+pdsg1(l)
            rtpp=rt/apel(i,j)
            dfip=rtpp*dpdp
            rtop(i,j,l)=rtpp
            dfi(i,j)=dfip*0.5
            fiup=filo(i,j)+dfip
            fim(i,j)=(filo(i,j)+fiup)*0.5
!
            filo(i,j)=fiup
          enddo
        enddo
!---pressure gradient force components, behind h points-----------------
        do j=jds,jde
          do i=ids+1,ide
            ppx=(fim(i,j)-fim(i-1,j)) &
               *(dpel(i-1,j  )+dpel(i  ,j  ))
            pcx(i,j,l)=(dfi (i-1,j  )+dfi (i  ,j  )) &
                      *(apel(i  ,j  )-apel(i-1,j  ))
            pgx(i,j)=ppx+pcx(i,j,l)
          enddo
        enddo
        do j=jds+1,jde
          do i=ids,ide
            ppy=(fim(i,j)-fim(i,j-1)) &
               *(dpel(i  ,j-1)+dpel(i  ,j  ))
            pcy(i,j,l)=(dfi (i  ,j-1)+dfi (i  ,j  )) &
                      *(apel(i  ,j  )-apel(i  ,j-1))
            pgy(i,j)=ppy+pcy(i,j,l)
          enddo
        enddo
        do j=jds+1,jde
          do i=ids+1,ide
            ppne=(fim (i  ,j  )-fim (i-1,j-1)) &
                *(dpel(i-1,j-1)+dpel(i  ,j  ))
            ppnw=(fim (i-1,j  )-fim (i  ,j-1)) &
                *(dpel(i  ,j-1)+dpel(i-1,j  ))
            pcne(i,j,l)=(dfi (i-1,j-1)+dfi (i  ,j  )) &
                       *(apel(i  ,j  )-apel(i-1,j-1))
            pcnw(i,j,l)=(dfi (i  ,j-1)+dfi (i-1,j  )) &
                       *(apel(i-1,j  )-apel(i  ,j-1))
            pgne(i,j)=ppne+pcne(i,j,l)
            pgnw(i,j)=ppnw+pcnw(i,j,l)
          enddo
        enddo
!---divergence correction, janjic 1974 mwr, 1979 beitrage---------------
        jcl=1
        jch=jw
        do j=jcl,jch
          do i=ids,ide
            div(i,j,l)=0.
          enddo
        enddo
!
        jcl=jm-jw+1
        jch=jm
        do j=jcl,jch
          do i=ids,ide
            div(i,j,l)=0.
          enddo
        enddo
!
        wprp=wpdar(jw+1)
        do i=ids+1,ide-1
          div(i,jw+1,l)=( ( pgx(i+1,jw+1)-pgx(i,jw+1) &
                           +pgy(i  ,jw+2)) &
                         -( pgne(i+1,jw+2) &
                           +pgnw(i  ,jw+2))*0.5)*wprp
        enddo
!
        wprp=wpdar(jm-jw)
        do i=ids+1,ide-1
          div(i,jm-jw,l)=( ( pgx(i+1,jm-jw)-pgx(i,jm-jw) &
                            -pgy(i  ,jm-jw)) &
                          -(-pgne(i  ,jm-jw) &
                            -pgnw(i+1,jm-jw))*0.5)*wprp
        enddo
!
        jcl=jw+2
        jch=jm-jw-1
        do j=jcl,jch
          wprp=wpdar(j)
          do i=ids+1,ide-1
            div(i,j,l)=( ( pgx (i+1,j  )-pgx (i  ,j  ) &
                          +pgy (i  ,j+1)-pgy (i  ,j  )) &
                        -( pgne(i+1,j+1)-pgne(i  ,j  ) &
                          +pgnw(i  ,j+1)-pgnw(i+1,j  ))*0.5)*wprp
          enddo
        enddo
!---first pass switch---------------------------------------------------
        if(.not.first) then
!---updating u and v due to pgf force, end of time-step for u and v-----
          rdv=rdyv*dt
          do j=jds+1,jde-2
            rdu=rdxv(j)*dt
            do i=ids+1,ide-2
              apd=(pd(i,j)+pd(i+1,j)+pd(i,j+1)+pd(i+1,j+1))*0.25
              rpdp=0.3333333333/(dsg2(l)*apd+pdsg1(l))
!
              tcu(i,j,l)=-((pgx(i+1,j)+pgx(i+1,j+1)) &
                          +(pgne(i+1,j+1)-pgnw(i+1,j+1))*0.5)*rdu*rpdp &
                         +tcu(i,j,l)
              tcv(i,j,l)=-((pgy(i,j+1)+pgy(i+1,j+1)) &
                          +(pgne(i+1,j+1)+pgnw(i+1,j+1))*0.5)*rdv*rpdp &
                         +tcv(i,j,l)
            enddo
          enddo
!-----------------------------------------------------------------------
        else
!-----------------------------------------------------------------------
          do j=jds+1,jde-2
            do i=ids+1,ide-2
              tcu(i,j,l)=0.
              tcv(i,j,l)=0.
            enddo
          enddo
!-----------------------------------------------------------------------
        endif
!-----------------------------------------------------------------------
      enddo      ! end of vertical loop
!-----------------------------------------------------------------------
                        endsubroutine pgforce
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine dht &
(global &
,ids,ide,jds,jde,lm &
,cp,dyv &
,dsg2,pdsg1 &
,dxv,fcp,fdiv &
,pd,pdo &
,u,v,q,cw &
,omgalf &
!---temporary arguments-------------------------------------------------
,div,pcne,pcnw,pcx,pcy,pfne,pfnw,pfx,pfy)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
logical(kind=klog):: &
 global

real(kind=kfpt),parameter:: &
 cfc=1.533 &                 ! adams-bashforth positioning in time
,bfc=1.-cfc &                ! adams bashforth positioning in time
,cliq=4190. &                ! heat capacity of liquid water
,cvap=1952.                  ! heat capacity at const. pressure vapor
!-----------------------------------------------------------------------
integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 cp &                        !
,dyv                         ! deltay, v point

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 dxv &                       ! deltax, v point
,fcp &                       !
,fdiv                        !

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 pd &                        ! sigma range pressure difference
,pdo                         ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 u &                         ! u wind component
,v &                         ! v wind component
,q &                         ! specific humidity
,cw                          ! condensate

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(out):: &
 omgalf                      ! omega-alfa (horizontal)

!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 div &                       ! horizontal mass divergence
,pcne &                      ! second term of pgf, ne direction
,pcnw &                      ! second term of pgf, nw direction
,pcx &                       ! second term of pgf, x direction
,pcy &                       ! second term of pgf, y direction
,pfne &                      ! mass flux, ne direction
,pfnw &                      ! mass flux, nw direction
,pfx &                       ! mass flux, x direction
,pfy                         ! mass flux, y direction

!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 dyp1 &                      ! 
,dyp &                       !
,dxp1 &                      !
,dxp &                       !
,fcliq &                     !
,fcpp &                      !
,fcvap &                     !
,fdp &                       ! 
,pdp &                       ! hydrostatic pressure difference at the point
,pdxp &                      ! hydrostatic pressure at the point
,pdyp &                      ! hydrostatic pressure at the point
,pdnep &                     ! hydrostatic pressure at the point
,pdnwp &                     ! hydrostatic pressure at the point
,udy &                       !
,vdx &                       !
,udy1 &                      !
,vdx1                        !

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 pdne &                      ! hydrostatic pressure difference at the point
,pdnw &                      ! hydrostatic pressure difference at the point
,pdx &                       ! hydrostatic pressure difference at the point
,pdy &                       ! hydrostatic pressure difference at the point
,tne &                       ! temperature flux, ne direction
,tnw &                       ! temperature flux, nw direction
,tx &                        ! temperature flux, x direction
,ty                          ! temperature flux, y direction
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      fcliq=cliq/cp-1.           ! factor for correction of cp for water
      fcvap=cvap/cp-1.           ! factor for correction of cp for vapor
!
      dyp1=dyv
      dyp=dyp1*0.5
!
      do j=jds,jde
        do i=ids+1,ide
          pdx (i,j)=((pd (i-1,j)+pd (i,j))*cfc &
                    +(pdo(i-1,j)+pdo(i,j))*bfc)*0.5
        enddo
      enddo
      do j=jds+1,jde
        do i=ids,ide
          pdy (i,j)=((pd (i,j-1)+pd (i,j))*cfc &
                    +(pdo(i,j-1)+pdo(i,j))*bfc)*0.5
        enddo
      enddo
      do j=jds+1,jde
        do i=ids+1,ide
          pdne(i,j)=((pd (i-1,j-1)+pd (i,j))*cfc &
                    +(pdo(i-1,j-1)+pdo(i,j))*bfc)*0.5
          pdnw(i,j)=((pd (i,j-1)+pd (i-1,j))*cfc &
                    +(pdo(i,j-1)+pdo(i-1,j))*bfc)*0.5
        enddo
      enddo
!---vertical grand loop-------------------------------------------------
      dyp1=dyv
      dyp=dyp1*0.5
      do l=1,lm
!---mass and pressure fluxes, on h points-------------------------------
        pdp=pdsg1(l)
        do j=jds+1,jde
          dxp1=dxv(j-1)
          dxp=dxp1*0.5
          do i=ids+1,ide
!-----------------------------------------------------------------------
            pdxp=dsg2(l)*pdx(i,j)+pdp
            pdyp=dsg2(l)*pdy(i,j)+pdp
            pdnep=dsg2(l)*pdne(i,j)+pdp
            pdnwp=dsg2(l)*pdnw(i,j)+pdp
!
            udy=(u(i-1,j-1,l)+u(i-1,j,l))*dyp
            vdx=(v(i-1,j-1,l)+v(i,j-1,l))*dxp
!
            pfx(i,j,l)=udy*pdxp
            pfy(i,j,l)=vdx*pdyp
!
            udy1=u(i-1,j-1,l)*dyp1
            vdx1=v(i-1,j-1,l)*dxp1
!
            pfne(i,j,l)=( udy1+vdx1)*pdnep
            pfnw(i,j,l)=(-udy1+vdx1)*pdnwp
!
            tx(i,j)=pcx(i,j,l)*udy
            ty(i,j)=pcy(i,j,l)*vdx
!
            tne(i,j)=pcne(i,j,l)*( udy1+vdx1)
            tnw(i,j)=pcnw(i,j,l)*(-udy1+vdx1)
          enddo
        enddo
!---divergence and hor. pressure advection in t eq.---------------------
        do j=jds+1,jde-1
          fdp=fdiv(j)
          fcpp=fcp(j)
          do i=ids+1,ide-1
            div(i,j,l)=((pfx (i+1,j  ,l)-pfx (i  ,j  ,l) &
                        +pfy (i  ,j+1,l)-pfy (i  ,j  ,l)) &
                       +(pfne(i+1,j+1,l)-pfne(i  ,j  ,l) &
                        +pfnw(i  ,j+1,l)-pfnw(i+1,j  ,l))*0.25)*fdp  &
                      +div(i,j,l)
            omgalf(i,j,l)=((tx (i  ,j  )+tx (i+1,j  ) &
                           +ty (i  ,j  )+ty (i  ,j+1)) &
                          +(tne(i+1,j+1)+tne(i  ,j  ) &
                           +tnw(i  ,j+1)+tnw(i+1,j  ))*0.25) &
                         *fcpp/((1.+fcvap*q(i,j,l)+fcliq*cw(i,j,l)) &
                               *(dsg2(l)*pd(i,j)+pdsg1(l)))
          enddo
        enddo
!--zero divergence along regional domain boundaries---------------------
        if(.not.global) then
          do i=ids,ide
            div(i,jds,l)=0.
          enddo
!
          do i=ids,ide
            div(i,jde,l)=0.
          enddo
!
          do j=jds,jde
            div(ids,j,l)=0.
          enddo
!
          do j=jds,jde
            div(ide,j,l)=0.
          enddo
        endif
!-----------------------------------------------------------------------
      enddo      ! end of vertical loop
!-----------------------------------------------------------------------
                        endsubroutine dht
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine ddamp &
(freerun &
,ids,ide,jds,jde,lm &
,ddmpv,pdtop &
,dsg2,pdsg1 &
,sg1,sg2 &
,ddmpu &
,pd,pdo &
,u,v &
!---temporary arguments-------------------------------------------------
,div)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 cfc=1.533 &                 ! adams-bashforth positioning in time
,bfc=1.-cfc                  ! adams bashforth positioning in time
!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 freerun

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 ddmpv &                     ! divergence damping, v component
,pdtop                       ! pressure coordinate depth

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(1:lm+1),intent(in):: &
 sg1 &                       !
,sg2                         !

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 ddmpu                       ! divergence damping, u direction

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 pd &                        ! sigma range pressure difference
,pdo                         ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 u &                         ! u wind component
,v                           ! v wind component

!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 div                         ! horizontal mass divergence

!--local variables------------------------------------------------------
logical(kind=klog),parameter:: &
 extmod=.true.

real(kind=kfpt),parameter:: &
 assimfc=15.0

integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 dfac &                      !
,dpb &                       !
,dpl &                       !
,fcim &                      !
,fcxm &                      ! blow up factor for external mode damping
,fint &                      !
,rdpd &                      !
,dud &                       !
,dvd                         !

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 apd &                       !
,dive &                      !
,rddu &                      !
,rddv                        !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      dfac=9. !5.
      dpb=1500.
      fint=(1.-dfac)/dpb
!-----------------------------------------------------------------------
      do j=jds+1,jde-2
        do i=ids+1,ide-2
          apd(i,j)=((pd (i  ,j  )+pd (i+1,j  ) &
                    +pd (i  ,j+1)+pd (i+1,j+1))*cfc &
                   +(pdo(i  ,j  )+pdo(i+1,j  ) &
                    +pdo(i  ,j+1)+pdo(i+1,j+1))*bfc)*0.25
        enddo
      enddo
!---external mode-------------------------------------------------------
      if(extmod) then
        if(freerun) then
          fcxm=1.0
        else
          fcxm=assimfc
        endif
!
        do j=jds,jde
          do i=ids,ide
            dive(i,j)=div(i,j,1)
          enddo
        enddo
        do l=2,lm
          do j=jds,jde
            do i=ids,ide
              dive(i,j)=div(i,j,l)+dive(i,j)
            enddo
          enddo
        enddo      
        dvd=ddmpv 
        do j=jds+1,jde-2
          dud=ddmpu(j)
          do i=ids+1,ide-2
            rdpd=fcxm/apd(i,j)
            rddu(i,j)=(dive(i+1,j)+dive(i+1,j+1) &
                      -dive(i  ,j)-dive(i  ,j+1))*dud*rdpd
            rddv(i,j)=(dive(i,j+1)+dive(i+1,j+1) &
                      -dive(i,j  )-dive(i+1,j  ))*dvd*rdpd         
          enddo
        enddo
!-----------------------------------------------------------------------
        do l=1,lm
!
          if(freerun) then
            dpl=sg1(l+1)*pdtop+sg2(l+1)*101300.
            if(dpl.lt.dpb) then
              fcim=fint*dpl+dfac
            else
              fcim=1.
            endif
          else
            fcim=assimfc
          endif
!
          dvd=ddmpv 
          do j=jds+1,jde-2
            dud=ddmpu(j)
            do i=ids+1,ide-2
              rdpd=fcim/(dsg2(l)*apd(i,j)+pdsg1(l))
              u(i,j,l)=((div(i+1,j,l)+div(i+1,j+1,l) &
                        -div(i  ,j,l)-div(i  ,j+1,l))*dud)*rdpd &
                      +rddu(i,j)+u(i,j,l)
              v(i,j,l)=((div(i,j+1,l)+div(i+1,j+1,l) &
                        -div(i,j  ,l)-div(i+1,j  ,l))*dvd)*rdpd &
                      +rddv(i,j)+v(i,j,l)                 
            enddo
          enddo
        enddo      ! end of vertical loop
!-----------------------------------------------------------------------
      else
!---divergence damping--------------------------------------------------
        do l=1,lm
!
          if(freerun) then
            dpl=sg1(l+1)*pdtop+sg2(l+1)*101300.
            if(dpl.lt.dpb) then
              fcim=fint*dpl+dfac
            else
              fcim=1.
            endif
          else
            fcim=assimfc
          endif
!
          dvd=ddmpv 
          do j=jds+1,jde-2
            dud=ddmpu(j)
            do i=ids+1,ide-2
              rdpd=1./(dsg2(l)*apd(i,j)+pdsg1(l))
              u(i,j,l)=(div(i+1,j,l)+div(i+1,j+1,l) &
                       -div(i  ,j,l)-div(i  ,j+1,l)) &
                      *dud*rdpd+u(i,j,l)
              v(i,j,l)=(div(i,j+1,l)+div(i+1,j+1,l) &
                       -div(i,j  ,l)-div(i+1,j  ,l)) &
                      *dvd*rdpd+v(i,j,l)
            enddo
          enddo
        enddo      ! end of vertical loop
!-----------------------------------------------------------------------
      endif
!-----------------------------------------------------------------------
                        endsubroutine ddamp
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine pdtsdt &
(ids,ide,jds,jde,lm &
,dt &
,sg2 &
,pd &
,pdo,psdt &
,psgdt &
!---temporary arguments-------------------------------------------------
,div)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 dt                          ! time step

real(kind=kfpt),dimension(1:lm+1),intent(in):: &
 sg2                        !

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(inout):: &
 pd                          ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(out):: &
 pdo &                       ! sigma range pressure difference
,psdt                        ! hydrostatic pressure tendency

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm-1),intent(out):: &
 psgdt                       ! vertical mass flux
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 div                         ! horizontal mass divergence
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      do l=2,lm
        do j=jds,jde
          do i=ids,ide
            div(i,j,l)=div(i,j,l-1)+div(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do j=jds,jde
        do i=ids,ide
          psdt(i,j)=0.
          pdo(i,j)=pd(i,j)
        enddo
      enddo

      do j=jds+1,jde-1
        do i=ids+1,ide-1
          psdt(i,j)=-div(i,j,lm)
          pd(i,j)=psdt(i,j)*dt+pd(i,j)
        enddo
      enddo
!---boundary conditions-------------------------------------------------
      do l=1,lm-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            psgdt(i,j,l)=-(psdt(i,j)*sg2(l+1)+div(i,j,l))
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
                        endsubroutine pdtsdt
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine adv1 &
(global,secadv &
,ids,ide,im,jds,jde,jm,lm,lnsad &
,dt,dyv,rdyh,rdyv &
,dsg2,pdsg1 &
,curv,dxv,fad,fah,rdxh,rdxv &
,f,pd,pdo &
,omgalf,psgdt &
,t,u,v &
,tp,up,vp &
!---temporary arguments-------------------------------------------------
,pfne,pfnw,pfx,pfy,tct,tcu,tcv)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 cfc=1.533 &                 ! adams-bashforth positioning in time
,bfc=1.-cfc &                ! adams bashforth positioning in time
,pfc=1.+4./6. &              ! 4th order momentum advection
,sfc=-1./6. &                ! 4th order momentum advection
,epscm=2.e-6 &               ! a floor value (not used)
,w1=1.0 &                    ! crank-nicholson uncentering
!,w1=0.80 &                   ! crank-nicholson uncentering
,w2=2.-w1                    ! crank-nicholson uncentering

logical(kind=klog),intent(in):: &
 global &                    ! global or regional
,secadv                      ! second order momentum advection

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,im &                        ! total # of columns
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,jm &                        ! total # of rows
,lm &                        ! total # of levels
,lnsad                       ! # of boundary lines w. upstream advection

real(kind=kfpt),intent(in):: &
 dt &                        ! dynamics time step
,dyv &                       ! deltay
,rdyh &                      ! 1/deltay
,rdyv                        ! 1/deltay

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 curv &                      ! curvature
,dxv &                       ! dxv
,fad &                       ! grid factor
,fah &                       ! grid factor
,rdxh &                      ! 1/deltax
,rdxv                        ! 1/deltax

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 f &                         ! coriolis parameter 
,pd &                        ! sigma range pressure difference
,pdo                         ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm-1),intent(in):: &
 psgdt                       ! vertical mass flux

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 omgalf &                    ! horizontal part of omegaalpha
,t &                         ! temperature
,u &                         ! u wind component
,v                           ! v wind component

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 tp &                        ! old temperature
,up &                        ! old u
,vp                          ! old v
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 pfne &                      ! mass flux, ne direction
,pfnw &                      ! mass flux, nw direction
,pfx &                       ! mass flux, x direction
,pfy &                       ! mass flux, y direction
,tct &                       ! time change of temperature
,tcu &                       ! time change of u
,tcv                         ! time change of v
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,iap &                       ! offset in x direction
,j &                         ! index in y direction
,jap &                       ! offset in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 cf &                        ! temporary
,cmt &                       ! temporary
,cmw &                       ! temporary
,crv &                       ! curvature
,dtq &                       ! dt/4
,dts &                       ! dt/16
,dux1 &                      ! momentum advection component
,duy1 &                      ! momentum advection component
,dvx1 &                      ! momentum advection component
,dvy1 &                      ! momentum advection component
,dxvp &                      ! 
,emhp &                      ! scaling in x direction
,enhp &                      ! scaling in y direction
,emvp &                      ! scaling in x direction
,envp &                      ! scaling in y direction
,fadp &                      ! grid factor at the point
,fahp &                      ! temporary grid factor
,fdpp &                      ! temporary grid factor
,fp &                        ! coriolis parameter factor
,fpp &                       ! coriolis with curvature
,pp &                        ! scaled trajectory, x direction
,qq &                        ! scaled trajectory, y direction
,rdp &                       ! 1/deltap
,rdxp &                      !
,rdyp &                      !
,vvlo &                      ! vertical velocity, lower interface
,vvup &                      ! vertical velocity, upper interface
,pvvup                       ! vertical mass flux, upper interface

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 pdop &                      ! hydrostatic pressure difference at v points
,pfnex1 &                    ! average mass flux for momentum advection
,pfney1 &                    ! average mass flux for momentum advection
,pfnwx1 &                    ! average mass flux for momentum advection
,pfnwy1 &                    ! average mass flux for momentum advection
,pfxx1 &                     ! average mass flux for momentum advection
,pfxy1 &                     ! average mass flux for momentum advection
,pfyx1 &                     ! average mass flux for momentum advection
,pfyy1 &                     ! average mass flux for momentum advection
,pvvlo &                     ! vertical mass flux, lower interface
,t1 &                        ! extrapolated temperature between time levels 
,tne &                       ! temperature flux, ne direction
,tnw &                       ! temperature flux, nw direction
,tx &                        ! temperature flux, x direction
,ty &                        ! temperature flux, y direction
,u1d &                       ! extrapolated diagonal u between time levels
,u2d &                       ! extrapolated diagonal u between time levels
,ufnex1 &                    ! average mass flux for momentum advection
,ufney1 &                    ! average mass flux for momentum advection
,ufnwx1 &                    ! average mass flux for momentum advection
,ufnwy1 &                    ! average mass flux for momentum advection
,ufxx1 &                     ! average mass flux for momentum advection
,ufxy1 &                     ! average mass flux for momentum advection
,ufyx1 &                     ! average mass flux for momentum advection
,ufyy1 &                     ! average mass flux for momentum advection
,v1d &                       ! extrapolated diagonal v between time levels
,v2d &                       ! extrapolated diagonal v between time levels
,vfnex1 &                    ! average mass flux for momentum advection
,vfney1 &                    ! average mass flux for momentum advection
,vfnwx1 &                    ! average mass flux for momentum advection
,vfnwy1 &                    ! average mass flux for momentum advection
,vfxx1 &                     ! average mass flux for momentum advection
,vfxy1 &                     ! average mass flux for momentum advection
,vfyx1 &                     ! average mass flux for momentum advection
,vfyy1                       ! average mass flux for momentum advection

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 crt &                       ! vertical advection temporary
,crw &                       ! vertical advection temporary
,rcmt &                      ! vertical advection temporary
,rcmw &                      ! vertical advection temporary
,rstt &                      ! vertical advection temporary
,rstu &                      ! vertical advection temporary
,rstv                        ! vertical advection temporary
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      do j=jds,jde
        do i=ids,ide
!          pdop(i,j)=(pd(i,j)+pdo(i,j))*0.5
          pdop(i,j)=pd(i,j)
        enddo
      enddo
!---crank-nicholson vertical advection----------------------------------
      dtq=dt*0.25
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          pvvlo(i,j)=psgdt(i,j,1)*dtq
          vvlo=pvvlo(i,j)/(dsg2(1)*pdop(i,j)+pdsg1(1))
          cmt=-vvlo*w2+1.
!          if(abs(cmt).lt.epscm) cmt=epscm
          rcmt(i,j,1)=1./cmt
          crt(i,j,1)=vvlo*w2
          rstt(i,j,1)=(-vvlo*w1)*(t(i,j,2)-t(i,j,1))+t(i,j,1)
        enddo
      enddo
      do l=2,lm-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            rdp=1./(dsg2(l)*pdop(i,j)+pdsg1(l))
            pvvup=pvvlo(i,j)
            pvvlo(i,j)=psgdt(i,j,l)*dtq
!
            vvup=pvvup*rdp
            vvlo=pvvlo(i,j)*rdp
!
            cf=-vvup*w2*rcmt(i,j,l-1)
            cmt=-crt(i,j,l-1)*cf+((vvup-vvlo)*w2+1.)
!            if(abs(cmt).lt.epscm) cmt=epscm
            rcmt(i,j,l)=1./cmt
            crt(i,j,l)=vvlo*w2
            rstt(i,j,l)=-rstt(i,j,l-1)*cf+t(i,j,l) &
                        -((t(i,j,l  )-t(i,j,l-1))*vvup &
                         +(t(i,j,l+1)-t(i,j,l  ))*vvlo)*w1
          enddo
        enddo
      enddo
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          pvvup=pvvlo(i,j)
          vvup=pvvup/(dsg2(lm)*pdop(i,j)+pdsg1(lm))
!
          cf=-vvup*w2*rcmt(i,j,lm-1)
          cmt=-crt(i,j,lm-1)*cf+(vvup*w2+1.)
!          if(abs(cmt).lt.epscm) cmt=epscm
          rcmt(i,j,lm)=1./cmt
          crt(i,j,lm)=0.
          rstt(i,j,lm)=-rstt(i,j,lm-1)*cf+t(i,j,lm) &
     &                 -(t(i,j,lm)-t(i,j,lm-1))*vvup*w1
!
          tct(i,j,lm)=rstt(i,j,lm)*rcmt(i,j,lm)-t(i,j,lm)
        enddo
      enddo
      do l=lm-1,1,-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            tct(i,j,l)=(-crt(i,j,l)*(t(i,j,l+1)+tct(i,j,l+1)) &
                        +rstt(i,j,l)) &
                      *rcmt(i,j,l)-t(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do l=1,lm
!-----------------------------------------------------------------------
        do j=jds,jde
          do i=ids,ide
            t1(i,j)=t(i,j,l)*cfc+tp(i,j,l)*bfc
            tp(i,j,l)=t(i,j,l)
          enddo
        enddo
!---temperature fluxes, on h points-------------------------------------
        do j=jds+1,jde
          do i=ids+1,ide
            tx(i,j)=(t1(i,j)-t1(i-1,j))*pfx(i,j,l)
            ty(i,j)=(t1(i,j)-t1(i,j-1))*pfy(i,j,l)
            tne(i,j)=(t1(i,j)-t1(i-1,j-1))*pfne(i,j,l)
            tnw(i,j)=(t1(i-1,j)-t1(i,j-1))*pfnw(i,j,l)
          enddo
        enddo
!---advection of temperature--------------------------------------------
        do j=jds+1+lnsad,jde-1-lnsad
          fahp=fah(j)
          do i=ids+1+lnsad,ide-1-lnsad
            tct(i,j,l)=(((tx(i,j)+tx(i+1,j)+ty(i,j)+ty(i,j+1)) &
                        +(tne(i+1,j+1)+tne(i  ,j) &
                         +tnw(i  ,j+1)+tnw(i+1,j))*0.25)*fahp) &
                      /(dsg2(l)*pdop(i,j)+pdsg1(l)) &
                      +omgalf(i,j,l)+tct(i,j,l)
          enddo
        enddo
!---regional branch-----------------------------------------------------
        if(.not.global) then
!-----------------------------------------------------------------------
          enhp=-dt*rdyh*0.25
          do j=2,2+lnsad-1
            emhp=-dt*rdxh(j)*0.25
            do i=2,im-1
              pp=(u(i-1,j-1,l)+u(i,j-1,l)+u(i-1,j,l)+u(i,j,l))*emhp
              qq=(v(i-1,j-1,l)+v(i,j-1,l)+v(i-1,j,l)+v(i,j,l))*enhp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tct(i,j,l)=(t(i+iap,j,l)-t(i,j,l))*pp &
                        +(t(i,j+jap,l)-t(i,j,l))*qq &
                        +(t(i,j,l)-t(i+iap,j,l) &
                         -t(i,j+jap,l)+t(i+iap,j+jap,l))*pp*qq &
                        +omgalf(i,j,l)+tct(i,j,l)
            enddo
          enddo
          do j=jm-1-lnsad+1,jm-1
            emhp=-dt*rdxh(j)*0.25
            do i=2,im-1
              pp=(u(i-1,j-1,l)+u(i,j-1,l)+u(i-1,j,l)+u(i,j,l))*emhp
              qq=(v(i-1,j-1,l)+v(i,j-1,l)+v(i-1,j,l)+v(i,j,l))*enhp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tct(i,j,l)=(t(i+iap,j,l)-t(i,j,l))*pp &
                        +(t(i,j+jap,l)-t(i,j,l))*qq &
                        +(t(i,j,l)-t(i+iap,j,l) &
                         -t(i,j+jap,l)+t(i+iap,j+jap,l))*pp*qq &
                        +omgalf(i,j,l)+tct(i,j,l)
            enddo
          enddo
          do j=2+lnsad,jm-1-lnsad
            emhp=-dt*rdxh(j)*0.25
            do i=2,2+lnsad-1
              pp=(u(i-1,j-1,l)+u(i,j-1,l)+u(i-1,j,l)+u(i,j,l))*emhp
              qq=(v(i-1,j-1,l)+v(i,j-1,l)+v(i-1,j,l)+v(i,j,l))*enhp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tct(i,j,l)=(t(i+iap,j,l)-t(i,j,l))*pp &
                        +(t(i,j+jap,l)-t(i,j,l))*qq &
                        +(t(i,j,l)-t(i+iap,j,l) &
                         -t(i,j+jap,l)+t(i+iap,j+jap,l))*pp*qq &
                        +omgalf(i,j,l)+tct(i,j,l)
            enddo
          enddo
          do j=2+lnsad,jm-1-lnsad
            emhp=-dt*rdxh(j)*0.25
            do i=im-1-lnsad+1,im-1
              pp=(u(i-1,j-1,l)+u(i,j-1,l)+u(i-1,j,l)+u(i,j,l))*emhp
              qq=(v(i-1,j-1,l)+v(i,j-1,l)+v(i-1,j,l)+v(i,j,l))*enhp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tct(i,j,l)=(t(i+iap,j,l)-t(i,j,l))*pp &
                        +(t(i,j+jap,l)-t(i,j,l))*qq &
                        +(t(i,j,l)-t(i+iap,j,l) &
                         -t(i,j+jap,l)+t(i+iap,j+jap,l))*pp*qq &
                        +omgalf(i,j,l)+tct(i,j,l)
            enddo
          enddo
!-----------------------------------------------------------------------
        endif ! regional lateral boundaries
!-----------------------------------------------------------------------
      enddo ! end of vertical loop
!---crank-nicholson vertical advection----------------------------------
      dts=dt*(0.25*0.25)
      do j=jds+1,jde-2
        do i=ids+1,ide-2
!          pdop(i,j)=((pd (i,j)+pd (i+1,j)+pd (i,j+1)+pd (i+1,j+1)) &
!                    +(pdo(i,j)+pdo(i+1,j)+pdo(i,j+1)+pdo(i+1,j+1))) &
!                   *0.125
          pdop(i,j)=(pd(i,j)+pd(i+1,j)+pd(i,j+1)+pd(i+1,j+1))*0.25
          pvvlo(i,j)=(psgdt(i,j  ,1)+psgdt(i+1,j  ,1) &
                     +psgdt(i,j+1,1)+psgdt(i+1,j+1,1))*dts
          vvlo=pvvlo(i,j)/(dsg2(1)*pdop(i,j)+pdsg1(1))
          cmw=-vvlo*w2+1.
!          if(abs(cmw).lt.epscm) cmw=epscm
          rcmw(i,j,1)=1./cmw
          crw(i,j,1)=vvlo*w2
          rstu(i,j,1)=(-vvlo*w1)*(u(i,j,2)-u(i,j,1))+u(i,j,1)
          rstv(i,j,1)=(-vvlo*w1)*(v(i,j,2)-v(i,j,1))+v(i,j,1)
        enddo
      enddo
      do l=2,lm-1
        do j=jds+1,jde-2
          do i=ids+1,ide-2
            rdp=1./(dsg2(l)*pdop(i,j)+pdsg1(l))
            pvvup=pvvlo(i,j)
            pvvlo(i,j)=(psgdt(i,j,l)+psgdt(i+1,j,l) &
                       +psgdt(i,j+1,l)+psgdt(i+1,j+1,l))*dts
            vvup=pvvup*rdp
            vvlo=pvvlo(i,j)*rdp
            cf=-vvup*w2*rcmw(i,j,l-1)
            cmw=-crw(i,j,l-1)*cf+((vvup-vvlo)*w2+1.)
!            if(abs(cmw).lt.epscm) cmw=epscm
            rcmw(i,j,l)=1./cmw
            crw(i,j,l)=vvlo*w2
            rstu(i,j,l)=-rstu(i,j,l-1)*cf+u(i,j,l) &
                        -((u(i,j,l  )-u(i,j,l-1))*vvup &
                         +(u(i,j,l+1)-u(i,j,l  ))*vvlo)*w1
            rstv(i,j,l)=-rstv(i,j,l-1)*cf+v(i,j,l) &
                        -((v(i,j,l  )-v(i,j,l-1))*vvup &
                         +(v(i,j,l+1)-v(i,j,l  ))*vvlo)*w1
          enddo
        enddo
      enddo
      do j=jds+1,jde-2
        do i=ids+1,ide-2
          rdp=1./(dsg2(lm)*pdop(i,j)+pdsg1(lm))
          pvvup=pvvlo(i,j)
          vvup=pvvup*rdp
          cf=-vvup*w2*rcmw(i,j,lm-1)
          cmw=-crw(i,j,lm-1)*cf+(vvup*w2+1.)
!          if(abs(cmw).lt.epscm) cmw=epscm
          rcmw(i,j,lm)=1./cmw
          crw(i,j,lm)=vvlo
          rstu(i,j,lm)=-rstu(i,j,lm-1)*cf+u(i,j,lm) &
                       -(u(i,j,lm)-u(i,j,lm-1))*vvup*w1
          rstv(i,j,lm)=-rstv(i,j,lm-1)*cf+v(i,j,lm) &
                       -(v(i,j,lm)-v(i,j,lm-1))*vvup*w1
          tcu(i,j,lm)=rstu(i,j,lm)*rcmw(i,j,lm)-u(i,j,lm)
          tcv(i,j,lm)=rstv(i,j,lm)*rcmw(i,j,lm)-v(i,j,lm)
        enddo
      enddo
      do l=lm-1,1,-1
        do j=jds+1,jde-2
          do i=ids+1,ide-2
            tcu(i,j,l)=(-crw(i,j,l)*(u(i,j,l+1)+tcu(i,j,l+1)) &
                        +rstu(i,j,l)) &
                      *rcmw(i,j,l)-u(i,j,l)
            tcv(i,j,l)=(-crw(i,j,l)*(v(i,j,l+1)+tcv(i,j,l+1)) &
                        +rstv(i,j,l)) &
                      *rcmw(i,j,l)-v(i,j,l)
          enddo
        enddo
      enddo
!---grand vertical loop-------------------------------------------------
      do l=1,lm
!-----------------------------------------------------------------------
        do j=jds,jde
          do i=ids,ide
            u1d(i,j)=u(i,j,l)*cfc+up(i,j,l)*bfc
            v1d(i,j)=v(i,j,l)*cfc+vp(i,j,l)*bfc
!
            up(i,j,l)=u(i,j,l)
            vp(i,j,l)=v(i,j,l)
          enddo
        enddo
!---coriolis force tendency---------------------------------------------
        do j=jds+1,jde-2
        crv=curv(j)*dt
          do i=ids+1,ide-2
            fp=f(i,j)*dt
            fpp=u1d(i,j)*crv+fp
!
            tcu(i,j,l)= fpp*v1d(i,j)+tcu(i,j,l)
            tcv(i,j,l)=-fpp*u1d(i,j)+tcv(i,j,l)
          enddo
        enddo
!---diagonally averaged fluxes on v points------------------------------
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            pfxx1 (i,j)=pfx (i  ,j  ,l)+pfx (i+1,j+1,l)
            pfyx1 (i,j)=pfy (i  ,j  ,l)+pfy (i+1,j+1,l)
            pfnex1(i,j)=pfne(i  ,j  ,l)+pfne(i+1,j+1,l)
            pfnwx1(i,j)=pfnw(i  ,j  ,l)+pfnw(i+1,j+1,l)
!
            pfxy1 (i,j)=pfx (i+1,j  ,l)+pfx (i  ,j+1,l)
            pfyy1 (i,j)=pfy (i+1,j  ,l)+pfy (i  ,j+1,l)
            pfney1(i,j)=pfne(i+1,j  ,l)+pfne(i  ,j+1,l)
            pfnwy1(i,j)=pfnw(i+1,j  ,l)+pfnw(i  ,j+1,l)
          enddo
        enddo
!---4th order momentum advection---------------------------------------
        if(.not.secadv) then
          do j=jds+1,jde-2
            do i=ids+1,ide-1
              u2d(i,j)=(u1d(i,j-1)+u1d(i-1,j) &
                       +u1d(i+1,j)+u1d(i,j+1))*sfc &
                      +(u1d(i,j)*pfc)
              v2d(i,j)=(v1d(i,j-1)+v1d(i-1,j) &
                       +v1d(i+1,j)+v1d(i,j+1))*sfc &
                      +(v1d(i,j)*pfc)
            enddo
          enddo
          if(global) then
            call swapw2(im,jm,u2d)
            call swapw2(im,jm,v2d)
            call poled2(im,jm,u2d,v2d)
          else
            do i=ids,ide
              u2d(i,jds)=u1d(i,jds)
              v2d(i,jds)=v1d(i,jds)
              u2d(i,jde-1)=u1d(i,jde-1)
              v2d(i,jde-1)=v1d(i,jde-1)
              u2d(i,jde)=u1d(i,jde)
              v2d(i,jde)=v1d(i,jde)
            enddo
            do j=jds,jde
              u2d(ids,j)=u1d(ids,j)
              v2d(ids,j)=v1d(ids,j)
              u2d(ide,j)=u1d(ide,j)
              v2d(ide,j)=v1d(ide,j)
            enddo
          endif
          do j=jds+1,jde-2
            do i=ids,ide
              u1d(i,j)=u2d(i,j)
              v1d(i,j)=v2d(i,j)
            enddo
          enddo
        endif
!---horizontal fluxes of momentum components on v points----------------
        do i=ids+1,ide-1
          pfyx1 (i,jds+1)=0.
          pfyy1 (i,jds+1)=0.
          pfnex1(i,jds+1)=0.
          pfnwx1(i,jds+1)=0.
          pfney1(i,jds+1)=0.
          pfnwy1(i,jds+1)=0.
!
          pfyx1 (i,jde-1)=0.
          pfyy1 (i,jde-1)=0.
          pfnex1(i,jde-1)=0.
          pfnwx1(i,jde-1)=0.
          pfney1(i,jde-1)=0.
          pfnwy1(i,jde-1)=0.
        enddo
!
        do j=jds+1,jde-1
          dxvp=dxv(j)
          do i=ids+1,ide-1
            ufxx1 (i,j)=(u1d(i  ,j  )-u1d(i-1,j  ))*pfxx1 (i,j)
            ufyx1 (i,j)=(u1d(i  ,j  )-u1d(i  ,j-1))*pfyx1 (i,j)
            ufnex1(i,j)=(u1d(i  ,j  )-u1d(i-1,j-1))*pfnex1(i,j)
            ufnwx1(i,j)=(u1d(i-1,j  )-u1d(i  ,j-1))*pfnwx1(i,j)
!
            ufxy1 (i,j)=(u1d(i  ,j  )-u1d(i-1,j  ))*pfxy1 (i,j)
            ufyy1 (i,j)=(u1d(i  ,j  )-u1d(i  ,j-1))*pfyy1 (i,j)
            ufney1(i,j)=(u1d(i  ,j  )-u1d(i-1,j-1))*pfney1(i,j)
            ufnwy1(i,j)=(u1d(i-1,j  )-u1d(i  ,j-1))*pfnwy1(i,j)
!
            vfxx1 (i,j)=(v1d(i  ,j  )-v1d(i-1,j  ))*pfxx1 (i,j)
            vfyx1 (i,j)=(v1d(i  ,j  )-v1d(i  ,j-1))*pfyx1 (i,j)
            vfnex1(i,j)=(v1d(i  ,j  )-v1d(i-1,j-1))*pfnex1(i,j)
            vfnwx1(i,j)=(v1d(i-1,j  )-v1d(i  ,j-1))*pfnwx1(i,j)
!
            vfxy1 (i,j)=(v1d(i  ,j  )-v1d(i-1,j  ))*pfxy1 (i,j)
            vfyy1 (i,j)=(v1d(i  ,j  )-v1d(i  ,j-1))*pfyy1 (i,j)
            vfney1(i,j)=(v1d(i  ,j  )-v1d(i-1,j-1))*pfney1(i,j)
            vfnwy1(i,j)=(v1d(i-1,j  )-v1d(i  ,j-1))*pfnwy1(i,j)
          enddo
        enddo
!---advection of momentum components------------------------------------
        rdyp=rdyv
        do j=jds+1+lnsad,jde-2-lnsad
          fadp=fad(j)
          dxvp=dxv(j)
          rdxp=rdxv(j)
          do i=ids+1+lnsad,ide-2-lnsad
!
            fdpp=fadp/(dsg2(l)*pdop(i,j)+pdsg1(l))    
!
            dux1= ufxx1 (i  ,j  )+ufxx1 (i+1,j  ) &
                 +ufyx1 (i  ,j  )+ufyx1 (i  ,j+1) &
                +(ufnex1(i+1,j+1)+ufnex1(i  ,  j) &
                 +ufnwx1(i  ,j+1)+ufnwx1(i+1,j  ))*0.25
!
            dvx1= vfxx1 (i  ,j  )+vfxx1 (i+1,j  ) &
                 +vfyx1 (i  ,j  )+vfyx1 (i  ,j+1) &
                +(vfnex1(i+1,j+1)+vfnex1(i  ,  j) &
                 +vfnwx1(i  ,j+1)+vfnwx1(i+1,j  ))*0.25
!
            duy1= ufxy1 (i  ,j  )+ufxy1 (i+1,j  ) &
                 +ufyy1 (i  ,j  )+ufyy1 (i  ,j+1) &
                +(ufney1(i+1,j+1)+ufney1(i  ,  j) &
                 +ufnwy1(i  ,j+1)+ufnwy1(i+1,j  ))*0.25
!
            dvy1= vfxy1 (i  ,j  )+vfxy1 (i+1,j  ) &
                 +vfyy1 (i  ,j  )+vfyy1 (i  ,j+1) &
                +(vfney1(i+1,j+1)+vfney1(i  ,  j) &
                 +vfnwy1(i  ,j+1)+vfnwy1(i+1,j  ))*0.25

            tcu(i,j,l)=((dux1+duy1)*dyv +(dvx1-dvy1)*dxvp)*rdyp*fdpp &
                      +tcu(i,j,l)
            tcv(i,j,l)=((dvx1+dvy1)*dxvp+(dux1-duy1)*dyv )*rdxp*fdpp &
                      +tcv(i,j,l)
          enddo
        enddo
!---regional branch-----------------------------------------------------
        if(.not.global) then
!---upstream advection of momentum along lateral boundaries-------------
          envp=-dt*rdyv
          do j=2,2+lnsad-1
            emvp=-dt*rdxv(j)
            do i=2,im-2
              pp=u(i,j,l)*emvp
              qq=v(i,j,l)*envp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tcu(i,j,l)=(u(i+iap,j,l)-u(i,j,l))*pp &
                        +(u(i,j+jap,l)-u(i,j,l))*qq &
                        +(u(i,j,l)-u(i+iap,j,l) &
                         -u(i,j+jap,l)+u(i+iap,j+jap,l))*pp*qq &
                        +tcu(i,j,l)
              tcv(i,j,l)=(v(i+iap,j,l)-v(i,j,l))*pp &
                        +(v(i,j+jap,l)-v(i,j,l))*qq &
                        +(v(i,j,l)-v(i+iap,j,l) &
                         -v(i,j+jap,l)+v(i+iap,j+jap,l))*pp*qq &
                        +tcv(i,j,l)
            enddo
          enddo
          do j=jm-2-lnsad+1,jm-2
            emvp=-dt*rdxv(j)
            do i=2,im-2
              pp=u(i,j,l)*emvp
              qq=v(i,j,l)*envp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tcu(i,j,l)=(u(i+iap,j,l)-u(i,j,l))*pp &
                        +(u(i,j+jap,l)-u(i,j,l))*qq &
                        +(u(i,j,l)-u(i+iap,j,l) &
                         -u(i,j+jap,l)+u(i+iap,j+jap,l))*pp*qq &
                        +tcu(i,j,l)
              tcv(i,j,l)=(v(i+iap,j,l)-v(i,j,l))*pp &
                        +(v(i,j+jap,l)-v(i,j,l))*qq &
                        +(v(i,j,l)-v(i+iap,j,l) &
                         -v(i,j+jap,l)+v(i+iap,j+jap,l))*pp*qq &
                        +tcv(i,j,l)
            enddo
          enddo
          do j=2+lnsad,jm-2-lnsad
            emvp=-dt*rdxv(j)
            do i=2,2+lnsad-1
              pp=u(i,j,l)*emvp
              qq=v(i,j,l)*envp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tcu(i,j,l)=(u(i+iap,j,l)-u(i,j,l))*pp &
                        +(u(i,j+jap,l)-u(i,j,l))*qq &
                        +(u(i,j,l)-u(i+iap,j,l) &
                         -u(i,j+jap,l)+u(i+iap,j+jap,l))*pp*qq &
                        +tcu(i,j,l)
              tcv(i,j,l)=(v(i+iap,j,l)-v(i,j,l))*pp &
                        +(v(i,j+jap,l)-v(i,j,l))*qq &
                        +(v(i,j,l)-v(i+iap,j,l) &
                         -v(i,j+jap,l)+v(i+iap,j+jap,l))*pp*qq &
                        +tcv(i,j,l)
            enddo
          enddo
          do j=2+lnsad,jm-2-lnsad
            emvp=-dt*rdxv(j)
            do i=im-2-lnsad+1,im-2
              pp=u(i,j,l)*emvp
              qq=v(i,j,l)*envp
!
              if(pp.le.0.) then
                iap=-1
                pp=-pp
              else
                iap=1
              endif
!
              if(qq.le.0.) then
                jap=-1
                qq=-qq
              else
                jap=1
              endif
!
              tcu(i,j,l)=(u(i+iap,j,l)-u(i,j,l))*pp &
                        +(u(i,j+jap,l)-u(i,j,l))*qq &
                        +(u(i,j,l)-u(i+iap,j,l) &
                         -u(i,j+jap,l)+u(i+iap,j+jap,l))*pp*qq &
                        +tcu(i,j,l)
              tcv(i,j,l)=(v(i+iap,j,l)-v(i,j,l))*pp &
                        +(v(i,j+jap,l)-v(i,j,l))*qq &
                        +(v(i,j,l)-v(i+iap,j,l) &
                         -v(i,j+jap,l)+v(i+iap,j+jap,l))*pp*qq &
                        +tcv(i,j,l)
            enddo
          enddo
!-----------------------------------------------------------------------
        endif ! regional lateral boundaries
!-----------------------------------------------------------------------
      enddo      ! end of vertical grand loop
!-----------------------------------------------------------------------
                        endsubroutine adv1
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine vtoa &
(ids,ide,jds,jde,lm &
,cp,dt,ef4t,pt &
,sg2 &
,psdt &
,dwdt,rtop,q,cw &
,omgalf &
,pint &
!---temporary arguments-------------------------------------------------
,div,tct)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
!--local variables------------------------------------------------------
real(kind=kfpt),parameter:: &
 cliq=4190. &                ! heat capacity of liquid water                         
,cvap=1952.                  ! heat capacity at const. pressure vapor

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 cp &                        !
,dt &                        ! dynamics time step
,ef4t &                      ! vertical grid parameter
,pt                          ! pressure at the top of the model's atmosphere

real(kind=kfpt),dimension(1:lm+1),intent(in):: &
 sg2                         ! delta sigmas

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 psdt                        ! surface geopotential

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 cw &                        ! condensate
,dwdt &                      ! nonhydrostatic correction factor
,q &                         ! specific humidity
,rtop                        ! RT/p

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 omgalf                      !

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm+1),intent(inout):: &
 pint                        ! pressure at interfaces

!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 div &                       ! horizontal mass divergence
,tct                         ! temperature tendency

!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 dwdtp &                     ! nonhydrostatic correction factor at the point
,fcliq &                     !
,fcvap &                     !
,toa &                       ! omega-alpha temporary
,tpmp                        ! pressure temporary at the point

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 tpm                         ! pressure temporary
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      fcliq=cliq/cp-1.       ! factor for correction of cp for liquid water
      fcvap=cvap/cp-1.       ! factor for correction of cp for vapor
!-----------------------------------------------------------------------
!*** Uncomment only for debugging without advection ***
!do l=1,lm
!  do j=jds,jde
!    do i=ids,ide
!      tct(i,j,l)=0.
!    enddo
!  enddo
!enddo
!-----------------------------------------------------------------------
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          pint(i,j,1)=pt
          tpm(i,j)=pt+pint(i,j,2)
!
          dwdtp=dwdt(i,j,1)
!
          tpmp=pint(i,j,2)+pint(i,j,3)
          toa=-div(i,j,1)*rtop(i,j,1)*dwdtp*ef4t &
              /(1.+fcvap*q(i,j,1)+fcliq*cw(i,j,1))
!
          omgalf(i,j,1)=omgalf(i,j,1)+toa
          tct(i,j,1)=tct(i,j,1)+toa
!
          pint(i,j,2)=psdt(i,j)*(sg2(1)+sg2(2))*dwdtp*dt &
                     +tpm(i,j)-pint(i,j,1)
!
          tpm(i,j)=tpmp
        enddo
      enddo
      do l=2,lm-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            dwdtp=dwdt(i,j,l)
!
            tpmp=pint(i,j,l+1)+pint(i,j,l+2)
            toa=-(div(i,j,l-1)+div(i,j,l))*rtop(i,j,l)*dwdtp*ef4t &
            /(1.+fcvap*q(i,j,l)+fcliq*cw(i,j,l))
!
            omgalf(i,j,l)=omgalf(i,j,l)+toa
            tct(i,j,l)=tct(i,j,l)+toa
!
            pint(i,j,l+1)=psdt(i,j)*(sg2(l)+sg2(l+1))*dwdtp*dt &
                         +tpm(i,j)-pint(i,j,l)
!
            tpm(i,j)=tpmp
          enddo
        enddo
      enddo
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          dwdtp=dwdt(i,j,lm)
!
          toa=-(div(i,j,lm-1)+div(i,j,lm))*rtop(i,j,lm)*dwdtp*ef4t &
              /(1.+fcvap*q(i,j,lm)+fcliq*cw(i,j,lm))
!
          omgalf(i,j,lm)=omgalf(i,j,lm)+toa
          tct(i,j,lm)=tct(i,j,lm)+toa
!
          pint(i,j,lm+1)=psdt(i,j)*(sg2(lm)+sg2(lm+1))*dwdtp*dt &
                        +tpm(i,j)-pint(i,j,lm)
        enddo
      enddo
!-----------------------------------------------------------------------
                        endsubroutine vtoa
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine updatet &
(ids,ide,jds,jde,lm &
,t &
!---temporary arguments-------------------------------------------------
,tct)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,lm                          ! total # of levels

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 t                           ! temperature
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 tct                         ! temperature time change
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      do l=1,lm
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            t(i,j,l)=t(i,j,l)+tct(i,j,l)
            tct(i,j,l)=0.
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
                        endsubroutine updatet
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine updateuv &
(ids,ide,jds,jde,lm &
,u,v &
!---temporary arguments-------------------------------------------------
,tcu,tcv)
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 cflfc=1./(160.*160.)            ! cfl limit

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,lm                          ! total # of levels

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 u &                         ! u
,v                           ! v
!-----------------------------------------------------------------------
!---temporary arguments-------------------------------------------------
!-----------------------------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 tcu &                       ! u time change
,tcv                         ! v time change
!-----------------------------------------------------------------------
!--local variables------------------------------------------------------
!-----------------------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 cflc &                      ! 
,rcflc                       !

!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
      do l=1,lm
        do j=jds+1,jde-2
          do i=ids+1,ide-2
            u(i,j,l)=u(i,j,l)+tcu(i,j,l)
            v(i,j,l)=v(i,j,l)+tcv(i,j,l)
!
            tcu(i,j,l)=0.
            tcv(i,j,l)=0.
!
            cflc=(u(i,j,l)*u(i,j,l)+v(i,j,l)*v(i,j,l))*cflfc
            if(cflc.gt.1.) then
              rcflc=sqrt(1./cflc)
              u(i,j,l)=u(i,j,l)*rcflc
              v(i,j,l)=v(i,j,l)*rcflc
            endif
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
                        endsubroutine updateuv

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
                        subroutine deform &
(global,hydro &
,ids,ide,jds,jde,lm &
,rdyh &
,epsq2 &
,rare,rdxh,rdxv &
,q2,u,v,w &
!---temporary arguments-------------------------------------------------
,def)
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 defsfc=1. &                 ! shear component weight
,deftfc=1. &                 ! longitudinal component weight
,defwfc=1. &                 ! vertical velocity shear weight
,scq2=50. &                  ! 2tke weighting factor
,epsq=1.e-20                 ! floor value for specific humidity

!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 global &                    ! global or regional
,hydro                       ! hydrostatic dynamics switch

integer(kind=kint),intent(in):: &
 ide &                       ! ending longitudinal index
,ids &                       ! starting longitudinal index
,jde &                       ! ending latitudinal index
,jds &                       ! starting latitudinal index
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 rdyh                        ! 1/deltay

real(kind=kfpt),dimension(1:lm),intent(in):: &
 epsq2                       ! floor value of 2tke

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 rare &                      !
,rdxh &                      ! 1/deltax
,rdxv  

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 q2 &                        ! 2tke
,u &                         ! u wind component
,v &                         ! v wind component
,w                           ! w wind component

!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(out):: &
 def                         ! 

!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 defc &                      ! floor
,defm &                      ! cap
,defp &                      ! 
,defs &                      ! horizontal wind shear component
,deft &                      ! longitudinal component 
,defw &                      ! w shear component
,q2trm                       !

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 dux2 &                      !
,duy2 &                      !
,dvx2 &                      !
,dvy2 &                      !
,dwx2 &                      !
,dwy2                        !
!-----------------------------------------------------------------------
!***********************************************************************
!-----------------------------------------------------------------------
!
      if(global) then
!        defm=1.35e-3  /8./2. ! /2. ! cap for smag2=0.4
!        defm=2.50e-3  /8./2. ! /2. ! cap for smag2=0.3
        defm=1.35e-3/2.  !/4. ! cap for smag2=0.2
!        defm=0.070
      else
        defm=9999.
      endif
!-----------------------------------------------------------------------
!---grand vertical loop-------------------------------------------------
!-----------------------------------------------------------------------
!
!.......................................................................
!zj$omp parallel do &
!zj$omp private(def1,def2,def3,def4,defc,defp,defs,deft,i,j,l,q2trm)
!.......................................................................
!-----------------------------------------------------------------------
!
      vertical_loop_1: do l=1,lm
!
!-----------------------------------------------------------------------
        if(global) then ! global
          if(l.le.3) then
            defc=0.005*(4.-float(l))/3.
          else
            defc=defm*0.01
          endif
        else ! regional
          if (l.le.3) then
            defc=0.005*(4.-float(l))/3.
          else
            defc=0.
          endif
        endif
!----------------------------------------------------------------------
        do j=jds,jde-1
          do i=ids+1,ide
            dux2(i,j)=((u(i  ,j  ,l)-u(i-1,j  ,l))*rdxv(j))**2
            dvx2(i,j)=((v(i  ,j  ,l)-v(i-1,j  ,l))*rdxv(j))**2
            dwy2(i,j)=((w(i  ,j+1,l)-w(i  ,j  ,l))*rdyh   )**2
          enddo
        enddo
!
        do j=jds+1,jde
          do i=ids,ide-1
            duy2(i,j)=((u(i  ,j  ,l)-u(i  ,j-1,l))*rdyh   )**2
            dvy2(i,j)=((v(i  ,j  ,l)-v(i  ,j-1,l))*rdyh   )**2
            dwx2(i,j)=((w(i+1,j  ,l)-w(i  ,j  ,l))*rdxh(j))**2
          enddo
        enddo
!
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            deft=dux2(i,j-1)+dux2(i,j)+dvy2(i-1,j)+dvy2(i,j)
            defs=dvx2(i,j-1)+dvx2(i,j)+duy2(i-1,j)+duy2(i,j)
!
            if(.not.hydro) then
              defw=dwx2(i-1,j)+dwx2(i,j)+dwy2(i,j-1)+dwy2(i,j)
            else
              defw=0.
            endif
!
            if(q2(i,j,l).gt.epsq2(l)) then
              q2trm=scq2*q2(i,j,l)*rare(j)
            else
              q2trm=0.
            endif
!
            defp=sqrt(deft*deftfc+defs*defsfc+defw*defwfc+q2trm)
            defp=max(defp,defc)
            defp=min(defp,defm)
!
            def(i,j,l)=defp
          enddo
        enddo
!-----------------------------------------------------------------------
!
      enddo vertical_loop_1
!
!-----------------------------------------------------------------------
!.......................................................................
!zj$omp end parallel do
!.......................................................................
!-----------------------------------------------------------------------
        if(.not.global) then
          do l=1,lm
            do i=ids,ide
              def(i,jds,l)=0.
            enddo
!
            do i=ids,ide
              def(i,jde,l)=0.
            enddo
!
            do j=jds,jde
              def(ids,j,l)=0.
            enddo
!
            do j=jds,jde
              def(ide,j,l)=0.
            enddo
         enddo
       endif
!-----------------------------------------------------------------------
!
                        endsubroutine deform
!
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
                        subroutine hdiff &
(hydro &
,ids,ide,jds,jde,lm,lpt2 &
,rdyh &
,dsg2,pdsg1 &
,rdxh &
,sice,sm &
,hdacx,hdacy,hdacvx,hdacvy &
,pd &
,z &
,cw,q,q2,t,u,v &
!---temporary arguments-------------------------------------------------
,def)
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 slopec=.01
! slopec=.05                  ! critical slope
!,slopec=9999.                !

!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 hydro                       ! logical switch for nonhydrostatic dynamics

integer(kind=kint),intent(in):: &
 ide &                       ! ending longitudinal index
,ids &                       ! starting longitudinal index
,jde &                       ! ending latitudinal index
,jds &                       ! starting latitudinal index                       
,lm  &                       ! total # of levels
,lpt2                        ! # of levels in the pressure range

real(kind=kfpt),intent(in):: &
 rdyh                        ! 1/deltay

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigma2 in hybrid
,pdsg1                       ! delta pd1 in hybrid

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 rdxh                        ! 1/deltax

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 hdacx &                     ! exchange coefficient for mass points
,hdacy &                     ! exchange coefficient for mass points
,hdacvx &                    ! exchange coefficient for velocity points
,hdacvy &                    ! exchange coefficient for velocity points
,pd &                        ! pressure difference
,sice &                      ! sea-ice mask
,sm                          ! sea mask

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 z                           ! height at mass points

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 cw &                        ! condensate
,q &                         ! specific humidity
,q2 &                        ! 2tke
,t &                         ! temperature
,u &                         ! u wind component
,v                           ! v wind component
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 def                         ! 
!-----------------------------------------------------------------------
!--local variables------------------------------------------------------
!-----------------------------------------------------------------------
logical(kind=klog):: &
 cilinx &                    ! coast/ice line
,ciliny                      ! coast/ice line

integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 hkfx &                      ! def with slope factor
,hkfy &                      ! def with slope factor
,rdpdp &                     ! 1./dp at v points
,slopx &                     ! x slope
,slopy                       ! y slope

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 cx &                        ! condensate difference, x direction
,cy &                        ! condensate difference, y direction
,dpd &                       ! delta pd at h points
,fmlx &                      ! x slope mask
,fmly &                      ! y slope mask
,hkx &                       ! sum, x direction
,hky &                       ! sum, y direction
,qx &                        ! specific humidity difference, x direction
,qy &                        ! specific humidity difference, y direction
,q2x &                       ! 2tke difference, x direction
,q2y &                       ! 2tke difference, y direction
,rdpd &                      ! 1./dp at h points
,tx &                        ! temperature difference, x direction
,ty &                        ! temperature difference, y direction
,ux &                        ! u wind component difference, x direction
,uy &                        ! u wind component difference, y direction
,vx &                        ! v wind component difference, x direction
,vy                          ! v wind component difference, y direction
!-----------------------------------------------------------------------
!***********************************************************************
!.......................................................................
!zj$omp parallel do &
!zj$omp private(cilinx,ciliny,cx,cy,fmlx,fmly, &
!zj$omp         hkfx,hkfy,hkx,hky,i,j,l, &
!zj$omp         q2x,q2y,qx,qy,slopx,slopy, &
!zj$omp         tx,ty,ux,uy,vx,vy)
!.......................................................................
!-----------------------------------------------------------------------
      vertical_loop_1: do l=1,lm
!-----------------------------------------------------------------------
        if(l.gt.lpt2.and..not.hydro) then
          do j=jds+1,jde
            do i=ids+1,ide
              cilinx=sice(i-1,j).ne.sice(i,j) &
                 .or.sm  (i-1,j).ne.sm  (i,j)
              ciliny=sice(i,j-1).ne.sice(i,j) &
                 .or.sm  (i,j-1).ne.sm  (i,j)
              slopx=abs((z(i,j,l)-z(i-1,j,l))*rdxh(j))
              slopy=abs((z(i,j,l)-z(i,j-1,l))*rdyh   )
!
              if(slopx.le.slopec.or.cilinx) then
                fmlx(i,j)=1.
              else
                fmlx(i,j)=0.
              endif
!
              if(slopy.le.slopec.or.ciliny) then
                fmly(i,j)=1.
              else
                fmly(i,j)=0.
              endif
!
            enddo
          enddo
        else
          do j=jds+1,jde
            do i=ids+1,ide
              fmlx(i,j)=1.
              fmly(i,j)=1.
            enddo
          enddo
        endif
!-----------------------------------------------------------------------
!---contributions behind mass points------------------------------------
!-----------------------------------------------------------------------
        do j=jds+1,jde
          do i=ids+1,ide
            dpd(i,j)=pd(i,j)*dsg2(l)+pdsg1(l)
            rdpd(i,j)=1./dpd(i,j)
            def(i,j,l)=def(i,j,l)*dpd(i,j)
          enddo
        enddo
!
        do j=jds+1,jde
          do i=ids+1,ide
            hkx(i,j)=(def(i-1,j,l)+def(i,j,l))
            hky(i,j)=(def(i,j-1,l)+def(i,j,l))
            hkfx=hkx(i,j)*fmlx(i,j)
            hkfy=hky(i,j)*fmly(i,j)
!
            tx (i,j)=(t (i,j,l)-t (i-1,j,l))*hkfx
            qx (i,j)=(q (i,j,l)-q (i-1,j,l))*hkfx
            cx (i,j)=(cw(i,j,l)-cw(i-1,j,l))*hkfx
            q2x(i,j)=(q2(i,j,l)-q2(i-1,j,l))*hkfx
!
            ty (i,j)=(t (i,j,l)-t (i,j-1,l))*hkfy
            qy (i,j)=(q (i,j,l)-q (i,j-1,l))*hkfy
            cy (i,j)=(cw(i,j,l)-cw(i,j-1,l))*hkfy
            q2y(i,j)=(q2(i,j,l)-q2(i,j-1,l))*hkfy
          enddo
        enddo
!-----------------------------------------------------------------------
!---u,v, contributions, behind v points---------------------------------
!-----------------------------------------------------------------------
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            ux(i,j)=(u(i,j,l)-u(i-1,j,l))*hky(i,j+1)
            vx(i,j)=(v(i,j,l)-v(i-1,j,l))*hky(i,j+1)
            uy(i,j)=(u(i,j,l)-u(i,j-1,l))*hkx(i+1,j)
            vy(i,j)=(v(i,j,l)-v(i,j-1,l))*hkx(i+1,j)
          enddo
        enddo
!-----------------------------------------------------------------------
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            t (i,j,l)=((tx (i+1,j)-tx (i,j))*hdacx(i,j) &
                      +(ty (i,j+1)-ty (i,j))*hdacy(i,j))*rdpd(i,j) &
                     +t (i,j,l)
            q (i,j,l)=((qx (i+1,j)-qx (i,j))*hdacx(i,j) &
                      +(qy (i,j+1)-qy (i,j))*hdacy(i,j))*rdpd(i,j) &
                     +q (i,j,l)
            cw(i,j,l)=((cx (i+1,j)-cx (i,j))*hdacx(i,j) &
                      +(cy (i,j+1)-cy (i,j))*hdacy(i,j))*rdpd(i,j) &
                     +cw(i,j,l)
            if(l.ne.lm) &
            q2(i,j,l)=((q2x(i+1,j)-q2x(i,j))*hdacx(i,j) &
                      +(q2y(i,j+1)-q2y(i,j))*hdacy(i,j))*rdpd(i,j) &
                     +q2(i,j,l)
          enddo
        enddo
!-----------------------------------------------------------------------
        do j=jds+1,jde-2
          do i=ids+1,ide-2
            rdpdp=4./(dpd(i  ,j  )+dpd(i+1,j  ) &
                     +dpd(i  ,j+1)+dpd(i+1,j+1))
!
            u(i,j,l)=((ux(i+1,j)-ux(i,j))*hdacvx(i,j) &
                     +(uy(i,j+1)-uy(i,j))*hdacvy(i,j))*rdpdp+u(i,j,l)
            v(i,j,l)=((vx(i+1,j)-vx(i,j))*hdacvx(i,j) &
                     +(vy(i,j+1)-vy(i,j))*hdacvy(i,j))*rdpdp+v(i,j,l)
          enddo
        enddo
!-----------------------------------------------------------------------
!
      enddo vertical_loop_1
!.......................................................................
!zj$omp end parallel do
!.......................................................................
!
!-----------------------------------------------------------------------
!
                        endsubroutine hdiff
!
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine cdzdt &
(global,hydro &
,ids,ide,im,jds,jde,jm,lm &
,dt,g,r &
,dsg2,pdsg1 &
,fah &
,fis,pd,pdo &
,psgdt &
,cw,q,rtop,t &
,pint &
,dwdt,pdwdt,w &
,z & 
!---temporary arguments-------------------------------------------------
,pfne,pfnw,pfx,pfy)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 cfc=1.533 &                 ! adams-bashforth positioning in time
,bfc=1.-cfc                  ! adams bashforth positioning in time
!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 global &                    ! global or regional
,hydro                       ! hydrostatic or nonhydrostatic

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,im &                        ! total # of columns
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,jm &                        ! total # of rows
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 dt &                        ! dynamics time step
,g &                         ! gravity
,r                           ! gas constant for dry air

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 fah                         !

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 fis &                       ! surface geopotential
,pd &                        ! sigma range pressure difference
,pdo                         ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm-1),intent(in):: &
 psgdt                       ! vertical mass flux

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 cw &                        ! condensate
,q &                         ! specific humidity
,rtop &                      ! rt/p
,t                           ! temperature

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm+1),intent(in):: &
 pint                        ! pressure at interfaces

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 dwdt &                      ! nonhydrostatic correction factor
,pdwdt &                     ! previous nonhydrostatic correction factor
,w                           ! w wind component

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(out):: &
 z                           ! heights in the middle of the layers
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 pfne &                      ! mass flux, ne direction
,pfnw &                      ! mass flux, nw direction
,pfx &                       ! mass flux, x direction
,pfy                         ! mass flux, y direction
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 dw &                        ! w wind component increment
,dz &                        ! layer thickness
,fahp &                      !
,rg &                        ! 
,rdt &                       ! 
,trog &                      ! 
,wup &                       ! w wind component at upper interface
,zup                         ! height of upper interface

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 tta &                       ! advection through upper interface
,ttb &                       ! advection through lower interface
,wlo &                       ! w wind component at lower interface
,zlo &                       ! height at lower interface
,zne &                       ! height flux, ne direction
,znw &                       ! height flux, nw direction
,zx &                        ! height flux, x direction
,zy                          ! height flux, y direction
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      rg=1./g
      rdt=1./dt
      trog=2.*r/g
!-----------------------------------------------------------------------
      do j=jds,jde
        do i=ids,ide
          wlo(i,j)=0.
          zlo(i,j)=fis(i,j)*rg
        enddo
      enddo
!---nonhydrostatic equation---------------------------------------------
      do l=lm,1,-1
        do j=jds,jde
          do i=ids,ide
            pdwdt(i,j,l)=dwdt(i,j,l)
            dwdt(i,j,l)=w(i,j,l)
!
            dz=(q(i,j,l)*0.608+(1.-cw(i,j,l)))*t(i,j,l)*trog &
              *(dsg2(l)*pd(i,j)+pdsg1(l))/(pint(i,j,l)+pint(i,j,l+1))
            dw=(dz-rtop(i,j,l)*(dsg2(l)*pdo(i,j)+pdsg1(l))*rg)*rdt
!
            zup=zlo(i,j)+dz
            wup=wlo(i,j)+dw
            z(i,j,l)=dz*0.5+zlo(i,j)
            w(i,j,l)=dw*0.5+wlo(i,j)
!
            zlo(i,j)=zup
            wlo(i,j)=wup
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      if(hydro) return
!-----------------------------------------------------------------------
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          ttb(i,j)=0.
        enddo
      enddo
      do l=1,lm-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            tta(i,j)=(z(i,j,l+1)-z(i,j,l))*psgdt(i,j,l)*0.5
            w(i,j,l)=(tta(i,j)+ttb(i,j))/(dsg2(l)*pdo(i,j)+pdsg1(l)) &
                    +w(i,j,l)
            ttb(i,j)=tta(i,j)
          enddo
        enddo
      enddo
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          w(i,j,lm)=ttb(i,j)/(dsg2(lm)*pdo(i,j)+pdsg1(lm))+w(i,j,lm)
        enddo
      enddo
!---grand horizontal loop-----------------------------------------------
      do l=1,lm
!-------------mass fluxes, on h points----------------------------------
        do j=jds+1,jde
          do i=ids+1,ide
            zx(i,j)=(z(i,j,l)-z(i-1,j,l))*pfx(i,j,l)
            zy(i,j)=(z(i,j,l)-z(i,j-1,l))*pfy(i,j,l)
            zne(i,j)=(z(i,j,l)-z(i-1,j-1,l))*pfne(i,j,l)
            znw(i,j)=(z(i-1,j,l)-z(i,j-1,l))*pfnw(i,j,l)
          enddo
        enddo
!---advection of height-------------------------------------------------
        do j=jds+1,jde-1
          fahp=-fah(j)/dt
          do i=ids+1,ide-1
            w(i,j,l)=((zx(i,j)+zx(i+1,j)+zy(i,j)+zy(i,j+1)) &
                    +(zne(i+1,j+1)+zne(i,j) &
                     +znw(i,j+1)+znw(i+1,j))*0.25)*fahp &
                    /(dsg2(l)*pdo(i,j)+pdsg1(l)) &
                    +w(i,j,l)
          enddo
        enddo
!---setting w on h points to 0. along boundaries------------------------
        if(.not.global) then
!-----------------------------------------------------------------------
          do j=1,2
            do i=1,im
              w(i,j,l)=0.
            enddo
          enddo
          do j=jm-1,jm
            do i=1,im
              w(i,j,l)=0.
            enddo
          enddo
          do j=1,jm
            do i=1,2
              w(i,j,l)=0.
            enddo
          enddo
          do j=1,jm
            do i=im-1,im
              w(i,j,l)=0.
            enddo
          enddo
!-----------------------------------------------------------------------
        endif
!-----------------------------------------------------------------------
      enddo      ! end of vertical loop
!-----------------------------------------------------------------------
                        endsubroutine cdzdt
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine cdwdt &
(global,hydro &
,ids,ide,im,jds,jde,jm,lm,ntsd &
,dt,g &
,dsg2,pdsg1,psgml1 &
,fah &
,hdacx,hdacy,pd,pdo &
,psgdt &
,dwdt,pdwdt,w &
,pint &
!---temporary arguments-------------------------------------------------
,def,pfne,pfnw,pfx,pfy)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
integer(kind=kint),parameter:: &
 nsmud=00 &                  ! number of smoothing iterations
,lnsnh=02                    ! # of rows with smoothing along boundaries

real(kind=kfpt),parameter:: &
 epsfc=9.81 &                ! limiter value
,epsn=-epsfc &               ! floor value for vertical acceleration
,epsp=epsfc &                ! upper limit for vertical acceleration
,epsvw=0.05 &                ! limit on horizontal advection of w           
!,epsvw=9999. &               ! limit on horizontal advection of w
,fwhy=1. &                   ! dwdt control factor
,rslpd=1./1500. &            ! 1./dwdt control layer depth (1./Pa)
,wa=0.125 &                  ! weighting factor
,wb=0.5 &                    ! weighting factor
!,wad=0.125 &                 ! lateral smoothing weight 
!,wad=0.0625 &                ! lateral smoothing weight
!,wad=0.075 &                 ! lateral smoothing weight
!,wad=0.050 &                 ! lateral smoothing weight
,wad=0. &                    ! lateral smoothing weight
!,wp=0.075 &                  ! time smoothing weight
,wp=0.                       ! time smoothing weight
!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 global &                    ! global or regional
,hydro                       ! hidrostatic or nonhydrostatic

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,im &                        ! total # of columns
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,jm &                        ! total # of rows
,lm &                        ! total # of levels
,ntsd                        ! time steps done

real(kind=kfpt),intent(in):: &
 dt &                        ! dynamics time step
,g                           ! gravity

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1 &                     ! delta pressures
,psgml1                      !

real(kind=kfpt),dimension(1:jm),intent(in):: &
 fah                         ! delta sigmas

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 hdacx &                     ! exchange coefficient for mass points          
,hdacy &                     ! exchange coefficient for mass points
,pd &                        ! sigma range pressure difference
,pdo                         ! old sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm-1),intent(in):: &
 psgdt                       ! vertical mass flux

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 dwdt &                      ! nonhydrostatic correction factor
,pdwdt &                     ! previous nonhydrostatic correction factor
,w                           ! w wind component

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm+1),intent(inout):: &
 pint                        ! pressure
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 def &                       ! deformation
,pfne &                      ! mass flux, ne direction
,pfnw &                      ! mass flux, nw direction
,pfx &                       ! mass flux, x direction
,pfy                         ! mass flux, y direction
!--local variables------------------------------------------------------
logical(kind=klog) :: &
 diffw=.false.               ! turn horizontal diffusion of w on/off
! diffw=.true.                ! turn horizontal diffusion of w on/off

integer(kind=kint):: &
 i &                         ! index in x direction
,imn &                       !
,imx &                       !
,j &                         ! index in y direction
,jmn &                       !
,jmx &                       !
,l &                         ! index in p direction
,lmn &                       !
,lmx &                       !
,kn &                        ! counter
,kp &                        ! counter
,ks &                        ! smoothing counter
,lmbaro &                    ! lower boundary for extracting baro
,lsbaro &                    ! upper boundary for extracting baro
,lsltp                       !

real(kind=kfpt):: &
 advec &                     ! horizontal advection of w
,arg &                       !
,dpdba &                     !
,dwdtmn &                    ! minimum value of dwdt
,dwdtmx &                    ! maximum value of dwdt
,dwdtp &                     ! nonhydrostatic correction factor at the point
,fahp &                      ! grid factor
,pi &                        !                                               
,rdt &                       ! 1/dt
,rg &                        ! 1/g
,sltp                        ! uppermost midlayer pressure

real(kind=kfpt),dimension(1:lm):: &
 why                         ! dwdt control weight

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 pdba &                      !
,tta &                       ! advection through upper interface
,ttb &                       ! advection through lower interface
,wne &                       ! height flux, ne direction
,wnw &                       ! height flux, nw direction
,ww &                        ! temporary for lateral smoothing
,wx &                        ! height flux, x direction
,wy                          ! height flux, y direction
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      if(hydro.or.ntsd.lt.3) then
!-----------------------------------------------------------------------
        do l=1,lm
          do j=jds,jde
            do i=ids,ide
              dwdt(i,j,l)=1.
              pdwdt(i,j,l)=1.
              pint(i,j,l+1)=dsg2(l)*pd(i,j)+pdsg1(l)+pint(i,j,l)
            enddo
          enddo
        enddo
!-----------------------------------------------------------------------
        return
!-----------------------------------------------------------------------
      endif
!-----------------------------------------------------------------------
      if(.not.global) then 
!---smoothing of w on h points along boundaries-------------------------
        if(nsmud.gt.0) then
!-----------------------------------------------------------------------
          do ks=1,nsmud
!-----------------------------------------------------------------------
            do l=1,lm
!-----------------------------------------------------------------------
              do j=2,lnsnh
                do i=2,im-1
                  ww(i,j)=(w(i,j-1,l)+w(i-1,j,l) &
                          +w(i+1,j,l)+w(i,j+1,l))*wa &
                          +w(i,j,l)*wb       
                enddo
              enddo
              do j=jm-lnsnh+1,jm-1
                do i=2,im-1
                  ww(i,j)=(w(i,j-1,l)+w(i-1,j,l) &
                          +w(i+1,j,l)+w(i,j+1,l))*wa &
                          +w(i,j,l)*wb       
                enddo
              enddo
              do j=lnsnh+1,jm-lnsnh
                do i=2,lnsnh
                  ww(i,j)=(w(i,j-1,l)+w(i-1,j,l) &
                          +w(i+1,j,l)+w(i,j+1,l))*wa &
                          +w(i,j,l)*wb       
                enddo
              enddo
              do j=lnsnh+1,jm-lnsnh
                do i=im-lnsnh+1,im-1
                  ww(i,j)=(w(i,j-1,l)+w(i-1,j,l) &
                          +w(i+1,j,l)+w(i,j+1,l))*wa &
                          +w(i,j,l)*wb       
                enddo
              enddo
!
              do j=2,lnsnh
                do i=2,im-1
                  w(i,j,l)=ww(i,j)
                enddo
              enddo
              do j=jm-lnsnh+1,jm-1
                do i=2,im-1
                  w(i,j,l)=ww(i,j)
                enddo
              enddo
              do j=lnsnh+1,jm-lnsnh
                do i=2,lnsnh
                  w(i,j,l)=ww(i,j)
                enddo
              enddo
              do j=lnsnh+1,jm-lnsnh
                do i=im-lnsnh+1,im-1
                  w(i,j,l)=ww(i,j)
                enddo
              enddo
!-----------------------------------------------------------------------
            enddo
!-----------------------------------------------------------------------
          enddo
!-----------------------------------------------------------------------
        endif ! end of smoothing of w along lateral boundaries
!-----------------------------------------------------------------------
      endif
!---local derivative of w on h points-----------------------------------
      rdt=1./dt
      do l=1,lm
        do j=jds,jde
          do i=ids,ide
            dwdt(i,j,l)=(w(i,j,l)-dwdt(i,j,l))*rdt
          enddo
        enddo
      enddo
!---lateral diffusion of w----------------------------------------------
      if(diffw) then
!-----------------------------------------------------------------------
        do l=1,lm
!---w fluxes, on h points-----------------------------------------------
          do j=jds+1,jde
            do i=ids+1,ide
              wx(i,j)=(w(i,j,l)-w(i-1,j,l))*(def(i-1,j,l)+def(i,j,l))
              wy(i,j)=(w(i,j,l)-w(i,j-1,l))*(def(i,j-1,l)+def(i,j,l))
            enddo
          enddo
!---diffusion of w------------------------------------------------------
          do j=jds+1,jde-1
            do i=ids+1,ide-1  
              dwdt(i,j,l)=-((wx(i+1,j)-wx(i,j))*hdacx(i,j) &
                           +(wy(i,j+1)-wy(i,j))*hdacy(i,j))*rdt &
                          /(pd(i,j)*dsg2(l)+pdsg1(l)) &
                         +dwdt(i,j,l)
            enddo
          enddo
!-----------------------------------------------------------------------
        enddo
!-----------------------------------------------------------------------
      endif
!---vertical advection of w---------------------------------------------
      do j=jds,jde
        do i=ids,ide
          ttb(i,j)=0.
        enddo
      enddo
      do l=1,lm-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            tta(i,j)=(w(i,j,l+1)-w(i,j,l))*psgdt(i,j,l)*0.5
            dwdt(i,j,l)=(tta(i,j)+ttb(i,j))/(dsg2(l)*pdo(i,j)+pdsg1(l)) &
                       +dwdt(i,j,l)
            ttb(i,j)=tta(i,j)
          enddo
        enddo
      enddo
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          dwdt(i,j,lm)=ttb(i,j)/(dsg2(lm)*pdo(i,j)+pdsg1(lm)) &
                      +dwdt(i,j,lm)
        enddo
      enddo
!-----------------------------------------------------------------------
      do l=1,lm
!---w fluxes, on h points-----------------------------------------------
        do j=jds+1,jde
          do i=ids+1,ide
            wx (i,j)=(w(i  ,j  ,l)-w(i-1,j  ,l))*pfx(i,j,l)
            wy (i,j)=(w(i  ,j  ,l)-w(i  ,j-1,l))*pfy(i,j,l)
            wne(i,j)=(w(i  ,j  ,l)-w(i-1,j-1,l))*pfne(i,j,l)
            wnw(i,j)=(w(i-1,j  ,l)-w(i  ,j-1,l))*pfnw(i,j,l)
          enddo
        enddo
!---advection of w------------------------------------------------------
        do j=jds+1,jde-1
          fahp=-fah(j)/dt
          do i=ids+1,ide-1
            advec=((wx(i,j)+wx(i+1,j)+wy(i,j)+wy(i,j+1)) &
                  +(wne(i+1,j+1)+wne(i,j) &
                   +wnw(i,j+1)+wnw(i+1,j))*0.25)*fahp &
                 /(dsg2(l)*pdo(i,j)+pdsg1(l))
!
            if(advec.gt. epsvw) advec= epsvw
            if(advec.lt.-epsvw) advec=-epsvw
!
            dwdt(i,j,l)=advec+dwdt(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
        pi=3.141592653589793
!
        do l=1,lm
          why(l)=-99.
        enddo
!
        lsltp=0
        sltp=(psgml1(1)+psgml1(2))*0.5
        do l=1,lm-1
          arg=((psgml1(l)+psgml1(l+1))*0.5-sltp)*rslpd
          if(arg.gt.1.) exit
          why(l)=1.-fwhy*cos(arg*pi*0.5)**2
          lsltp=l
        enddo
!
        do l=1,lsltp
          do j=jds,jde
            do i=ids,ide
              dwdt(i,j,l)=dwdt(i,j,l)*why(l)
            enddo
          enddo
        enddo
!-----------------------------------------------------------------------
        lsbaro=1
        lmbaro=lm
!
        do j=jds,jde
          do i=ids,ide
            ttb (i,j)=0.
            pdba(i,j)=0.
          enddo
        enddo
!
        do l=lsbaro,lmbaro
          do j=jds,jde
            do i=ids,ide
              dpdba=dsg2(l)*pdo(i,j)+pdsg1(l)
              ttb (i,j)=dwdt(i,j,l)*dpdba+ttb (i,j)
              pdba(i,j)=pdba(i,j)+dpdba
            enddo
          enddo
        enddo
!
        do j=jds,jde
          do i=ids,ide
            ttb(i,j)=ttb(i,j)/pdba(i,j)
          enddo
        enddo
!
        do l=1,lm
          do j=jds,jde
            do i=ids,ide
              dwdt(i,j,l)=dwdt(i,j,l)-ttb(i,j)
            enddo
          enddo
        enddo
!-----------------------------------------------------------------------
      if(global) then
        call swaphn(im,jm,lm,dwdt)
        call polehn(im,jm,lm,dwdt)
      endif
!------------spatial filtering of dwdt----------------------------------
      if(wad.gt.0.) then
        do l=1,lm
          do j=jds+1,jde-1
            do i=ids+1,ide-1
              ww(i,j)=(dwdt(i,j-1,l)+dwdt(i-1,j,l) &
                      +dwdt(i+1,j,l)+dwdt(i,j+1,l)-dwdt(i,j,l)*4.0)
            enddo
          enddo
          do j=jds+1,jde-1
            do i=ids+1,ide-1
              dwdt(i,j,l)=ww(i,j)*wad+dwdt(i,j,l)
            enddo
          enddo
        enddo
      endif
!-----------------------------------------------------------------------
      if(global) then
        call swaphn(im,jm,lm,dwdt)
        call polehn(im,jm,lm,dwdt)
      endif
!-----------------------------------------------------------------------
      rg=1./g
!
      kn=0
      kp=0
      imn=0
      jmn=0
      lmn=0
      imx=0
      jmx=0
      lmx=0
      dwdtmx=0.
      dwdtmn=0.
!
      do l=1,lm
        do j=jds,jde
          do i=ids,ide
            dwdtp=dwdt(i,j,l)
            if(dwdtp.gt.dwdtmx) then
              dwdtmx=dwdtp
              imx=i
              jmx=j
              lmx=l
            endif
            if(dwdtp.lt.dwdtmn) then
              dwdtmn=dwdtp
              imn=i
              jmn=j
              lmn=l
            endif
            if(dwdtp.lt.epsn) then
              dwdtp=epsn
              kn=kn+1
            endif
            if(dwdtp.gt.epsp) then
              dwdtp=epsp
              kp=kp+1
            endif
            dwdt(i,j,l)=(dwdtp*rg+1.)*(1.-wp)+pdwdt(i,j,l)*wp
          enddo
        enddo
      enddo
!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
      print*,'kn=',kn,' kp=',kp
      print*,'imn=',imn,' jmn=',jmn,' lmn=',lmn,' dwmn=',dwdtmn
      print*,'imx=',imx,' jmx=',jmx,' lmx=',lmx,' dwmx=',dwdtmx
!aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
!-----------------------------------------------------------------------
!        do l=1,lm
!          why(l)=-99.
!        enddo
!
!        lsltp=0
!        sltp=(psgml1(1)+psgml1(2))*0.5
!        do l=1,lm-1
!          arg=((psgml1(l)+psgml1(l+1))*0.5-sltp)*rslpd
!          if(arg.gt.1.) exit
!          why(l)=1.-fwhy*cos(arg*pi*0.5)**2
!          lsltp=l
!        enddo
!
!        do l=1,lsltp
!          do j=jds,jde
!            do i=ids,ide
!              dwdt(i,j,l)=dwdt(i,j,l)*why(l)
!            enddo
!          enddo
!        enddo
!-----------------------------------------------------------------------
!        lsbaro=1
!        lmbaro=lm
!
!        do j=jds,jde
!          do i=ids,ide
!            ttb (i,j)=0.
!            pdba(i,j)=0.
!          enddo
!        enddo
!
!        do l=lsbaro,lmbaro
!          do j=jds,jde
!            do i=ids,ide
!              dpdba=dsg2(l)*pdo(i,j)+pdsg1(l)
!              ttb (i,j)=dwdt(i,j,l)*dpdba+ttb (i,j)
!              pdba(i,j)=pdba(i,j)+dpdba
!            enddo
!          enddo
!        enddo
!!
!        do j=jds,jde
!          do i=ids,ide
!            ttb(i,j)=ttb(i,j)/pdba(i,j)
!          enddo
!        enddo
!
!        do l=1,lm
!          do j=jds,jde
!            do i=ids,ide
!              dwdt(i,j,l)=dwdt(i,j,l)-ttb(i,j)
!            enddo
!          enddo
!        enddo
!----------------------------------------------------------------------
      if(.not.global) then 
!---setting dwdt on h points to 1. along boundaries---------------------
        do l=1,lm
          do j=1,2
            do i=1,im
              dwdt(i,j,l)=1.
            enddo
          enddo
          do j=jm-1,jm
            do i=1,im
              dwdt(i,j,l)=1.
            enddo
          enddo
          do j=1,jm
            do i=1,2
              dwdt(i,j,l)=1.
            enddo
          enddo
            do j=1,jm
              do i=im-1,im
                dwdt(i,j,l)=1.
            enddo
          enddo
        enddo
!-----------------------------------------------------------------------
      endif ! regional 
!-----------------------------------------------------------------------
                        endsubroutine cdwdt
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine vsound &
(global,hydro &
,ids,ide,im,jds,jde,jm,lm,ntsd &
,cp,dt,g,pt,r &
,dsg2,pdsg1 &
,pd &
,cw,q,rtop &
,dwdt,t,w &
,pint)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
integer(kind=kint),parameter:: &
 nsmud=0                     ! number of smoothing iterations

real(kind=kfpt),parameter:: &
 wght=0.35                   ! first guess weight
!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 global &                    ! global or regional
,hydro                       ! hydrostatic or nonhydrostatic

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,im &                        ! total # of columns
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,jm &                        ! total # of rows
,lm &                        ! total # of levels
,ntsd                        ! time step

real(kind=kfpt),intent(in):: &
 cp &                        ! cp
,dt &                        ! dynamics time step
,g &                         ! gravity
,pt &                        ! pressure at the top of the model's atmosphere
,r                           ! gas constant for dry air

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 pd                          ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 cw &                        ! condensate
,q &                         ! specific humidity
,rtop                        ! rt/p

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 dwdt &                      ! nonhydrostatic correction factor
,t &                         ! previous nonhydrostatic correction factor
,w                           ! w wind component

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm+1),intent(inout):: &
 pint
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction

real(kind=kfpt):: &
 cappa &                     ! R/cp
,cofl &                      !
,dp &                        !
,delp &                      !
,dppl &                      !
,dpstr &                     !
,dptl &                      !
,fcc &                       ! 
,ffc &                       ! 
,gdt &                       ! g*dt
,gdt2 &                      ! gdt**2
,pp1 &                       !
,rcph &                      ! 
,rdpdn &                     !
,rdpup &                     !
,tfc &                       !
,tmp                         !

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 dptu                        !

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 b1 &                        !
,b2 &                        !
,b3 &                        !
,c0 &                        !
,rdpp                        !

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm+1):: &
 chi &                       !
,coff &                      !
,dfrh &                      !
,pnp1 &                      !
,pone &                      !
,pstr                        !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      if(hydro.or.ntsd.lt.3) return
!-----------------------------------------------------------------------
      cappa=r/cp
      gdt=g*dt
      gdt2=gdt*gdt
      ffc=-r*0.25/gdt2
      rcph=0.5/cp
!-----------------------------------------------------------------------
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          pone(i,j,1)=pt
          pstr(i,j,1)=pt
          pnp1(i,j,1)=pt
        enddo
      enddo
      do l=2,lm+1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            dppl=dsg2(l-1)*pd(i,j)+pdsg1(l-1)
            rdpp(i,j,l-1)=1./dppl
            dpstr=dwdt(i,j,l-1)*dppl
            pstr(i,j,l)=pstr(i,j,l-1)+dpstr
            pp1=pnp1(i,j,l-1)+dpstr
            pone(i,j,l)=pint(i,j,l)
            dp=(pp1-pone(i,j,l))*wght
            pnp1(i,j,l)=pone(i,j,l)+dp
            tfc=q(i,j,l-1)*0.608-cw(i,j,l-1)+1.
            fcc=(1.-cappa*tfc)*tfc*ffc
            cofl=t(i,j,l-1)*dppl*fcc &
                /((pnp1(i,j,l-1)+pnp1(i,j,l))*0.5)**2
            coff(i,j,l-1)=cofl
            dfrh(i,j,l)=(pstr(i,j,l-1)+pstr(i,j,l) &
                        -pone(i,j,l-1)-pone(i,j,l))*cofl
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do l=2,lm
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            rdpdn=rdpp(i,j,l)
            rdpup=rdpp(i,j,l-1)
            b1(i,j,l)=coff(i,j,l-1)+rdpup
            b2(i,j,l)=coff(i,j,l-1)+coff(i,j,l)-rdpup-rdpdn
            b3(i,j,l)=coff(i,j,l)+rdpdn
            c0(i,j,l)=-dfrh(i,j,l)-dfrh(i,j,l+1)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          b2(i,j,lm)=b2(i,j,lm)+b3(i,j,lm)
        enddo
      enddo
      do l=3,lm
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            tmp=-b1(i,j,l)/b2(i,j,l-1)
            b2(i,j,l)=b3(i,j,l-1)*tmp+b2(i,j,l)
            c0(i,j,l)=c0(i,j,l-1)*tmp+c0(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          chi(i,j,1)=0.
          chi(i,j,lm)=c0(i,j,lm)/b2(i,j,lm)
          chi(i,j,lm+1)=chi(i,j,lm)
        enddo
      enddo
      do l=lm-1,2,-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            chi(i,j,l)=(-b3(i,j,l)*chi(i,j,l+1)+c0(i,j,l))/b2(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do l=1,lm+1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            pnp1(i,j,l)=chi(i,j,l)+pstr(i,j,l)
            pint(i,j,l)=pnp1(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          dptu(i,j)=0.
        enddo
      enddo
      do l=1,lm
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            dptl=pnp1(i,j,l+1)-pone(i,j,l+1)
            t(i,j,l)=(dptu(i,j)+dptl)*rtop(i,j,l)*rcph+t(i,j,l)
            delp=(pnp1(i,j,l+1)-pnp1(i,j,l))*rdpp(i,j,l)
            w(i,j,l)=(delp-dwdt(i,j,l))*gdt+w(i,j,l)
            dwdt(i,j,l)=delp
            dptu(i,j)=dptl
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      if(.not.global) then 
!---setting dwdt on h points to 1. along boundaries---------------------
        do l=1,lm
          do j=1,2
            do i=1,im
              dwdt(i,j,l)=1.
            enddo
          enddo
          do j=jm-1,jm
            do i=1,im
              dwdt(i,j,l)=1.
            enddo
          enddo
          do j=1,jm
            do i=1,2
              dwdt(i,j,l)=1.
            enddo
          enddo
            do j=1,jm
              do i=im-1,im
                dwdt(i,j,l)=1.
            enddo
          enddo
        enddo
!-----------------------------------------------------------------------
      endif ! regional 
!----------------------------------------------------------------------
                        endsubroutine vsound
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine adv2 &
(idtad,ids,ide,jds,jde,lm,kss,kse &
,dt &
,dsg2,epsq2,pdsg1 &
,fah &
,pd,pdo &
,psgdt &
,q,cw,rrw,q2 &
,s,sp &
!---temporary arguments-------------------------------------------------
,pfne,pfnw,pfx,pfy,s1,tcs)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 cfc=1.533 &                 ! adams-bashforth positioning in time
,bfc=1.-cfc &                ! adams bashforth positioning in time
,epsq=1.e-20 &               ! floor value for specific humidity
,pfc=1.+4./6. &              ! 4th order momentum advection
,sfc=-1./6. &                ! 4th order momentum advection
,epscm=2.e-6 &               ! a floor value (not used)
,w1=1.0 &                    ! crank-nicholson uncentering
!,w1=0.80 &                   ! crank-nicholson uncentering
,w2=2.-w1                    ! crank-nicholson uncentering

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,idtad &                     !
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,kse &                       ! terminal species index
,kss &                       ! initial species index
,lm                          ! total # of levels

real(kind=kfpt),intent(in):: &
 dt                          ! dynamics time step

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,epsq2 &                     ! floor value of 2tke
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 fah                         ! grid factor

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 pd &                        ! sigma range pressure difference
,pdo                         ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm-1),intent(in):: &
 psgdt                       ! vertical mass flux

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(in):: &
 q &                         ! specific humidity
,cw &                        ! cloudwater
,rrw &                       ! microphysics specie
,q2                          ! 2.*TKE

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm,kss:kse),intent(inout):: &
 s &                         ! tracers
,sp                          ! s at previous time level
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 pfne &                      ! mass flux, ne direction
,pfnw &                      ! mass flux, nw direction
,pfx &                       ! mass flux, x direction
,pfy                         ! mass flux, y direction

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm,kss:kse),intent(inout):: &
 s1 &                        ! intermediate value of s
,tcs                         ! timechange of s
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         !
,j &                         !
,ks &                        !
,l                           !

real(kind=kfpt):: &
 cf &                        ! temporary
,cms &                       ! temporary
,dtq &                       ! dt/4
,fahp &                      ! temporary grid factor
,rdp &                       ! 1/deltap
,vvlo &                      ! vertical velocity, lower interface
,vvup &                      ! vertical velocity, upper interface
,pvvup                       ! vertical mass flux, upper interface

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 pdop &                      ! hydrostatic pressure difference at v points
,pvvlo &                     ! vertical mass flux, lower interface
,ss1 &                       ! extrapolated species between time levels 
,ssne &                      ! temperature flux, ne direction
,ssnw &                      ! temperature flux, nw direction
,ssx &                       ! temperature flux, x direction
,ssy                         ! temperature flux, y direction

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 crs &                       ! vertical advection temporary
,rcms                        ! vertical advection temporary

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm,kss:kse):: &
 rsts                        ! vertical advection temporary
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      do l=1,lm
        do j=jds,jde
          do i=ids,ide
            s(i,j,l,1)=max(q  (i,j,l),epsq)
            s(i,j,l,2)=max(cw (i,j,l),epsq)
            s(i,j,l,3)=max(rrw(i,j,l),epsq)
          enddo
        enddo
      enddo
      do j=jds,jde
        do i=ids,ide
          s(i,j,1,4)=max((q2 (i,j,1)+epsq2(1))*0.5,epsq2(1))
        enddo
      enddo
      do l=2,lm
        do j=jds,jde
          do i=ids,ide
            s(i,j,l,4)=max((q2 (i,j,l-1)+q2 (i,j,l))*0.5,epsq2(l))
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do ks=kss,kse ! loop by species
!-----------------------------------------------------------------------
        do l=1,lm
          do j=jds,jde
            do i=ids,ide
              s1(i,j,l,ks)=sqrt(s(i,j,l,ks))
            enddo
          enddo
        enddo
!-----------------------------------------------------------------------
      enddo ! end of the loop by species
!-----------------------------------------------------------------------
      do j=jds,jde
        do i=ids,ide
!          pdop(i,j)=(pd(i,j)+pdo(i,j))*0.5
          pdop(i,j)=pd(i,j)
        enddo
      enddo
!---crank-nicholson vertical advection----------------------------------
      dtq=dt*0.25*idtad
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          pvvlo(i,j)=psgdt(i,j,1)*dtq
          vvlo=pvvlo(i,j)/(dsg2(1)*pdop(i,j)+pdsg1(1))
          cms=-vvlo*w2+1.
          rcms(i,j,1)=1./cms
          crs(i,j,1)=vvlo*w2
!
          do ks=kss,kse
            rsts(i,j,1,ks)=(-vvlo*w1)*(s1(i,j,2,ks)-s1(i,j,1,ks)) &
                          +s1(i,j,1,ks)
          enddo
        enddo
      enddo
      do l=2,lm-1
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            rdp=1./(dsg2(l)*pdop(i,j)+pdsg1(l))
            pvvup=pvvlo(i,j)
            pvvlo(i,j)=psgdt(i,j,l)*dtq
!
            vvup=pvvup*rdp
            vvlo=pvvlo(i,j)*rdp
!
            cf=-vvup*w2*rcms(i,j,l-1)
            cms=-crs(i,j,l-1)*cf+((vvup-vvlo)*w2+1.)
            rcms(i,j,l)=1./cms
            crs(i,j,l)=vvlo*w2
!
            do ks=kss,kse
              rsts(i,j,l,ks)=-rsts(i,j,l-1,ks)*cf+s1(i,j,l,ks) &
                             -(s1(i,j,l  ,ks)-s1(i,j,l-1,ks))*vvup*w1 &
                             -(s1(i,j,l+1,ks)-s1(i,j,l  ,ks))*vvlo*w1
            enddo
          enddo
        enddo
      enddo
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          pvvup=pvvlo(i,j)
          vvup=pvvup/(dsg2(lm)*pdop(i,j)+pdsg1(lm))
!
          cf=-vvup*w2*rcms(i,j,lm-1)
          cms=-crs(i,j,lm-1)*cf+(vvup*w2+1.)
          rcms(i,j,lm)=1./cms
          crs(i,j,lm)=0.
!
          do ks=kss,kse
            rsts(i,j,lm,ks)=-rsts(i,j,lm-1,ks)*cf+s1(i,j,lm,ks) &
                           -(s1(i,j,lm,ks)-s1(i,j,lm-1,ks))*vvup*w1
!
            tcs(i,j,lm,ks)=rsts(i,j,lm,ks)*rcms(i,j,lm)-s1(i,j,lm,ks)
          enddo
        enddo
      enddo
      do ks=kss,kse
        do l=lm-1,1,-1
          do j=jds+1,jde-1
            do i=ids+1,ide-1
              tcs(i,j,l,ks)=(-crs(i,j,l)*(s1(i,j,l+1,ks)+tcs(i,j,l+1,ks)) &
                             +rsts(i,j,l,ks)) &
                           *rcms(i,j,l)-s1(i,j,l,ks)
            enddo
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do ks=kss,kse ! loop by species
!-----------------------------------------------------------------------
        do l=1,lm
          do j=jds,jde
            do i=ids,ide
              ss1(i,j)=s1(i,j,l,ks)*cfc+sp(i,j,l,ks)*bfc
              sp(i,j,l,ks)=s1(i,j,l,ks)
            enddo
          enddo
!---temperature fluxes, on h points-------------------------------------
          do j=jds+1,jde
            do i=ids+1,ide
              ssx(i,j)=(ss1(i,j)-ss1(i-1,j))*pfx(i,j,l)
              ssy(i,j)=(ss1(i,j)-ss1(i,j-1))*pfy(i,j,l)
!
              ssne(i,j)=(ss1(i,j)-ss1(i-1,j-1))*pfne(i,j,l)
              ssnw(i,j)=(ss1(i-1,j)-ss1(i,j-1))*pfnw(i,j,l)
            enddo
          enddo
!---advection of species------------------------------------------------
          do j=jds+1,jde-1
            fahp=fah(j)*idtad
            do i=ids+1,ide-1
              tcs(i,j,l,ks)=(((ssx (i  ,j  )+ssx (i+1,j  ) &
                              +ssy (i  ,j  )+ssy (i  ,j+1)) &
                             +(ssne(i+1,j+1)+ssne(i  ,j  ) &
                              +ssnw(i  ,j+1)+ssnw(i+1,j  ))*0.25) &
                             *fahp) &
                           /(dsg2(l)*pdop(i,j)+pdsg1(l)) &
                           +tcs(i,j,l,ks)
            enddo
          enddo
        enddo
!-----------------------------------------------------------------------
      enddo ! end of the loop by the species
!-----------------------------------------------------------------------
                        endsubroutine adv2
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine mono &
(ids,ide,jds,jde,lm,kss,kse &
,dsg2,pdsg1 &
,dare &
,pd &
,s &
!---temporary arguments-------------------------------------------------
,s1,tcs)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 epsq=1.e-20                 ! floor value for specific humidity

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,kse &                       ! terminal species index
,kss &                       ! initial species index
,lm                          ! total # of levels

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1                       ! delta pressures

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 dare                        ! grid box area

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 pd                          ! sigma range pressure difference

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm,kss:kse),intent(inout):: &
 s                           ! tracers at current time level

!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm,kss:kse),intent(inout):: &
 s1 &                        ! intermediate value of s
,tcs                         ! timechange of s
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         !
,j &                         !
,ks &                        !
,l                           !

real(kind=kfpt):: &
 s1p &                       !
,smax &                      ! local maximum
,smin &                      ! local minimum
,smaxh &                     ! horizontal local maximum
,sminh &                     ! horizontal local minimum
,smaxv &                     ! vertical local maximum
,sminv &                     ! vertical local minimum
,sumq                        !

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 dvol                        ! grid box volume
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
open(unit=101,file='../output/tracer.txt',status='unknown')
!-----------------------------------------------------------------------
sumq=0.
      do l=1,lm
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            dvol (i,j,l)=(dsg2(l)*pd(i,j)+pdsg1(l))*dare(j)
sumq=s(i,j,l,1)*dvol(i,j,l)+sumq
          enddo
        enddo
      enddo
!---monotonization------------------------------------------------------
      do ks=kss,kse ! loop by species
!-----------------------------------------------------------------------
        do l=1,lm
          do j=jds+1,jde-1
            do i=ids+1,ide-1
              s1p=(s1(i,j,l,ks)+tcs(i,j,l,ks))**2
              tcs(i,j,l,ks)=s1p-s(i,j,l,ks)
!
              sminh=min(s(i-1,j-1,l,ks) &
                       ,s(i  ,j-1,l,ks) &
                       ,s(i+1,j-1,l,ks) &
                       ,s(i-1,j  ,l,ks) &
                       ,s(i  ,j  ,l,ks) &
                       ,s(i+1,j  ,l,ks) &
                       ,s(i-1,j+1,l,ks) &
                       ,s(i  ,j+1,l,ks) &
                       ,s(i+1,j+1,l,ks))
              smaxh=max(s(i-1,j-1,l,ks) &
                       ,s(i  ,j-1,l,ks) &
                       ,s(i+1,j-1,l,ks) &
                       ,s(i-1,j  ,l,ks) &
                       ,s(i  ,j  ,l,ks) &
                       ,s(i+1,j  ,l,ks) &
                       ,s(i-1,j+1,l,ks) &
                       ,s(i  ,j+1,l,ks) &
                       ,s(i+1,j+1,l,ks))
!
              if(l.gt.1.and.l.lt.lm) then
                sminv=min(s(i,j,l-1,ks),s(i,j,l  ,ks),s(i,j,l+1,ks))
                smaxv=max(s(i,j,l-1,ks),s(i,j,l  ,ks),s(i,j,l+1,ks))
              elseif(l.eq.1) then
                sminv=min(s(i,j,l  ,ks),s(i,j,l+1,ks))
                smaxv=max(s(i,j,l  ,ks),s(i,j,l+1,ks))
              else
                sminv=min(s(i,j,l-1,ks),s(i,j,l  ,ks))
                smaxv=max(s(i,j,l-1,ks),s(i,j,l  ,ks))
              endif
!
              smin=min(sminh,sminv)
              smax=max(smaxh,smaxv)
!
              s1p=(s1(i,j,l,ks)+tcs(i,j,l,ks))**2
!
              if(s1p.gt.smax) s1p=smax
              if(s1p.lt.smin) s1p=smin
!
              tcs(i,j,l,ks)=s1p-s(i,j,l,ks)
!
            enddo
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do l=1,lm
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            tcs(i,j,l,4)=(dsg2(l)*pd(i,j)+pdsg1(l))*tcs(i,j,l,4)
          enddo
        enddo
      enddo
      do l=1,lm-1
          do j=jds+1,jde-1
            do i=ids+1,ide-1
            tcs(i,j,l,4)=(tcs(i,j,l,4)+tcs(i,j,l+1,4)) &
                        /((dsg2(l  )*pd(i,j)+pdsg1(l  )) &
                         +(dsg2(l+1)*pd(i,j)+pdsg1(l+1)))
          enddo
        enddo
      enddo
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          tcs(i,j,lm,4)=0.
        enddo
      enddo
!-----------------------------------------------------------------------
                        endsubroutine mono
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine updates &
(ids,ide,jds,jde,lm &
,s &
!---temporary arguments-------------------------------------------------
,tcs)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,lm                          ! total # of levels

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 s                           ! species
!---temporary arguments-------------------------------------------------
real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 tcs                         ! species time change
!--local variables------------------------------------------------------
integer(kind=kint):: &
 i &                         ! index in x direction
,j &                         ! index in y direction
,l                           ! index in p direction
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      do l=1,lm
        do j=jds+1,jde-1
          do i=ids+1,ide-1
            s(i,j,l)=s(i,j,l)+tcs(i,j,l)
            tcs(i,j,l)=0.
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
                        endsubroutine updates
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        endmodule module_dynamics
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
