!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        module module_radiation
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
use module_fltbnds
use module_control,only:zjexp
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        contains
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine radtn &
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      (ihrst,nradl,nrads,ntsd &
      ,dt &
      ,idat &
      ,dsg2,sg2,sgml2 &
      ,pdsg1,psg1,psgml1 &
      ,lbot,ltop &
      ,albase,albedo,snoalb,cldefi,glat,glon &
      ,pd,rswin,rlwin,sno,th02,ths,vegfrc &
      ,cw,q,t)
!-----------------------------------------------------------------------                            
      include 'modelgrid.inc'
!-----------------------------------------------------------------------                            
      parameter &
      (sc=1366.22*0.97 &
      ,a1=610.78e0,a2=17.2693882,a3=273.15,a4=35.86,pq0=379.90516 &
!      ,snoalb=0.60 &
      ,crmin=.60,rhcr=0.60 &
      ,epsq=1.e-20)
!-----------------------------------------------------------------------
      dimension &
       lbot(im,jm),ltop(im,jm) 
!-----------------------------------------------------------------------
      dimension &
       rswin(im,jm),rlwin(im,jm)
      dimension &
       tendl(im,jm,lm),tends(im,jm,lm)
      save tendl,tends
!-----------------------------------------------------------------------
      dimension &
       idat(3) &
      ,dsg2(lm),sg2(lm+1),sgml2(lm) &
      ,pdsg1(lm),psg1(lm+1),psgml1(lm) &
      ,albase(im,jm),albedo(im,jm),snoalb(im,jm) &
      ,cldefi(im,jm),czen(im,jm) &
      ,glat(im,jm),glon(im,jm) &
      ,pd(im,jm),sno(im,jm),th02(im,jm),ths(im,jm),vegfrc(im,jm) &
      ,cw(im,jm,lm),q(im,jm,lm),t(im,jm,lm)
!-------------local variables requiring declaration--------------------- 
      dimension pm(lm),dpl(lm),pb(lm+1),tm(lm),qm(lm),cm(lm)
      dimension oz(lm),or(lm),om(lm)
      dimension tndl(lm+2),tnds(lm+2)
!
      logical long,shrt
      logical lfirst
      save lfirst
      data lfirst/.true./
!-----------------------------------------------------------------------
      if(lfirst) then
        do j=1,jm
          do i=1,im
            rlwin(i,j)=0.0
            rswin(i,j)=0.0
            ltop(i,j)=lm
            lbot(i,j)=lm
            do l=1,lm
              tendl(i,j,l)=0.0
              tends(i,j,l)=0.0
            enddo
          enddo
        enddo
      endif
!-----------------------------------------------------------------------
      shrt=mod(ntsd-1,nrads).eq.0
      long=mod(ntsd-1,nradl).eq.0
!-----------------------------------------------------------------------
      if(long.or.shrt.or.lfirst) then
        lfirst=.false.
!-------------test for clouds only in (roughly) lowest 80% of domain----
        lcld=int(0.8*lm+0.5)
        if(mod(lcld,3).ne.0) lcld=lcld+1
        if(mod(lcld,3).ne.0) lcld=lcld-2
        lhi=lm-lcld+1
!-------------computation of zenith angle-------------------------------
        if(shrt) then
          times=(ntsd-1+nrads/2)*dt
          call zenith &
          (1,im,1,jm,ihrst,idat &
          ,times,dayi,hour &
          ,glon,glat &
          ,czen)
        endif
!-------------grand horizontal loop-------------------------------------
        do j=2,jm-1
          do i=2,im-1
!-----------------------------------------------------------------------
            do l=1,lm
              q(i,j,l)=max(q(i,j,l),epsq)
            enddo
!
            pdp=pd(i,j)
            do l=1,lm
              pm(l)=(sgml2(l)*pdp+psgml1(l))*0.01 ! pm from pa to mb
              pb(l)=(sg2(l)*pdp+psg1(l))*0.01     ! pb from pa to mb
              dpl(l)=(dsg2(l)*pdp+pdsg1(l))*0.01  ! dpl from pa to mb
              tm(l)=t (i,j,l)
              qm(l)=q (i,j,l)
              cm(l)=cw(i,j,l)
              oz(l)=0.0
              or(l)=0.0
              om(l)=0.0
            enddo
            pb(lm+1)=(pdp+pt)*0.01          ! pb from pa to mb
            apes=(100000.0/(pdp+pt))**0.2858964143
            ts=ths(i,j)/apes
            ta=th02(i,j)/apes
!-------------calculate random overlap clouds (after slingo)------------
            do l=lhi,lm
              tpl=tm(l)
              qsat=pq0/(sgml2(l)*pdp+psgml1(l)) &
                  *zjexp(a2*(tpl-a3)/(tpl-a4))
              rh=qm(l)/qsat
              rh=min(rh,1.0)
              rh=max(rh,rhcr)
              or(l)=(5.0*(rh-rhcr))**2
!zjtest              or(l)=((rh-rhcr)/(1.-rhcr))**2 !zjtest
              if(or(l).lt.crmin) then 
                or(l)=0.0
              endif
              if(cm(l).gt.epsq) or(l)=1.
            enddo
!------------convective clouds------------------------------------------
            if(ltop(i,j).lt.lbot(i,j)) then
              ccvr=1.-cldefi(i,j)*0.4
              do l=ltop(i,j),lbot(i,j)
                om(l)=ccvr
              enddo
            endif
!------------evaluate longwave radiative effects------------------------
            if(long) then
              call lwrad &
              (pm,pb,dpl,tm,ts,ta,qm,oz,tndl,rnet,ulwtop,dlwbot)
!-----------------------------------------------------------------------
              rlwin(i,j)=dlwbot
              do l=1,lm
                tendl(i,j,l)=tndl(l)
              enddo
            endif
!------------evaluate shortwave radiative effects-----------------------
            if(shrt) then
              czen(i,j)=max(min(czen(i,j),1.0),0.0)
              if(czen(i,j).gt.0.000001) then
!
!*** albedo modified by snow, k. mitchell 03/96
                sno(i,j)=max(sno(i,j),0.)
!                snofac=min(sno(i,j)/0.02, 1.0)
!                alb=albedo(i,j) &
!                   +(1.0-vegfrc(i,j))*snofac*(snoalb-albedo(i,j))
!                
                 if(sno(i,j).lt.0.00001) then
                   albedo(i,j)=albase(i,j)
                 else
                   albedo(i,j)=albase(i,j)*vegfrc(i,j) &
                              +snoalb(i,j)*(1.-vegfrc(i,j))
                endif 
!
                call swrad &
                (sc,albedo(i,j),czen(i,j),pb,dpl,tm,qm,oz,or,om &
                ,swinc,dswl,dswo,ssl,sso,tnds)
                do l=1,lm
                  tends(i,j,l)=tnds(l)
                enddo
                  rswin(i,j)=sso
              else
                do l=1,lm
                  tends(i,j,l)=0.0
                enddo
                rswin(i,j)=0.0
              endif
            endif
!-----------------------------------------------------------------------
          enddo
        enddo
!-----------------------------------------------------------------------
      endif
!------------direct radiative heating-----------------------------------
      do l=1,lm
        do j=2,jm-1
          do i=2,im-1
            t(i,j,l)=(-tendl(i,j,l)+tends(i,j,l))*dt/(24.*3600.) &
                    +t(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
                        endsubroutine radtn
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++                            
                        subroutine zenith &
(ids,ide,jds,jde,ihrst,idat &
,times,dayi,hour &
,glon,glat &
,czen)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++                            
implicit none
!----------------------------------------------------------------------
real,parameter:: &
 gstc1=24110.54841 &         !
,gstc2=8640184.812866 &      !
,gstc3=9.3104e-2 &           !
,gstc4=-6.2e-6 &             !
,pi=3.1415926 &              !
,pi2=2.*pi &                 !
,pih=0.5*pi &                !
,deg2rd=3.1415926/180. &     !     
,obliq=23.440*deg2rd &       !
,zerojd=2451545.0

integer,intent(in):: &
 ids &                       !
,ide &                       !
,jds &                       !
,jde &                       !
,ihrst                       !

integer,intent(in),dimension(1:3):: &
 idat

real,intent(in):: &
 times                       !

real,dimension(ids:ide,jds:jde),intent(in):: &
 glat &                      !
,glon                        !

real,intent(out):: &
 hour &                      !
,dayi

real,dimension(ids:ide,jds:jde),intent(out):: &
 czen                        !
!-----------------------------------------------------------------------
      logical :: leap
      integer :: kmnth,knt,idifyr,j,i
      real    :: day,yfctr,addday,startyr,datjul,difjd,slonm,   &
                 anom,slon,dec,ra,datj0,tu,stim0,sidtim,hrang
      real    :: hrlcl,sinalt
!-----------------------------------------------------------------------
      integer :: month (12)
!-----------------------------------------------------------------------
      data month/31,28,31,30,31,30,31,31,30,31,30,31/
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++                            
!     save month
      day=0.
      if(mod(idat(3),4).eq.0.and.idat(3).ne.2000) then
        month(2)=29
        leap=.true.
      else
        leap=.false.
      endif
      if(idat(2).gt.1)then
        kmnth=idat(2)-1
        do knt=1,kmnth
          day=day+real(month(knt))
        enddo
      endif
!***
!***  calculate exact number of days from beginning of year to
!***  forecast time of interest
!***
      day=day+real(idat(1)-1)+(real(ihrst)+times/3600.)/24.
      dayi=real(int(day)+1)
      hour=(day-dayi+1.)*24.
      yfctr=2000.-idat(3)
!-----------------------------------------------------------------------
!***
!***  find celestial longitude of the sun then the solar declination and
!***  right ascension.
!***
!-----------------------------------------------------------------------
      idifyr=idat(3)-2000
!***
!***  find julian date of start of the relevant year
!***  adding in leap days as needed
!***
      if(idifyr.lt.0)then
        addday=real(idifyr/4)
      else
        addday=real((idifyr+3)/4)
      endif
      startyr=zerojd+idifyr*365.+addday-0.5
!***
!***  the julian date of the time in question
!***
      datjul=startyr+day
!
!***  difference of actual julian date from julian date
!***  at 00h 1 january 2000
!
      difjd=datjul-zerojd
!
!***  mean geometric longitude of the sun
!
      slonm=(280.460+0.9856474*difjd)*deg2rd+yfctr*pi2
!
!***  the mean anomoly
!
      anom=(357.528+0.9856003*difjd)*deg2rd
!
!***  apparent geometric longitude of the sun
!
      slon=slonm+(1.915*sin(anom)+0.020*sin(2.*anom))*deg2rd
      if(slon.gt.pi2)slon=slon-pi2
!
!***  declination and right ascension
!
      dec=asin(sin(slon)*sin(obliq))
      ra=acos(cos(slon)/cos(dec))
      if(slon.gt.pi)ra=pi2-ra
!***
!***  find the greenwich sidereal time then the local solar
!***  hour angle.
!***
      datj0=startyr+dayi-1.
      tu=(datj0-2451545.)/36525.
      stim0=gstc1+tu*(gstc2+gstc3*tu+gstc4*tu*tu)
      sidtim=stim0/3600.+yfctr*24.+1.00273791*hour
      sidtim=sidtim*15.*deg2rd
      if(sidtim.lt.0.)sidtim=sidtim+pi2
      if(sidtim.gt.pi2)sidtim=sidtim-pi2
      hrang=sidtim-ra
!
      do j=jds,jde
        do i=ids,ide
          hrlcl=hrang+glon(i,j)+pi2
!***
!***  the zenith angle is the complement of the altitude thus the
!***  cosine of the zenith angle equals the sine of the altitude.
!***
          sinalt=sin(dec)*sin(glat(i,j)) &
                +cos(dec)*cos(hrlcl)*cos(glat(i,j))
          if(sinalt.lt.0.) sinalt=0.
          czen(i,j)=sinalt
        enddo
      enddo
!***
!***  if the forecast is in a different year than the start time,
!***  reset dayi to the proper day of the new year (it must not be
!***  reset before the solar zenith angle is computed).
!***
      if(dayi.gt.365.)then
        if(.not.leap)then
          dayi=dayi-365.
        elseif(leap.and.dayi.gt.366.)then
          dayi=dayi-366.
        endif
      endif
!-----------------------------------------------------------------------
                        endsubroutine zenith
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine lwrad(pm,pb,dpl0,tm,ts,ta,qm,oz, &
                       atlx,rnet,ulwtop,dlwbot)
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
!-----------------------------------------------------------------------
      parameter &
      (np=lm+2,npp1=np+1 &
      ,fac=3.6*24./10030.*.98)
!zjtest      ,fac=8.441874377)
!-----------------------------------------------------------------------
      dimension &
       pm(lm), pb(np-1), dpl0(lm) &
      ,tm (lm), qm(lm), oz(lm) &
      ,x2  (np-1),  x4 (np-1) &
      ,atlx(np),fe(np),coolr(np)
!
      common/var/ dpi(np),pai(np),tai(np) &
                 ,rawi(np),ozai(np)
      common/press/pu(np),dp0(np)
      common/temp/tui(npp1)
      common/flux/fd(np),fu(np)
!
!-----------------------------------------------------------------------
!
      do l=1,lm
        oz(l)=max(oz(l),0.1e-12)
      enddo
!
      do l=3,np
        pai(l)=pm(l-2)
        tai(l)=tm(l-2)
        dp0(l)=dpl0(l-2)
        dpi(l)=dp0(l)*1.02
        rawi(l)=qm(l-2)*dpi(l)
        ozai(l)=oz(l-2)*dpi(l)
      enddo
!
      pu(1)=0.
!
      do l=2,np
        pu(l)=pb(l-1)
      enddo
!
      do l=3,np-1
        x2(l)=log(pu(l)/pai(l))
        x4(l)=log(pai(l+1)/pai(l))
        tui(l)=tai(l)+x2(l)/x4(l)*(tai(l+1)-tai(l))
      enddo
!
      tui(1)=tai(3)
      tui(2)=tai(3)
      tui(np)=ta
      tui(npp1)=ts
!
      dp0(2)=(pu(2)-pu(1))
      dpi(2)=dp0(2)*1.02
!
      dp0(1)=0.
      dpi(1)=0.
      pai(1)=0.
      pai(2)=0.5*pu(2)
!
      do l=1,2
        tai(l)=tai(3)
        rawi(l)=rawi(3)
        ozai(l)=ozai(3)
      enddo
!
 110  call setup
!
      do ip=1,np
        fe(ip)=fd(ip)-fu(ip)
      enddo
!
      ulwtop=fu(1)*0.001
      dlwbot=fd(np)*0.001
      rnet=-fe(np)*0.001
!
      do ip=2,np
        coolr(ip)=(fe(ip)-fe(ip-1))*fac/dp0(ip)
      enddo
!
      do j=1,lm
        atlx(j)=coolr(j+2)
      enddo
!
                        endsubroutine lwrad
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine setup
!-----------------------------------------------------------------------
!
!**********   input parameters   ***********
!
!     lm.......number of layers
!     ptop.....model top in mb
!     pl.......layer middle pressure in mb
!     pl20.....layer bottom pressure in mb
!     tl.......layer middle temperature in kelvin
!     tg.......surface temperature in kelvin
!     ql.......water vapor mixing ratio in g/g
!     o3l......ozone mixing ratio in g/g
!     cld......random overlap cloud fraction
!     clu......maximum overlap cloud fraction
!
!**********   output parameters   **********
!
!     atlx.....layer cooling rate in celsius/day
!     rs.......net upward flux at surface in w/m**2
!     ulwtop...upward flux at top of atmosphere in w/m**2
!     dlwbot...downward flux at surface in w/m**2
!
!
!                        ip                             k
!            level        1   --- pu , tui ---------    1
!
!            layer        2   - - tai , pai - - - -     1
!
!            level        2   ----------------------    2
!
!            layer        3   - - - - - - - - - - -     2
!                               .
!                               .
!                               .
!            level     np-1   -----------------------  kbm1
!
!            layer       np   - - - - - - - - - - - -  kbm1
!
!            level       np   -----------------------    kb
!
!                             ************************
!            level       npp1 ////////////////////////   ground
!                                                       sea surface
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
!-----------------------------------------------------------------------
      parameter &
      (np=lm+2)
      parameter &
      (r630=1./630.)
!
      common/var/ dpi(np),pai(np),tai(np) &
                 ,rawi(np),ozai(np)
      common/press/pu(np),dp0(np)
      common/sh2o/ubar(np),vbar(np),wbar(np),ubarm(np), &
                  vbarm(np),wbarm(np)
      common/sco2o3/pscalv(np),pscalw(np),u(np),v(np), &
                    w(np),pscamv(np),pscamw(np),um(np), &
                    vm(np),wm(np)
!
      dimension pre(2),pr(2)
      dimension sz1(np),sz2(np),xx(np)
      dimension rv(np),rw(np),pm(np),pmv(np),pmw(np)
!-------------------------------------------------------------------
      data pre/275.,550./,pr/30.,300./,pp/1./
      save pre,pr,pp
!-------------------------------------------------------------------
!
!********** compute scaled water vapor amounts*********
!
      ubar(1)=0.
      vbar(1)=0.0
      wbar(1)=0.0
      rpre1=1./pre(1)
      rpre2=1./pre(2)
      do ip=2,np
        sz1(ip)=zjexp(0.005*(tai(ip)-225.0))
        sz2(ip)=zjexp(0.016*(tai(ip)-256.0))
        vbar(ip)=vbar(ip-1)+rawi(ip)*sz1(ip)* &
                     (pai(ip)*rpre1)
        wbar(ip)=wbar(ip-1)+rawi(ip)*sz2(ip)* &
                     (pai(ip)*rpre2)
        xx(ip)=(pai(ip)*r630)*rawi(ip)*rawi(ip)/dpi(ip)
        xx(ip)=xx(ip)*zjexp(1800./tai(ip)-6.0811)
        ubar(ip)=ubar(ip-1)+xx(ip)
        ubarm(ip)=0.5*(ubar(ip)+ubar(ip-1))
        vbarm(ip)=0.5*(vbar(ip)+vbar(ip-1))
        wbarm(ip)=0.5*(wbar(ip)+wbar(ip-1))
      enddo
!
!********* compute scaled co2 amounts **********
!
      pscalv(1)=0.
      pscalw(1)=0.
      rpr1=1./pr(1)
      rpr2=1./pr(2)
      do ip=2,np
        rv(ip)=zjexp(0.0089*(tai(ip)-240.0))
        rw(ip)=zjexp(0.025*(tai(ip)-240.0))
        pm(ip)=max(pai(ip),pp)
        pmv(ip)=(pm(ip)*rpr1)**0.85*rv(ip)
        pmw(ip)=sqrt(pm(ip)*rpr2)*rw(ip)
!
!********** 0.26 is for co2=330 ppmv ***********
!********** 0.30 is for co2=380 ppmv ***********
!
!zjtest        pscalv(ip)=dp0(ip)*pmv(ip)*0.26+pscalv(ip-1)
!zjtest        pscalw(ip)=dp0(ip)*pmw(ip)*0.26+pscalw(ip-1)
!
        pscalv(ip)=dp0(ip)*pmv(ip)*0.27+pscalv(ip-1)
        pscalw(ip)=dp0(ip)*pmw(ip)*0.27+pscalw(ip-1)
        pscamv(ip)=0.5*(pscalv(ip)+pscalv(ip-1))
        pscamw(ip)=0.5*(pscalw(ip)+pscalw(ip-1))
      enddo
!
!********** compute scaled ozone amounts ********
!
      u(1)=0.
      v(1)=0.
      w(1)=0.
      do ip=2,np
        u(ip)=pai(ip)*ozai(ip)+u(ip-1)
        w(ip)=tai(ip)*ozai(ip)+w(ip-1)
        v(ip)=ozai(ip)+v(ip-1)
        um(ip)=0.5*(u(ip)+u(ip-1))
        vm(ip)=0.5*(v(ip)+v(ip-1))
        wm(ip)=0.5*(w(ip)+w(ip-1))
      enddo
!
      call irflux
!
                        endsubroutine setup
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine irflux
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
      parameter (np=lm+2,npp1=np+1)
!-----------------------------------------------------------------------
      dimension sv(npp1),sw(npp1),tui1(npp1),tui2(npp1) &
               ,bai(npp1),bwi(npp1) &
               ,x1(npp1),x2(npp1),x3(npp1),x4(npp1) &
               ,x5(npp1),x6(npp1),x7(npp1),x8(npp1) &
               ,fh(npp1),dh(npp1),f1(npp1),it(npp1) &
               ,a(npp1),b(npp1),c(npp1),wv(npp1) &
               ,yv(npp1),zv(npp1),ww(npp1),yw(npp1) &
               ,d(npp1),e(npp1),f(npp1),g(npp1) &
               ,sh(npp1),tauf(npp1),txuf(npp1),zw(npp1)
      dimension sg(np,np),sht(np),shu(np)
      dimension bh2ov(25),bh2ow(25),blkco2(25),blkwin(25)
!
      common/sh2o/ubar(np),vbar(np),wbar(np),ubarm(np), &
                   vbarm(np),wbarm(np)
      common/sco2o3/pscalv(np),pscalw(np),u(np),v(np), &
                    w(np),pscamv(np),pscamw(np),um(np), &
                    vm(np),wm(np)
      common/flux/fd(np),fu(np)
      common/temp/tui(npp1)
      common/csc/css(np),ccu(np),ch(np),cc(np),cs(np,np)
      parameter (temp1=190.,dtt=5.,rdtt=1./dtt,nt=25)
!
!******* h2o planck function **********
!
      data bh2ov/22661.,24086.,25575.,27135.,28775.,30506.,32339., &
          34286.,36361.,38578.,40954.,43505.,46248.,49203.,52388., &
          55824.,59532.,63533.,67849.,72502.,77516.,82913.,88717., &
          94952.,101640./
      data bh2ow/30708.,34405.,38417.,42763.,47461.,52529.,57985., &
          63850.,70141.,76880.,84088.,91784.,99990.,108726.,118016., &
          127881.,138344.,149429.,161160.,173561.,186659.,200478., &
          215046.,230390.,246539./
!
!******* window planck function **********
!
      data blkwin/1944.,2377.,2877.,3450.,4102.,4838.,5664.,6585., &
                   7606.,8733.,9969.,11320.,12788.,14380.,16097., &
                   17944.,19923.,22038.,24292.,26685.,29221.,31902., &
                   34729.,37703.,40825./
!
!******* co2 planck function *********
!
      data blkco2/18606.,21145.,23884.,26826.,29973.,33325.,36883., &
                  40647.,44617.,48792.,53170.,57750.,62530.,67509., &
                  72683.,78050.,83609.,89354.,95285.,101397.,107688., &
                  114155.,120794.,127601.,134574./

      save bh2ov,bh2ow,blkco2,blkwin

      do ip=1,np
        sht(ip)=0.
        shu(ip)=0.
      enddo
      do ip=1,np
        do ix=1,np
          sg(ix,ip)=0.
        enddo
      enddo
      do ip=1,npp1
        fh(ip)=(tui(ip)-temp1)*rdtt+1.5
        it(ip)=fh(ip)
        it(ip)=max0(min0(it(ip),nt-1),1)
        f1(ip)=float(it(ip)-1)
        dh(ip)=tui(ip)-(temp1+f1(ip)*dtt)
        sv(ip)=bh2ov(it(ip)) &
              +(bh2ov(it(ip)+1)-bh2ov(it(ip)))*dh(ip)*rdtt
        sw(ip)=bh2ow(it(ip)) &
              +(bh2ow(it(ip)+1)-bh2ow(it(ip)))*dh(ip)*rdtt
        bai(ip)=blkco2(it(ip)) &
               +(blkco2(it(ip)+1)-blkco2(it(ip)))*dh(ip)*rdtt
        bwi(ip)=blkwin(it(ip)) &
               +(blkwin(it(ip)+1)-blkwin(it(ip)))*dh(ip)*rdtt
      enddo
      do ip=1,npp1
        sh(ip)=sv(ip)+sw(ip)+bai(ip)+bwi(ip)
        tui1(ip)=tui(ip)-250.
        tui2(ip)=tui1(ip)*tui1(ip)
      enddo
!
!*****************              ****************
!*****************      h2o     ****************
!*****************              ****************
!
      do ip=2,np
        a(ip-1)=abs(ubar(ip))
        b(ip-1)=abs(vbar(ip))
        c(ip-1)=abs(wbar(ip))
      enddo
      jp=np-1
!
!     call table(jp,a,b,c,wv,yv,zv,ww,yw,zw)
      call gfunc(jp,a,b,c,wv,yv,zv,ww,yw,zw)
!
      do ip=2,np
        x1(ip)=wv(ip-1)*sv(1)
        x2(ip)=1.+yv(ip-1)*tui1(1)+zv(ip-1)*tui2(1)
        x3(ip)=ww(ip-1)*sw(1)
        x4(ip)=1.+yw(ip-1)*tui1(1)+zw(ip-1)*tui2(1)
        sht(ip)=x1(ip)*x2(ip)+x3(ip)*x4(ip)
      enddo
      do ip=1,np-1
        a(ip)=abs(ubar(np)-ubar(ip))
        b(ip)=abs(vbar(np)-vbar(ip))
        c(ip)=abs(wbar(np)-wbar(ip))
      enddo
      jp=np-1
!
!     call table(jp,a,b,c,wv,yv,zv,ww,yw,zw)
      call gfunc(jp,a,b,c,wv,yv,zv,ww,yw,zw)
!
      do ip=1,np-1
        x1(ip)=wv(ip)*sv(npp1)
        x2(ip)=1.+yv(ip)*tui1(npp1)+zv(ip)*tui2(npp1)
        x3(ip)=ww(ip)*sw(npp1)
        x4(ip)=1.+yw(ip)*tui1(npp1)+zw(ip)*tui2(npp1)
        x5(ip)=wv(ip)*sv(np)
        x6(ip)=1.+yv(ip)*tui1(np)+zv(ip)*tui2(np)
        x7(ip)=ww(ip)*sw(np)
        x8(ip)=1.+yw(ip)*tui1(np)+zw(ip)*tui2(np)
        shu(ip)=x1(ip)*x2(ip)+x3(ip)*x4(ip) &
               -x5(ip)*x6(ip)-x7(ip)*x8(ip)
      enddo
!
!*********downward flux sg(ix,ip)************
!
      do ip=2,np
        do ix=2,ip
          a(ix-1)=abs(ubar(ip)-ubarm(ix))
          b(ix-1)=abs(vbar(ip)-vbarm(ix))
          c(ix-1)=abs(wbar(ip)-wbarm(ix))
        enddo
        jp=ip-1
!
!     call table(jp,a,b,c,wv,yv,zv,ww,yw,zw)
      call gfunc(jp,a,b,c,wv,yv,zv,ww,yw,zw)
!
        do ix=2,ip
          x1(ix-1)=wv(ix-1)*sv(ix-1)
          x2(ix-1)=1.+yv(ix-1)*tui1(ix-1)+zv(ix-1)*tui2(ix-1)
          x3(ix-1)=ww(ix-1)*sw(ix-1)
          x4(ix-1)=1.+yw(ix-1)*tui1(ix-1)+zw(ix-1)*tui2(ix-1)
          x5(ix-1)=wv(ix-1)*sv(ix)
          x6(ix-1)=1.+yv(ix-1)*tui1(ix)+zv(ix-1)*tui2(ix)
          x7(ix-1)=ww(ix-1)*sw(ix)
          x8(ix-1)=1.+yw(ix-1)*tui1(ix)+zw(ix-1)*tui2(ix)
          sg(ix-1,ip)=x1(ix-1)*x2(ix-1)+x3(ix-1)*x4(ix-1) &
                     -x5(ix-1)*x6(ix-1)-x7(ix-1)*x8(ix-1)
        enddo
      enddo
!
!*********upward flux sg(ix,ip)*********
!
      do ip=1,np-1
        ip1=np-ip
        do ix=1,ip1
          a(ix)=abs(ubar(ip)-ubarm(ix+ip))
          b(ix)=abs(vbar(ip)-vbarm(ix+ip))
          c(ix)=abs(wbar(ip)-wbarm(ix+ip))
        enddo
        jp=ip1
!
!     call table(jp,a,b,c,wv,yv,zv,ww,yw,zw)
      call gfunc(jp,a,b,c,wv,yv,zv,ww,yw,zw)
!
        do ix=1,ip1
          x1(ix)=wv(ix)*sv(ix+ip)
          x2(ix)=1.+yv(ix)*tui1(ix+ip)+zv(ix)*tui2(ix+ip)
          x3(ix)=ww(ix)*sw(ix+ip)
          x4(ix)=1.+yw(ix)*tui1(ix+ip)+zw(ix)*tui2(ix+ip)
          x5(ix)=wv(ix)*sv(ix+ip-1)
          x6(ix)=1.+yv(ix)*tui1(ix+ip-1)+zv(ix)*tui2(ix+ip-1)
          x7(ix)=ww(ix)*sw(ix+ip-1)
          x8(ix)=1.+yw(ix)*tui1(ix+ip-1)+zw(ix)*tui2(ix+ip-1)
          sg(ix+ip,ip)=x1(ix)*x2(ix)+x3(ix)*x4(ix) &
                      -x5(ix)*x6(ix)-x7(ix)*x8(ix)
        enddo
      enddo
!
!***********                ***********
!***********   co2 and o3   ***********
!***********                ***********
!
      do ip=2,np
        a(ip-1)=abs(pscalv(ip))
        b(ip-1)=abs(pscalw(ip))
        c(ip-1)=abs(wbar(ip))
        d(ip-1)=abs(ubar(ip))
        e(ip-1)=abs(v(ip))
        f(ip-1)=abs(u(ip))
        g(ip-1)=abs(w(ip))
      enddo
      jp=np-1
!
      call trco3(jp,a,b,c,d,e,f,g,tauf,txuf)
!
      do ip=2,np
        sht(ip)=sht(ip)+bai(1)*tauf(ip-1) &
                       +bwi(1)*txuf(ip-1)
      enddo
      do ip=1,np-1
        a(ip)=abs(pscalv(np)-pscalv(ip))
        b(ip)=abs(pscalw(np)-pscalw(ip))
        c(ip)=abs(wbar(np)-wbar(ip))
        d(ip)=abs(ubar(np)-ubar(ip))
        e(ip)=abs(v(np)-v(ip))
        f(ip)=abs(u(np)-u(ip))
        g(ip)=abs(w(np)-w(ip))
      enddo
      jp=np-1
!
      call trco3(jp,a,b,c,d,e,f,g,tauf,txuf)
!
      do ip=1,np-1
        shu(ip)=shu(ip)+(bai(npp1)-bai(np))*tauf(ip) &
                       +(bwi(npp1)-bwi(np))*txuf(ip)
      enddo
!
!*********downward flux sg(ix,ip)************
!
      do ip=2,np
        do ix=2,ip
          a(ix-1)=abs(pscalv(ip)-pscamv(ix))
          b(ix-1)=abs(pscalw(ip)-pscamw(ix))
          c(ix-1)=abs(wbar(ip)-wbarm(ix))
          d(ix-1)=abs(ubar(ip)-ubarm(ix))
          e(ix-1)=abs(v(ip)-vm(ix))
          f(ix-1)=abs(u(ip)-um(ix))
          g(ix-1)=abs(w(ip)-wm(ix))
        enddo
        jp=ip-1
!
      call trco3(jp,a,b,c,d,e,f,g,tauf,txuf)
!
        do ix=2,ip
          sg(ix-1,ip)=sg(ix-1,ip)+(bai(ix-1)-bai(ix)) &
                      *tauf(ix-1)+(bwi(ix-1)-bwi(ix)) &
                      *txuf(ix-1)
        enddo
      enddo
!
!*********upward flux sg(ix,ip)*********
!
      do ip=1,np-1
        ip1=np-ip
        do ix=1,ip1
          a(ix)=abs(pscalv(ip)-pscamv(ix+ip))
          b(ix)=abs(pscalw(ip)-pscamw(ix+ip))
          c(ix)=abs(wbar(ip)-wbarm(ix+ip))
          d(ix)=abs(ubar(ip)-ubarm(ix+ip))
          e(ix)=abs(v(ip)-vm(ix+ip))
          f(ix)=abs(u(ip)-um(ix+ip))
          g(ix)=abs(w(ip)-wm(ix+ip))
        enddo
        jp=ip1
!
      call trco3(jp,a,b,c,d,e,f,g,tauf,txuf)
!
        do ix=1,ip1
          sg(ix+ip,ip)=sg(ix+ip,ip) &
                      +(bai(ix+ip)-bai(ix+ip-1))*tauf(ix) &
                      +(bwi(ix+ip)-bwi(ix+ip-1))*txuf(ix)
        enddo
      enddo
!
      call clouds
!
      do ip=1,np
        fu(ip)=0.
        fd(ip)=0.
      enddo
!
!***********  compute downward fluxes **********
!
      do ip=2,np
        summ=0.
        ipm1=ip-1
        do ix=1,ipm1
          summ=summ+sg(ix,ip)
!          summ=summ+sg(ix,ip)*cs(ix,ip)
        enddo
        fd(ip)=summ-sht(ip)          +sh(ip)
!        fd(ip)=summ-sht(ip)*ch(ip)+sh(ip)
      enddo
!
!********** compute upward fluxes ************
!
      do ip=1,np-1
        summ=0.
        ipp1=ip+1
        do ix=ipp1,np
!          summ=summ+sg(ix,ip)
          summ=summ+sg(ix,ip)*cs(ix,ip)
        enddo
!        fu(ip)=summ+shu(ip)       +sh(ip)
        fu(ip)=summ+shu(ip)*cc(ip)+sh(ip)
      enddo
      fu(np)=sh(npp1)
!
                        endsubroutine irflux
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine gfunc(n,a,b,c,wv,yv,zv,ww,yw,zw)
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
      parameter(np=lm+2,npp1=np+1)
!-----------------------------------------------------------------------
      dimension a(npp1),b(npp1),c(npp1),wv(npp1), &
        yv(npp1), zv(npp1),ww(npp1),yw(npp1),zw(npp1)
      dimension a1(np),a2(np),a3(np),b1(np),b2(np), &
        b3(np),b4(np),c1(np),c2(np),c3(np), &
        c4(np)
!--------------------------------------------------------------------
          do j=1,n
      a1(j)=(1.0+32.2095*a(j))/(1.0+52.85*a(j))
      a2(j)=(0.534874+199.0*a(j)-1990.63*a(j)*a(j))/(1.0+ &
               333.244*a(j))
      a3(j)=(1.0+74.144*a(j))/(0.43368+24.7442*a(j))
          enddo
!
          do j=1,n
      sqrtc=sqrt(c(j))
      ww(j)=(a1(j)+a2(j)*sqrtc) &
           /(1.0+a3(j)*sqrtc)
      ww(j)=max(ww(j),0.0)
      wv(j)=1.0/(1.0+9.22411*sqrt(b(j))+33.1236*b(j)+176.396 &
              *b(j)*b(j))
      wv(j)=max(wv(j),0.0)
          enddo
!
          do j=1,n
      a(j)=min(a(j),0.06)
      b(j)=min(b(j),2.0)
      c(j)=min(c(j),8.0)
          enddo
!
          do j=1,n
      sqrtb=sqrt(b(j))
      yv(j)=0.1*(0.0851069*sqrtb+0.323105*b(j)-0.187096*b(j)*sqrtb)
      zv(j)=0.001*(0.239186*b(j)-0.0922289*b(j)*sqrtb &
           -0.0167413*b(j)*b(j))
          enddo
!
          do j=1,n
      sqrtc=sqrt(c(j))
      asq=a(j)*a(j)
      b1(j)=(5.6383e-4+1.05173*a(j)-39.0722*asq)/(1.0+202.357*a(j))
      b2(j)=(0.0779555+4.40720*a(j)+3.15851*asq)/(1.0+40.2298*a(j))
      b3(j)=(-0.0381305-3.63684*a(j)+7.98951*asq)/(1.0+62.5692*a(j))
      b4(j)=(0.00621039+0.710061*a(j)-2.85241*asq)/(1.0+70.2912*a(j))
      yw(j)=0.1*(b1(j)+b2(j)*sqrtc+b3(j)*c(j)+b4(j)*c(j)*sqrtc)
      c1(j)=(-2.99542e-4+0.238219*a(j)+0.519264*asq)/(1.0+10.7775*a(j))
      c2(j)=(-0.0291325-2.30007*a(j)+10.9460*asq)/(1.0+63.519*a(j))
      c3(j)=(0.0143812+1.80265*a(j)-10.1311*asq)/(1.0+98.4758*a(j))
      c4(j)=(-0.00239016-0.371427*a(j)+2.35443*asq)/(1.0+120.228*a(j))
      zw(j)=0.001*(c1(j)+c2(j)*sqrtc+c3(j)*c(j)+c4(j)*c(j)*sqrtc)
          enddo
!
      return
                        endsubroutine gfunc
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine clouds
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
      parameter(np=lm+2)
!-----------------------------------------------------------------------
      common/csc/css(np),ccu(np),ch(np),cc(np),cs(np,np)
      dimension ct(np),cu(np),cr(np,np)
      do ip=1,np
      ch(ip)=1.0
      cc(ip)=1.0
      ct(ip)=1.0
      cu(ip)=1.0
      enddo
      do ip=1,np
      do ix=1,np
      cs(ix,ip)=1.0
      cr(ix,ip)=1.0
      enddo
      enddo
      do ip=2,np
      ct(ip)=1.-ccu(ip-1)
      ct(ip)=min(ct(ip-1),ct(ip))
      ch(ip)=ch(ip-1)*(1.-css(ip-1))
      enddo
      do ip=2,np
      ch(ip)=ch(ip)*ct(ip)
      enddo
      do i=1,np-1
      ip=np-i
      cu(ip)=1.-ccu(ip)
      cu(ip)=min(cu(ip+1),cu(ip))
      cc(ip)=cc(ip+1)*(1.-css(ip))
      enddo
      do i=1,np-1
      ip=np-i
      cc(ip)=cc(ip)*cu(ip)
      enddo
      do ip=2,np
      ipm1=ip-1
      do ix=1,ipm1
      cr(ix,ip)=1.-ccu(ip-1)
      cr(ix,ip)=min(cr(ix,ip-1),cr(ix,ip))
      cs(ix,ip)=cs(ix,ip-1)*(1.-css(ip-1))
      enddo
      enddo
      do ip=2,np
      ipm1=ip-1
      do ix=1,ipm1
      cs(ix,ip)=cs(ix,ip)*cr(ix,ip)
      enddo
      enddo
      do i=1,np-1
      ip=np-i
      ipp1=ip+1
      do ix=ipp1,np
      cr(ix,ip)=1.-ccu(ip)
      cr(ix,ip)=min(cr(ix,ip+1),cr(ix,ip))
      cs(ix,ip)=cs(ix,ip+1)*(1.-css(ip))
      enddo
      enddo
      do i=1,np-1
      ip=np-i
      ipp1=ip+1
      do ix=ipp1,np
      cs(ix,ip)=cs(ix,ip)*cr(ix,ip)
      enddo
      enddo
                        endsubroutine clouds
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine trco3(n,a,b,c,d,e,f,g,tauf,txuf)
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
      parameter(np=lm+2,npp1=np+1)
!-----------------------------------------------------------------------
      dimension a(npp1),b(npp1),c(npp1)
      dimension d(npp1),e(npp1),f(npp1)
      dimension g(npp1),tauf(npp1),txuf(npp1)
      dimension xv(np),xw(np),xz(np),dx(np),dy(np)
      logical lozone
      data lozone/.true./
      save lozone
!-----------------------------------------------------------------------
          do j=1,n
      xv(j)=3.1*a(j)/(1.+15.1*a(j)**0.56)
      xw(j)=0.04*b(j)/(1.+0.9*b(j)**0.57)
          if(c(j).ne.0.) then
      dx(j)=6.7*c(j)/(1.+16.*c(j)**0.6)
          else
      dx(j)=0.
          endif
          if(d(j).ne.0.) then
      dy(j)=27.*d(j)**0.83
          else
      dy(j)=0.
          endif
      xv(j)=xv(j)+dx(j)+dy(j)
      xw(j)=xw(j)+dx(j)+dy(j)
      xv(j)=min(xv(j),30.)
      xw(j)=min(xw(j),30.)
      tauf(j)=0.384615384*zjexp(-xv(j))+0.615384615*zjexp(-xw(j))
      dy(j)=9.79*d(j)
      dy(j)=min(dy(j),30.)
      if( lozone) then
        f(j)=f(j)/(e(j)*1013.25)
        g(j)=6.5e-5*sqrt(g(j)/e(j))
        xz(j)=4.*345.28*e(j)/(3.14159265*(0.28*f(j)+g(j)))
        xz(j)=-15.70796*(0.28*f(j)+g(j))*(sqrt(1.+xz(j))-1.)
        xz(j)=1.0-81.21/120.0*(1.0-zjexp(xz(j)))
        txuf(j)=xz(j)*zjexp(-dy(j))
      else
        txuf(j)=zjexp(-dy(j))
      endif
          enddo
                        endsubroutine trco3
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine swrad &
      (s0,xsurf,cosz,pb,dpl0,tm,qm,oz,or,om &
      ,swinc,dswl,dswo,ssl,sso,htr)
!-------------------------------------------------------------------
!     lm.......number of layers
!     ptop.....pressure at top of model
!     xsurfv...diffuse surface albedo from 0.0 to 0.7 microns
!     xsurfn...diffuse surface albedo from 0.7 to 4.0 microns
!     cosz.....cosine of solar zenith angle
!     s0.......solar constant
!     swinc....solar input at top of atmosphere
!     pl20.....pressure at bottom of layer
!     tl.......layer temperature
!     ql.......layer water vapor mixing ratio in g/g
!     o3l......layer ozone mixing ratio in g/g
!     cld......supersaturation cloud fraction
!     clu......convective cloud fraction
!     dswl.....total clear sky flux absorbed by earth-atmos system
!     dswo.....total cloudy sky flux absorbed by earth-atmos system
!     ssl......clear sky surface absorption
!     sso......cloudy sky surface absorption
!     asl......layer absorption for clear skies
!     asc......layer absorption for cloudy skies
!     htr......layer heating rates in deg/day
!-------------------------------------------------------------------
      include 'modelgrid.inc'
!-----------------------------------------------------------------------
      parameter(np=lm+2 &
      ,fac=8.441874377)
!
      dimension pb(np-1),tm(lm),qm(lm),oz(lm)
      dimension or(lm),om(lm)
      dimension dpl0(lm)
!
      dimension asl(np),asc(np),htr(np)
!
      common/prs/dpc(np)
      common/set/pu(np),ta(np),wa(np),oa(np)
      common/rfl/ag,rsurf
      common/sun/scosz,cmu,cosmag
      common/cld/css(np),ccu(np)
      common/fll/sl,al(np),dsl
      common/flc/sc,ac(np),dsc
!
      do l=2,np-1
        ta(l)=tm(l-1)
        dpc(l)=dpl0(l-1)
        wa(l)=qm(l-1)
        oa(l)=oz(l-1)
        css(l)=or(l-1)
        ccu(l)=om(l-1)
      enddo
!
      do l=2,np
        pu(l)=pb(l-1)
      enddo    
!
      pu(1)=0.
      dpc(1)=(pu(2)-pu(1))
!
      ta(1)=ta(2)
      wa(1)=wa(2)
      oa(1)=oa(2)
!
      ag=xsurf
      cmu=max(min(cosz,1.0),0.0)
      cosmag=35.0/sqrt(1224.0*cmu*cmu+1.0)
      swinc=s0*cmu
      scosz=swinc 
!
      do j=1,np/2
        wa(j)=min(wa(j),3.0e-6)
      enddo
!
      do j=1,np-1
        wa(j)=max(wa(j),1.0e-23)
        oa(j)=max(oa(j),1.0e-10)
      enddo
!
      call setsw
!
      ssl=sl
      sso=sc
      dswl=dsl
      dswo=dsc
!
      do j=1,np-1
        asl(j)=al(j)
        asc(j)=ac(j)
      enddo
!
      do j=1,lm
        htr(j)=(asc(j+1)+asl(j+1))*fac/dpc(j+1)
      enddo
!-----------------------------------------------------------------------
                         endsubroutine swrad
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                         subroutine setsw
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
      parameter(np=lm+2)
      parameter &
      (pi=3.141592653,rpi=1./pi,rd945=1./0.945687,r120=1./120.1612 &
      ,r6d55=1./6.55,r3=1./3.,r4d354=1./4.354)
!-----------------------------------------------------------------------
      common/prs/dpc(np)
      common/set/pu(np),ta(np),wa(np),oa(np)
      common/rfl/ag,rsurf
      common/sun/scosz,cmu,cosmag
      common/cld/css(np),ccu(np)
      common/wat/swil(np),swale(np)
      common/ozn/ozale(np),rlo,rlg
      common/tac/tauc(np),taut,rco,rcg
!
! ******* compute albedo of surface for direct beam *********
!
      rg=(ag-0.054313)*rd945
      rg=max(rg,0.0)
      arg=-18.0*(0.5*pi-acos(cmu))*rpi
      rsurf=rg+(1.0-rg)*zjexp(arg)
!
! ******** zero out ozone below 500 mb ***********
!
      do j=1,np-1
        if(pu(j+1).gt.500.0) oa(j)=0.0
      enddo
!
! ******** compute ozone and water vapor amounts *******
!
      ozale(1)=0.0
      swale(1)=0.0
!
      do j=2,np
        ozale(j)=ozale(j-1)+476.0*oa(j-1)*dpc(j-1)
      enddo
!
      do j=1,np-1
        swil(j)=(pu(j+1)*pu(j+1)-pu(j)*pu(j)) &
               *wa(j)/sqrt(ta(j))*r120
      enddo 
!
      do j=2,np
        swale(j)=swale(j-1)+swil(j-1)
      enddo
!
!****** compute clear sky reflectivities ********
!
!****** clear sky ozone reflection, tauray=0.85 *******
!
      e=zjexp(-0.85*cosmag)
      a=(2.0+3.0*cmu+(2.0-3.0*cmu)*e)*r6d55
      up=1.0-a
      dn=a-e
      rlo=up+(e*rsurf+dn*ag)*0.576004/(1.0-0.423996*ag)
            
      rlo=(2.0*rsurf+rlo)*r3
!
!****** clear sky ground reflection, tauray=0.118 ******
!
      e=zjexp(-0.118*cosmag)
      a=(2.0+3.0*cmu+(2.0-3.0*cmu)*e)*r4d354
      up=1.0-a
      dn=a-e
      rlg=up+(e*rsurf+dn*ag)*0.907284/(1.0-0.092716*ag)
!
      call clear
!
!******* find cloud top *********
!
      do j=1,np-1
        css(j)=max(css(j),ccu(j))
      enddo
!
      icl=np
!
      do k=1,np-1
        j=np-k
        if(css(j).gt.0.0) icl=j
      enddo
!
!******** zero out ozone below cloud top *******
!
      do j=1,np-1
        if(icl.le.j) oa(j)=0.0
      enddo
!
!********** recompute ozone amounts *********
!
      ozale(1)=0.0
!
      do j=2,np
        ozale(j)=ozale(j-1)+476.0*oa(j-1)*dpc(j-1)
      enddo
!
!******* compute cloud optical depth ********
!
      do j=1,np-1
        if((ta(j).lt.253.0).and.(ccu(j).eq.0.0)) then
          con=0.025
        else
          con=0.05
        endif
        tauc(j)=con*dpc(j)*css(j)
      enddo          
!
      taut=0.0
!
      do j=1,np-1
        taut=taut+tauc(j)
      enddo
!
!******** compute cloudy sky reflectivities *********
!
!****** cloudy sky ozone reflection, tauray=0.0, gcld=0.85 ********
!
      tl=1.0/(1.0+0.1299*taut)
      arg=0.2775*taut*cosmag
      if(arg.le.30.0) then
        e=zjexp(-arg)
      else
        e=0.0
      endif
      a=2.0+3.0*cmu+(2.0-3.0*cmu)*e
      a=a/(4.0+0.45*taut)
      up=1.0-a
      dn=a-e
      rc1=up+(e*rsurf+dn*ag)*tl/(1.0-(1.0-tl)*ag)             
!
!****** cloudy sky ozone reflection, tauray=0.85, gcld=0.85 *****
!
      tau=taut+0.85
      g=0.85*taut/tau
      tl=1.0/(1.0+0.866*(1.0-g)*tau)
      arg=(1.0-g*g)*tau*cosmag
      if(arg.le.30.0) then
        e=zjexp(-arg)
      else
        e=0.0
      endif
      a=2.0+3.0*cmu+(2.0-3.0*cmu)*e
      a=a/(4.0+3.0*(1.0-g)*tau)
      up=1.0-a
      dn=a-e
      rco=up+(e*rsurf+dn*ag)*tl/(1.0-(1.0-tl)*ag)             
!
      rco=(2.0*rc1+rco)/3.0
!
!****** cloudy sky ground reflection, tauray=0.118, gcld=0.85 *****
!
      tau=taut+0.118
      g=0.85*taut/tau
      tl=1.0/(1.0+0.866*(1.0-g)*tau)
      arg=(1.0-g*g)*tau*cosmag
      if(arg.le.30.0) then
        e=zjexp(-arg)
      else
        e=0.0
      endif
      a=2.0+3.0*cmu+(2.0-3.0*cmu)*e
      a=a/(4.0+3.0*(1.0-g)*tau)
      up=1.0-a
      dn=a-e
      rcg=up+(e*rsurf+dn*ag)*tl/(1.0-(1.0-tl)*ag)             
!
      call cloudy
!
                        endsubroutine setsw
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine clear
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
      parameter(np=lm+2)
!-----------------------------------------------------------------------
      dimension up(np),dn(np),fk(5),xk(5)
      common/rfl/ag,rsurf
      common/sun/scosz,cmu,cosmag
      common/wat/swil(np),swale(np)
      common/ozn/ozale(np),rlo,rlg
      common/fll/sl,al(np),dsl
      data fk/0.107,0.104,0.073,0.044,0.025/
      data xk/0.005,0.041,0.416,4.752,72.459/
      save fk,xk
      sqrt3=sqrt(3.0)
      sl=0.0
!
      do j=1,np
        al(j)=0.0
      enddo
!
      dn(1)=0.0
!
      do j=2,np
        w=ozale(j)*cosmag
        dn(j)=0.02118*w/(1.0+0.042*w+0.000323*w*w) &
               +1.08173*w/(1.0+138.57*w)**0.805 &
               +0.0658*w/(1.0+(103.63*w)**3)
      enddo
!
!***** compute ozone heating on the way down *******
!
      do j=1,np-1
        al(j)=(dn(j+1)-dn(j))*scosz
      enddo
!
!****  ground absorption for wavelengths under 0.9 microns  ****
!
      sl=(0.647-dn(np))*(1.0-rlg)*scosz
!
      up(np)=dn(np)
      ww=ozale(np)*(1.90+cosmag)
!
      do k=1,np-1
        j=np-k
        w=ww-1.90*ozale(j)
        up(j)=0.02118*w/(1.0+0.042*w+0.000323*w*w) &
               +1.08173*w/(1.0+138.57*w)**0.805 &
               +0.0658*w/(1.0+(103.63*w)**3)
      enddo
!
!**** compute ozone heating on the way up *********
!
      do k=1,np-1
        j=np-k
        al(j)=al(j)+(up(j)-up(j+1))*rlo*scosz
      enddo
!
      do j=1,np-1
        al(j)=max(al(j),0.0)
      enddo
!
!***** cycle over five bands *********
!
      do ik=1,5
!
        do j=1,np
          w=swale(j)*cosmag*xk(ik)
          if(w.le.30.0) then
            dn(j)=zjexp(-w)
          else
            dn(j)=0.0
          endif
        enddo   
!
!****** compute water vapor heating on the way down ********
!
        do j=1,np-1
          al(j)=al(j)+(dn(j)-dn(j+1))*scosz*fk(ik)
        enddo
!
!****  ground absorption for wavelengths over 0.9 microns  ****
!
        sl=sl+dn(np)*(1.0-rsurf)*scosz*fk(ik)
!
        up(np)=dn(np)
        ww=swale(np)*(sqrt3+cosmag)*xk(ik)
!
        do k=1,np-1
          j=np-k
          w=ww-sqrt3*swale(j)*xk(ik)
          if(w.le.30.0) then
            up(j)=zjexp(-w)
          else
            up(j)=0.0
          endif
        enddo
!
!******* compute water vapor heating on the way up ********
!
        do k=1,np-1
          j=np-k
          al(j)=al(j)+(up(j+1)-up(j))*rsurf*scosz*fk(ik)
        enddo
!
      enddo
!
      dsl=sl
!
      do j=1,np-1
        dsl=dsl+al(j)
      enddo
!
                        endsubroutine clear
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine cloudy
!-----------------------------------------------------------------------
      include 'modelgrid.inc'
      parameter(np=lm+2)
!-----------------------------------------------------------------------
      dimension up(np),dn(np),fk(5),xk(5)
      dimension tau(np),sso(np),rl(np),tl(np),arg(np)
      dimension a(np),b(np),c(np),d(np),e(np),f(np)
      dimension sol(np),e0(np),u0(np),d0(np),u(np)
      dimension d2(np),da(np),db(np),den(np),ak(np)
      dimension gam1(np),gam2(np)
      dimension alf1(np),alf2(np),aa(np),bb(np),cr(np)
      dimension cm(np),vp(np),vm(np),ul(np),dl(np)
      common/tac/tauc(np),taut,rco,rcg
      common/rfl/ag,rsurf
      common/sun/scosz,cmu,cosmag
      common/wat/swil(np),swale(np)
      common/ozn/ozale(np),rlo,rlg
      common/fll/sl,al(np),dsl
      common/flc/sc,ac(np),dsc
      data fk/0.107,0.104,0.073,0.044,0.025/
      data xk/0.005,0.041,0.416,4.752,72.459/
      save fk,xk
      sqrt3=sqrt(3.0)
      g=0.85
      gp=g/(1.0+g)
!
      sc=0.0
!
      do j=1,np
      ac(j)=0.0
      enddo
!
      dn(1)=0.0
!
      do j=2,np
      w=ozale(j)*cosmag
      dn(j)=0.02118*w/(1.0+0.042*w+0.000323*w*w) &
             +1.08173*w/(1.0+138.57*w)**0.805 &
             +0.0658*w/(1.0+(103.63*w)**3)
      enddo
!
!***** compute ozone heating on the way down *******
!
      do j=1,np-1
      ac(j)=(dn(j+1)-dn(j))*scosz
      enddo
!
!****  ground absorption for wavelengths under 0.9 microns  ****
!
      sc=(0.647-dn(np))*(1.0-rcg)*scosz
!
      up(np)=dn(np)
      ww=ozale(np)*(1.90+cosmag)
!
      do k=1,np-1
      j=np-k
      w=ww-1.90*ozale(j)
      up(j)=0.02118*w/(1.0+0.042*w+0.000323*w*w) &
             +1.08173*w/(1.0+138.57*w)**0.805 &
             +0.0658*w/(1.0+(103.63*w)**3)
      enddo 
!
!**** compute ozone heating on the way up *********
!
      do k=1,np-1
      j=np-k
      ac(j)=ac(j)+(up(j)-up(j+1))*rco*scosz
      enddo
!
      do j=1,np-1
      ac(j)=max(ac(j),0.0)
      enddo
!
!****** cycle over five bands ***********
!
            do ik=1,5
!
!*** compute total optical depth and single scattering albedo *******
!****** pizero of clouds is 0.99 *********
!
      do j=1,np-1
      tau(j)=tauc(j)+swil(j)*xk(ik)
      sso(j)=0.99*tauc(j)/tau(j)
      enddo
!
!******* compute reflection and transmission for diffuse incidence *****
!
      do j=1,np-1
      a(j)=sqrt(1.0-sso(j)*g)
      b(j)=sqrt(1.0-sso(j))
      u(j)=a(j)/b(j)
      arg(j)=sqrt3*a(j)*b(j)*tau(j)
      e(j)=0.0
      if(arg(j).le.30.0) e(j)=zjexp(-arg(j))
      c(j)=(u(j)+1.0)*(u(j)+1.0)-(u(j)-1.0)*(u(j)-1.0) &
             *e(j)*e(j)
      rl(j)=(u(j)*u(j)-1.0)*(1.0-e(j)*e(j))/c(j)
      tl(j)=4.0*u(j)*e(j)/c(j)
      enddo
!
!******* delta-scaling of all pertinent quantities *******
!
      do j=1,np-1
        a(j)=1.0-g*g*sso(j)
        tau(j)=tau(j)*a(j)
        sso(j)=sso(j)*(1.0-g*g)/a(j)
      enddo
!
!********* compute direct transmission *********
!
      sol(1)=scosz*fk(ik)
!
      do j=1,np-1
        arg(j)=tau(j)*cosmag
        if(arg(j).le.30.0) then
          e0(j)=zjexp(-arg(j))
        else
          e0(j)=0.0
        endif
        sol(j+1)=sol(j)*e0(j)
      enddo
!
!***** compute diffuse reflection and transmission for direct beam *****
!****** set up delta-eddington parameters *********
!
      do j=1,np-1
        gam1(j)=0.25*(7.0-sso(j)*(4.0+3.0*gp))
        gam2(j)=-0.25*(1.0-sso(j)*(4.0-3.0*gp))
        ak(j)=sqrt(gam1(j)*gam1(j)-gam2(j)*gam2(j))
      enddo
!
      gam3=0.25*(2.0-3.0*gp*cmu)
      gam4=1.0-gam3
!
      do j=1,np-1
      alf1(j)=gam1(j)*gam4+gam2(j)*gam3
      alf2(j)=gam1(j)*gam3+gam2(j)*gam4
      enddo
!
      do j=1,np-1
        a(j)=1.0-ak(j)*cmu
        b(j)=1.0+ak(j)*cmu
        aa(j)=gam3-alf2(j)*cmu
        bb(j)=gam4+alf1(j)*cmu
        c(j)=alf2(j)+ak(j)*gam3
        d(j)=alf2(j)-ak(j)*gam3
        e(j)=alf1(j)-ak(j)*gam4
        f(j)=alf1(j)+ak(j)*gam4
      enddo
!
      do j=1,np-1
        d2(j)=1.0-ak(j)*ak(j)*cmu*cmu
        if(abs(d2(j)).lt.1.0e-6) d2(j)=1.0e-6
      enddo
!
      do j=1,np-1
        arg(j)=ak(j)*tau(j)
        if(arg(j).le.30.0) then
          da(j)=zjexp(-arg(j))
        else
          da(j)=0.0
        endif
        arg(j)=2.0*arg(j)
        if(arg(j).le.30.0) then
          db(j)=zjexp(-arg(j))
        else
          db(j)=0.0
        endif
        den(j)=d2(j)*(ak(j)+gam1(j)+(ak(j)-gam1(j))*db(j))
      enddo
!
!***** compute upward and downward fluxes at layer boundaries *******
!
      do j=1,np-1
        u0(j)=sso(j)*(a(j)*c(j)-b(j)*d(j)*db(j) &
               -2.0*ak(j)*aa(j)*e0(j)*da(j))/den(j)
        d0(j+1)=-sso(j)*(b(j)*f(j)*e0(j)-a(j)*e(j)*e0(j) &
                 *db(j)-2.0*ak(j)*bb(j)*da(j))/den(j)
        u0(j)=u0(j)*sol(j)
        d0(j+1)=d0(j+1)*sol(j)
      enddo
!
!******* fill up boundary terms *********
!
      u0(np)=rsurf*sol(np)
      rl(np)=ag
      d0(1)=0.0
      cr(1)=0.0
      cm(1)=1.0
      vp(1)=0.0
      vm(1)=u0(1)
!
!********* safety net **********
!
! dule      do 240 j=1,np
      do j=1,np-1
        rl(j)=max(rl(j),0.0)
        tl(j)=max(tl(j),0.0)
        u0(j)=max(u0(j),0.0)
        d0(j)=max(d0(j),0.0)
        sol(j)=max(sol(j),0.0)
      enddo
!
!************* flux adding of layers *********
!
!******** compute magnification factors *********
!
      do j=1,np-1
        cr(j+1)=rl(j)+tl(j)*cr(j)*tl(j)*cm(j)
        cm(j+1)=1.0/(1.0-cr(j+1)*rl(j+1))
      enddo
!
!******* compute fluxes on the way down ********
!
      do j=1,np-1
        vp(j+1)=tl(j)*cm(j)*(vp(j)+cr(j)*u0(j))+d0(j+1)
        vm(j+1)=cm(j+1)*(vp(j+1)*rl(j+1)+u0(j+1))
      enddo
!
      ul(np)=vm(np)
      dl(np)=vp(np)+ul(np)*cr(np)+sol(np)
      ac(np)=ac(np)+dl(np)-ul(np)
!
!******* compute fluxes on the way up *************
!
      do k=1,np-1
        j=np-k
        ul(j)=vm(j)+ul(j+1)*tl(j)*cm(j)
        dl(j)=vp(j)+ul(j)*cr(j)+sol(j)
        ac(j)=ac(j)+dl(j)-ul(j)-(dl(j+1)-ul(j+1))
      enddo
!
                                 enddo
!
!******** add up absorption in column ********
!
      sc=sc+ac(np)
      dsc=sc
!
      do j=1,np-1
        dsc=dsc+ac(j)
      enddo
!
                        endsubroutine cloudy
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        endmodule module_radiation
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
