!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        module module_sfcpbl
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
use module_control,only:g
use module_surface,only:runsfc
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
implicit none

include 'kind.inc'
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!--for myj turbulence---------------------------------------------------
real(kind=kfpt),parameter:: &
 dldt=2274. &                !
,eliv=2.834e6 &              !                                                  
,elwv=2.50e6 &               !
,epsc=1.e-11 &               !
,epsq=1.e-12 &               !
,f2=17.2693882 &             ! saturation spec. humidity formula coeff.
,f3=273.15 &                 ! saturation spec. humidity formula coeff.
,f4=35.86 &                  ! saturation spec. humidity formula coeff.   
,pq0=379.90516 &             !
,seafc=0.98  &               !
,pq0sea=pq0*seafc            !

!--mellor-yamada-janjic (janjic 2001) constants-------------------------
real(kind=kfpt),parameter:: &
!g=9.8060226 &
 vkarman=0.4 &
!
,eps1=1.e-12,eps2=0. &
,epsru=1.e-7,epsrs=1.e-7 &
,epstrb=1.e-24 &
,fh=1.10 &
!
,alph=0.30,beta=1./273.,el0max=1000.,el0min=1. &
,elfc=1.0,gam1=0.2222222222222222222 &
!,elfc=0.23*0.25,gam1=0.2222222222222222222 &
!,elfc=5.,gam1=0.2222222222222222222 &
!
,a1=0.659888514560862645 &
,a2=0.6574209922667784586 &
,b1=11.87799326209552761 &
,b2=7.226971804046074028 &
,c1=0.000830955950095854396 &
,elz0=0.,esq=5.0 &
!
,btg=beta*g &
,esqhf=0.5*5.0 &
,rb1=1./b1 &
!
,adnh= 9.*a1*a2*a2*(12.*a1+3.*b2)*btg*btg &
,adnm=18.*a1*a1*a2*(b2-3.*a2)*btg &
,anmh=-9.*a1*a2*a2*btg*btg &
,anmm=-3.*a1*a2*(3.*a2+3.*b2*c1+18.*a1*c1-b2)*btg &
,bdnh= 3.*a2*(7.*a1+b2)*btg &
,bdnm= 6.*a1*a1 &
,beqh= a2*b1*btg+3.*a2*(7.*a1+b2)*btg &
,beqm=-a1*b1*(1.-3.*c1)+6.*a1*a1 &
,bnmh=-a2*btg &
,bnmm=a1*(1.-3.*c1) &
,bshh=9.*a1*a2*a2*btg &
,bshm=18.*a1*a1*a2*c1 &
,bsmh=-3.*a1*a2*(3.*a2+3.*b2*c1+12.*a1*c1-b2)*btg &
,cesh=a2 &
,cesm=a1*(1.-3.*c1) &
!--free term in the equilibrium equation for (l/q)**2-------------------
,aeqh=9.*a1*a2*a2*b1*btg*btg &
     +9.*a1*a2*a2*(12.*a1+3.*b2)*btg*btg &
,aeqm=3.*a1*a2*b1*(3.*a2+3.*b2*c1+18.*a1*c1-b2) &
     *btg+18.*a1*a1*a2*(b2-3.*a2)*btg &
!--forbidden turbulence area--------------------------------------------
,requ=-aeqh/aeqm &
,epsgh=1.e-9,epsgm=requ*epsgh &
!--near isotropy for shear turbulence, ww/q2 lower limit----------------
,ubryl=(18.*requ*a1*a1*a2*b2*c1*btg &
       +9.*a1*a2*a2*b2*btg*btg) &
      /(requ*adnm+adnh) &
,ubry=(1.+epsrs)*ubryl,ubry3=3.*ubry &
,aubh=27.*a1*a2*a2*b2*btg*btg-adnh*ubry3 &
,aubm=54.*a1*a1*a2*b2*c1*btg  -adnm*ubry3 &
,bubh=(9.*a1*a2+3.*a2*b2)*btg-bdnh*ubry3 &
,bubm=18.*a1*a1*c1             -bdnm*ubry3 &
,cubr=1.                       -     ubry3 &
,rcubr=1./cubr

!---herff look-up table-------------------------------------------------
integer(kind=kint),parameter:: &
 kerfm=301 &                 ! size of erf half table
,kerfm2=kerfm-2              ! internal points of erf half table

real(kind=kfpt),parameter:: &
 xemin=0. &                  ! lower bound of erf half table
,xemax=3.                    ! upper bound of erf half table

real(kind=kfpt):: &
 rdxe

real(kind=kfpt),dimension(1:kerfm),private,save:: &
 herff                       ! half erf table
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        contains
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine turbl &
(global &
,ids,ide,itb,jds,jde,jtb,kmsc,kztm,kztm2,lm,ntsd,nwetc &
,cp,dtdif,dzeta1,dzeta2,elivw,epsl,epsq2,fh01,fh02,r &
,rdq,rdth,rdyv,thl &
,qs0,sqs &
,ztmax1,ztmax2,ztmin1,ztmin2 &
,dsg2,pdsg1 &
,rdxv &
,insoil,inveg &
,psih1,psih2,psim1,psim2 &
,fis,pd,rlwin,rswin,sice,sm,sst,stdh,vegfrc &
,pint &
,ptbl &
,kmp,lpbl &
,akhs,akms,albedo,epsr,ghct,prcu,prra,prsn,q02,q10,qs,qz0 &
,roff,sno,th02,th10,ths,thz0,deept &
,u10,ustar,uz0,v10,vz0,wliq,z0 &
,tg,wg &
,cw,q,q2,t,u,v,w)
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!-----------------------------------------------------------------------
real(kind=kfpt),parameter:: &
 stdfac=3. &                  ! height standard dev. factor
,dirfac=10.                   !

!-----------------------------------------------------------------------
logical(kind=klog),intent(in):: &
 global                      ! global or regional

integer(kind=kint),intent(in):: &
 ids &                       ! starting index, x direction
,ide &                       ! ending index, x direction
,itb &                       ! table index
,jds &                       ! starting index, y direction
,jde &                       ! ending index, y direction
,jtb &                       ! table index
,kmsc &                      ! maximum # of soil temperature levels
,kztm &                      ! size of stab. function look-up table
,kztm2 &                     ! inner part of stab. function table
,lm &                        ! total # of levels
,ntsd &                      ! number of tume steps done
,nwetc                       ! number of soil moisture levels

real(kind=kfpt),intent(in):: &
 cp &                        ! cp
,dtdif &                      ! turbulence time step
,dzeta1 &                    ! stab function look-up table step
,dzeta2 &                    ! stab function look-up table step
,elivw &                     !
,fh01 &                      ! prandtl # for stability table
,fh02 &                      ! prandtl # for stability table
,r &                         ! gas constant for dry air
,rdq &                       !
,rdth &                      !
,rdyv &                      !
,thl &                       ! upper bound of potential temperature range
,ztmax1 &                    !
,ztmax2 &                    !
,ztmin1 &                    !
,ztmin2                      !

real(kind=kfpt),dimension(1:lm),intent(in):: &
 dsg2 &                      ! delta sigmas
,pdsg1 &                     ! delta pressures
,epsl &                      ! lower bound for master length scale
,epsq2                       ! lower bound for 2tke

real(kind=kfpt),dimension(jds:jde),intent(in):: &
 rdxv

integer(kind=kint),dimension(ids:ide,jds:jde),intent(in):: &
 insoil &                    !
,inveg                       !

real(kind=kfpt),dimension(1:jtb),intent(in):: &
 qs0 &                       ! base for saturation specific humidity
,sqs                         ! range for saturation specific humidity

real(kind=kfpt),dimension(kztm),intent(in):: &
 psih1 &                     !
,psih2 &                     !
,psim1 &                     !
,psim2                       !

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(in):: &
 albedo &                    !
,fis &                       ! surface geopotential
,pd &                        ! sigma range pressure difference
,rlwin &                     ! total incoming radiation at the surface
,rswin &                     ! shortwave in radiation at the surface
,sice &                      ! sea ice
,sm &                        ! sea mask
,sst &                       ! sea surface temperature
,stdh &                      !
,vegfrc                      ! vegetation fraction

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm+1),intent(in):: &
 pint                        ! pressure at interfaces

real(kind=kfpt),dimension(1:itb,1:jtb):: &
 ptbl                        ! saturation pressure table

integer(kind=kint),dimension(ids:ide,jds:jde),intent(inout):: &
 kmp &                       !
,lpbl                        !

real(kind=kfpt),dimension(ids:ide,jds:jde),intent(inout):: &
 akhs &                      ! proportional to sfc. heat exchange coeff.
,akms &                      ! proportional to sfc. momentum exchange coeff.
,epsr &                      ! emissivity
,ghct &                      ! countergradient correction
,prcu &                      ! convective time-step precip
,prra &                      ! time-step liquid precip
,prsn &                      ! time-step snow fall
,qs &                        ! surface specific humidity
,q02 &                       ! 2m specific humidity
,q10 &                       ! 10m specific humidity
,qz0 &                       ! spec. humidity at top of visc. sublayer
,roff &                      ! surface run-off
,sno &                       ! water equivalent of snow
,th02 &                      ! 2m potential temperature
,th10 &                      ! 10m potential temperature
,ths &                       ! surface potential temperature
,thz0 &                      ! pot. temperature at top of viscous layer
,deept &                     ! lower boundary soil temperature
,u10 &                       ! 10m u wind component
,ustar &                     ! friction velocity
,uz0 &                       !
,v10 &                       ! 10m v wind component
,vz0 &                       !
,wliq &                      ! canopy reservoir
,z0                          ! roughness length

real(kind=kfpt),dimension(ids:ide,jds:jde,1:nwetc),intent(inout):: &
 wg                          ! soil moisture

real(kind=kfpt),dimension(ids:ide,jds:jde,1:kmsc),intent(inout):: &
 tg                          ! soil temperature

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm),intent(inout):: &
 cw &                        ! condensate
,q &                         ! specific humidity
,q2 &                        !
,t &                         ! temperature
,u &                         ! u wind component
,v &                         ! v wind component
,w                           ! w wind component

!--local variables------------------------------------------------------
logical(kind=klog):: &
 first &                     ! first pass 
,print_diag

integer(kind=kint):: &
 i &                         ! index in x direction
,iqtb &                      ! table index
,ittb &                      ! table index
,j &                         ! index in y direction
,l &                         ! index in p direction
,lmxl                        ! level of mixed layer top

real(kind=kfpt):: &
 blmx &                      !
,bq &                        !
,bqs00 &                     !
,bqs10 &                     !
,chklmq &                    !
,cl &                        !
,clm &                       !
!,dcl &                       !
,denom &                     !
,elocp &                     !
!,elv &                       !
,epsrp &                     ! emissivity
,hpbl &                      !
,hrg &                       !
,p00 &                       !
,p01 &                       !
,p10 &                       !
,p11 &                       !
,plm &                       !
,pp1 &                       !
,ps &                        !
,psgml &                     !
,psp &                       !
!,qc &                        !
,ql &                        !
,qlm &                       !
,qq1 &                       !
,radin &                     !
,rcp &                       !
,rib &                       !
,rdt &                       !
,rkhs &                      !
,rkmsv &                     !
,rlmo &                      ! 1./ monin-obukhov length
,rg &                        !
,rlwnet &                    !
,rswnet &                    !
,rocp &                      !
,rol &                       !
,rrg &                       !
,rsqdt &                     !
,rstdhv &                    !
,rxnrs &                     ! 1./exner at saturation pressure
,smv &                       !
,sprod &                     !
,sq &                        !
,sqs00 &                     !
,sqs10 &                     !
,stdhv &                     !
,thdl &                      !
,thel &                      !
,thvl &                      !
,thlm &                      !
,thvlm &                     !
,tl &                        !
,tlm &                       !
,tpsp &                      !
,tq &                        !
,trf &                       !
,tth &                       !
,tvfacl &                    !
,ulm &                       !
,uhl &                       !
,vlm &                       !
,vhl &                       !
,wl &                        !
,wstar &                     ! convective vertical velocity scale
,rxnrlm &                    !
,rxns &                       !
,z0p &                       !
,zsf &                       !
,zsl &                       !
,zsx &                       !
,zsy &                       !
,zuv

real(kind=kfpt),dimension(1:nwetc):: &
 wgp                         ! soil moisture

real(kind=kfpt),dimension(1:kmsc):: &
 tgp                         ! soil temperature

real(kind=kfpt),dimension(1:lm-1):: &
 akhcol &                    !
,akmcol &                    !
,dcol &
,rkhcol &                    !
,rkmcol &                    !
,el &                        !
,gh &                        !
,gm                          !

real(kind=kfpt),dimension(1:lm):: &
 ccol &                      !
,pcol &                      !
,q2col &                     !
,qcol &                      !
,rocol &                     !
,rxnrcol &                   !
,spcol &                     !
,tcol &                      !
,thcol &                    !
,thecol &                    !
,thvcol &                    !
,ucol &                      !
,vcol &                      !
,wcol                        !

real(kind=kfpt),dimension(1:lm+1):: &
 zcol(lm+1)

real(kind=kfpt),dimension(ids:ide,jds:jde):: &
 uz0h &                      !
,vz0h                        !

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm-1):: &
 akh &
,akm

real(kind=kfpt),dimension(ids:ide,jds:jde,1:lm):: &
 uh &                        !
,vh                          ! 

data first/.true./
!-----------------------------------------------------------------------
      if(first) then
        call herftbl
        first=.false.
      endif
!
      print_diag=.false.
!
      chklmq=1.
      rcp=1./cp
      elocp=elivw/cp
      rocp=r/cp
      rg=1./g
      hrg=rg*0.5
      rrg=r/g
!
      rdt=1./dtdif
      rsqdt=1./sqrt(dtdif)
!-------------remove negative q2----------------------------------------
      do l=1,lm
        do j=jds,jde
          do i=ids,ide
            q2(i,j,l)=max(q2(i,j,l),epsq2(l))
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
      do j=jds,jde
        do i=ids,ide
          uz0h(i,j)=0.
          vz0h(i,j)=0.
        enddo
      enddo
!----------------------------------------------------------------------
      do j=jds+1,jde-1
        do i=ids+1,ide-1
          zcol(lm+1)=fis(i,j)*rg
          do l=lm,1,-1
            pcol   (l)=(pint(i,j,l)+pint(i,j,l+1))*0.5
            tl        =t(i,j,l)
            tcol   (l)=tl
            ql        =q(i,j,l)
            qcol   (l)=ql
            ccol   (l)=cw(i,j,l)
            wcol   (l)=w (i,j,l)
            rxnrcol(l)=(100000./pcol(l))**rocp
            thcol  (l)=tl*rxnrcol(l)
            tvfacl    =ql*0.608+(1.-ccol(l))
            thvcol (l)=thcol(l)*tvfacl  
            rocol  (l)=pcol(l)/(tl*tvfacl*r)
            zcol   (l)=tl*tvfacl*rrg &
                      *(dsg2(l)*pd(i,j)+pdsg1(l))/pcol(l) &
                      +zcol(l+1)
!
            tth=(thcol(l)-thl)*rdth
            qq1=tth-aint(tth)
            ittb=int(tth)+1
!
            if(ittb.lt.1) then
              ittb=1
              qq1=0.
            endif
            if(ittb.ge.jtb) then
              ittb=jtb-1
              qq1=0.
            endif
!
            bqs00=qs0(ittb)
            sqs00=sqs(ittb)
            bqs10=qs0(ittb+1)
            sqs10=sqs(ittb+1)
!
            bq=(bqs10-bqs00)*qq1+bqs00
            sq=(sqs10-sqs00)*qq1+sqs00
            tq=(ql-bq)/sq*rdq
            pp1=tq-aint(tq)
            iqtb=int(tq)+1
!
            if(iqtb.lt.1) then
              iqtb=1
              pp1=0.
              endif
            if(iqtb.ge.itb) then
              iqtb=itb-1
              pp1=0.
            endif
!
            p00=ptbl(iqtb,ittb)
            p10=ptbl(iqtb+1,ittb)
            p01=ptbl(iqtb,ittb+1)
            p11=ptbl(iqtb+1,ittb+1)
!
            psp=p00+(p10-p00)*pp1+(p01-p00)*qq1 &
                   +(p00-p10-p01+p11)*pp1*qq1
            psp=max(psp,1.)
            spcol  (l)=psp
            rxnrs=(100000./psp)**rocp
            thecol (l)=thcol(l)*exp(elocp*qcol(l)*rxnrs/thcol(l))
!
            q2col  (l)=q2(i,j,l)
!
            if(    j.eq.jds+1) then
              ucol   (l)=(u(i-1,j  ,l)+u(i  ,j  ,l))*0.5       
              vcol   (l)=(v(i-1,j  ,l)+v(i  ,j  ,l))*0.5
            elseif(j.eq.jde-1) then
              ucol   (l)=(u(i-1,j-1,l)+u(i  ,j-1,l))*0.5       
              vcol   (l)=(v(i-1,j-1,l)+v(i  ,j-1,l))*0.5
            else
              ucol   (l)=(u(i-1,j-1,l)+u(i  ,j-1,l) &
                         +u(i-1,j  ,l)+u(i  ,j  ,l))*0.25       
              vcol   (l)=(v(i-1,j-1,l)+v(i  ,j-1,l) &
                         +v(i-1,j  ,l)+v(i  ,j  ,l))*0.25
            endif
!
            uh(i,j,l)=ucol(l)
            vh(i,j,l)=vcol(l)
          enddo
!if(i.eq.150) then
!print*,(thvcol(l),l=23,lm)
!print*,(qcol  (l),l=23,lm)
!print*,(q2col (l),l=23,lm)
!endif
!-------------call the working routines---------------------------------
          call mixlen &
          (lm,epsl,epsq2,rdt,rsqdt,ghct(i,j) &
          ,ucol,vcol,wcol,thvcol,thecol &                        
          ,q2col,zcol,pcol,spcol,rxnrcol &
          ,gm,gh,el,hpbl,lpbl(i,j),blmx,lmxl)
!
          call prodq2 &
          (lm,epsl,epsq2,dtdif,ustar(i,j) &
          ,gm,gh,el,q2col)
!
          call difcof &
          (lm,lmxl,gm,gh,el,tcol,q2col,zcol &
          ,akmcol,akhcol)
!
          do l=1,lm-1
            akh(i,j,l)=akhcol(l)
            akm(i,j,l)=akmcol(l)
          enddo
!
          call vdifq &
          (lm,dtdif,q2col,el,zcol)
!          
          do l=1,lm
            q2(i,j,l)=max(q2col(l),epsq2(l))
          enddo
!
          ulm=ucol(lm)
          vlm=vcol(lm)
          tlm=tcol(lm)
          thlm=tlm*rxnrcol(lm)
          thvlm=thvcol(lm)
          qlm=qcol(lm)
          clm=ccol(lm)
          zsl=(zcol(lm)-zcol(lm+1))*0.5
!
          ps=pint(i,j,lm+1)
          rxns=(100000./ps)**rocp
!
          z0p=z0(i,j)
!          if(sno(i,j).gt.0.) z0p=0.0013
!
          if(sm(i,j).gt.0.5) then                                             
            zsx=(fis(i+1,j)-fis(i-1,j))*rdxv(j)*hrg
            zsy=(fis(i,j+1)-fis(i,j-1))*rdyv   *hrg
            sprod=max(zsx*ulm,0.)+max(zsy*vlm,0.)
            denom=sqrt((zsx*zsx+zsy*zsy)*(ulm*ulm+vlm*vlm))
            if(denom.gt.0.) then
              z0p=((sprod/denom)*dirfac+1.)*z0p
            endif
          endif
!
          uz0h(i,j)=(uz0(i-1,j-1)+uz0(i,j-1)+uz0(i-1,j)+uz0(i,j))*0.25          
          vz0h(i,j)=(vz0(i-1,j-1)+vz0(i,j-1)+vz0(i-1,j)+vz0(i,j))*0.25
!
!if(abs(u10(i,j)).gt.100..or.abs(v10(i,j)).gt.100.) &
!print*,'pre',i,j,u10(i,j),v10(i,j),z0p,fis(i,j)
          call sfcdif &
          (sm(i,j),ths(i,j),qs(i,j) &
          ,uz0h(i,j),vz0h(i,j),thz0(i,j),qz0(i,j),hpbl &
          ,z0p,ustar(i,j),wstar &
          ,ulm,vlm,tlm,thlm,thvlm,qlm,zsl &
          ,rib,rlmo,akms(i,j),akhs(i,j),ghct(i,j) &
          ,u10(i,j),v10(i,j),th02(i,j),th10(i,j) &
          ,q02(i,j),q10(i,j) &
          ,kztm,kztm2,ztmax1,ztmax2,ztmin1,ztmin2 &
          ,dzeta1,dzeta2,fh01,fh02 &
          ,psih1,psih2,psim1,psim2 &
          ,i,j,ntsd)
!if(abs(u10(i,j)).gt.100..or.abs(v10(i,j)).gt.100.) &
!print*,'posle',i,j,u10(i,j),v10(i,j)
!-----------------------------------------------------------------------
          plm=(pint(i,j,lm)+ps)*0.5
          rxnrlm=rxnrcol(lm)
          trf=tg(i,j,1)
!
          epsrp=epsr(i,j)
          if(sice(i,j).gt.0.5.or.sno(i,j).gt.0.0) epsrp=0.9
!---radiation input-----------------------------------------------------
          rswnet=rswin(i,j)*(1.-albedo(i,j))
          rlwnet=rlwin(i,j)*epsrp
          radin=max(rswnet+rlwnet,000.)  
!-----------------------------------------------------------------------
          do l=1,nwetc
            wgp(l)=wg(i,j,l)
          enddo
!
          do l=1,kmsc
            tgp(l)=tg(i,j,l)
          enddo
!
!print*,sno(i,j)
!if(i.eq.91.and.j.eq.2) then
!print*,kmsc
!print*,tgp
!print*,'insoil(i,j),inveg(i,j),sm(i,j)',insoil(i,j),inveg(i,j),sm(i,j)
!print*,'sst(i,j),sice(i,j),vegfrc(i,j)',sst(i,j),sice(i,j),vegfrc(i,j)
!print*,'kmp(i,j),dtdif',kmp(i,j),dtdif
!print*,'epsrp,radin,rswin(i,j),akhs(i,j)',epsrp,radin,rswin(i,j),akhs(i,j)
!print*,'plm,rxnrlm,tlm,qlm,clm',plm,rxnrlm,tlm,qlm,clm
!print*,'ps,rxns,ths(i,j),qs(i,j),deept(i,j)',ps,rxns,ths(i,j),qs(i,j),deept(i,j)
!print*,'prra(i,j),prsn(i,j),prcu(i,j),sno(i,j),roff(i,j)',prra(i,j),prsn(i,j),prcu(i,j),sno(i,j),roff(i,j)
!print*,'tg(i,j,:),wliq(i,j),wg(i,j,:)',tgp,wliq(i,j),wgp
!endif
!
          call runsfc &
          (insoil(i,j),inveg(i,j),sm(i,j) &
          ,sst(i,j),sice(i,j),vegfrc(i,j) &
          ,kmp(i,j),dtdif &
          ,epsrp,radin,rswnet,akhs(i,j) &
          ,plm,rxnrlm,tlm,qlm,clm &
          ,ps,rxns,ths(i,j),qs(i,j),deept(i,j) &
          ,prra(i,j),prsn(i,j),prcu(i,j),sno(i,j),roff(i,j) &
          ,tgp,wliq(i,j),wgp)
!
!if(i.eq.91.and.j.eq.2) then
!print*,'insoil(i,j),inveg(i,j),sm(i,j)',insoil(i,j),inveg(i,j),sm(i,j)
!print*,'sst(i,j),sice(i,j),vegfrc(i,j)',sst(i,j),sice(i,j),vegfrc(i,j)
!print*,'kmp(i,j),dtdif',kmp(i,j),dtdif
!print*,'epsrp,radin,rswin(i,j),akhs(i,j)',epsrp,radin,rswin(i,j),akhs(i,j)
!print*,'plm,rxnrlm,tlm,qlm,clm',plm,rxnrlm,tlm,qlm,clm
!print*,'ps,rxns,ths(i,j),qs(i,j),deept(i,j)',ps,rxns,ths(i,j),qs(i,j),deept(i,j)
!print*,'prra(i,j),prsn(i,j),prcu(i,j),sno(i,j),roff(i,j)',prra(i,j),prsn(i,j),prcu(i,j),sno(i,j),roff(i,j)
!print*,'tg(i,j,:),wliq(i,j),wg(:,i,j)',tg(i,j,:),wliq(i,j),wg(i,j,:) 
!endif
!-----------------------------------------------------------------------
          do l=1,nwetc
            wg(i,j,l)=wgp(l)
          enddo
!
          do l=1,kmsc
            tg(i,j,l)=tgp(l)
          enddo
!
          thz0(i,j)=(1.-sm(i,j))*ths(i,j)+sm(i,j)*thz0(i,j)
          qz0 (i,j)=(1.-sm(i,j))*qs (i,j)+sm(i,j)*qz0 (i,j)
!
          do l=1,lm-1
            rkhcol(l)=(rocol(l)+rocol(l+1))*0.5*akh(i,j,l)
          enddo
          rkhs=rocol(lm)*akhs(i,j)
!-------------the vertical diffusion of temperature and water vapor-----
          call vdifh &
          (lm,dtdif,thz0(i,j),qz0(i,j),rkhs,chklmq,ghct(i,j) &
          ,thcol,qcol,ccol,rkhcol,zcol,rocol)
!
          do l=1,lm
            cw(i,j,l)=ccol(l)
            q (i,j,l)=qcol(l)
            t (i,j,l)=tcol(l)
          enddo
!-----------------------------------------------------------------------
          smv=sm(i,j)
          stdhv=stdh(i,j)*stdfac
          if(stdhv.gt.0.) then
            rstdhv=1./stdhv
          else
            rstdhv=0.
          endif
          zsf=stdhv+zcol(lm+1)
!
          rkmsv=akms(i,j)*rocol(lm)
!
          do l=1,lm-1
            rkmcol(l)=(rocol(l)+rocol(l+1))*0.5*akm(i,j,l)
!
            zuv=(zcol(l)+zcol(l+1))*0.5
            if(smv.gt.0.5) then                                             
              dcol(l)=0.
            else
              if(zuv.gt.zsf) then
                dcol(l)=0.
              else
                zsx=(fis(i+1,j)-fis(i-1,j))*rdxv(j)*hrg
                zsy=(fis(i,j+1)-fis(i,j-1))*rdyv   *hrg
                sprod=max(zsx*uh(i,j,l),0.)+max(zsy*vh(i,j,l),0.)
                denom=sqrt((zsx*zsx+zsy*zsy) &
                     *(uh(i,j,l)*uh(i,j,l)+vh(i,j,l)*vh(i,j,l)))
                if(denom.gt.0.) then
                  dcol(l)=(1.-min((zuv-zcol(lm+1))*rstdhv,1.))*0.5 &
                         *((sprod/denom)*dirfac+1.)
                else
                  dcol(l)=0.
                endif
              endif
            endif
          enddo
!-------------the vertical diffusion of velocity components-------------
          call vdifv &
          (lm,dtdif,uz0h(i,j),vz0h(i,j) &
          ,rkmsv,dcol,ucol,vcol,rkmcol,zcol,rocol)
!
          do l=1,lm
            uh(i,j,l)=ucol(l)-uh(i,j,l)
            vh(i,j,l)=vcol(l)-vh(i,j,l)
          enddo
!-----------------------------------------------------------------------
        enddo
      enddo
!-------------interpolate back to v points------------------------------
      do j=jds+1,jde-2
        do i=ids+1,ide-2
          uz0(i,j)=(uz0h(i  ,j  )+uz0h(i+1,j  ) &
                   +uz0h(i  ,j+1)+uz0h(i+1,j+1))*0.25
          vz0(i,j)=(vz0h(i  ,j  )+vz0h(i+1,j  ) &
                   +vz0h(i  ,j+1)+vz0h(i+1,j+1))*0.25
        enddo
      enddo
!
      do l=1,lm
        do j=jds+1,jde-2
          do i=ids+1,ide-2
            u(i,j,l)=(uh(i  ,j  ,l)+uh(i+1,j  ,l) &
                     +uh(i  ,j+1,l)+uh(i+1,j+1,l))*0.25+u(i,j,l)
            v(i,j,l)=(vh(i  ,j  ,l)+vh(i+1,j  ,l) &
                     +vh(i  ,j+1,l)+vh(i+1,j+1,l))*0.25+v(i,j,l)
          enddo
        enddo
      enddo
!-----------------------------------------------------------------------
!      if(global) then
!      endif
!-----------------------------------------------------------------------
                        endsubroutine turbl
!-----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      subroutine mixlen &
      (lm,epsl,epsq2,rdt,rsqdt,ct &
      ,u,v,w_tot,thv,the,q2,z,p,ps,rxnr &
      ,gm,gh,el,hpbl,lpbl,blmx,lmxl)
!----------------------------------------------------------------------
!   ******************************************************************
!   *                                                                *
!   *                   level 2.5 mixing length                      *
!   *                                                                *
!   ******************************************************************
!----------------------------------------------------------------------
!
      implicit none
!
!----------------------------------------------------------------------
      integer(kind=kint),intent(in):: &
       lm
!
      real(kind=kfpt),intent(in):: &
       rdt,rsqdt
!
      integer(kind=kint),intent(out):: &
       lmxl,lpbl
!
      real(kind=kfpt),dimension(1:lm-1),intent(in):: &
       epsl

      real(kind=kfpt),dimension(1:lm),intent(in):: &
       p,ps,q2,epsq2,rxnr,the,thv,u,v,w_tot
!
      real(kind=kfpt),dimension(1:lm+1),intent(in):: &
       z
!
      real(kind=kfpt),intent(out):: &
       blmx &
      ,hpbl
!
      real(kind=kfpt),dimension(1:lm-1),intent(out):: &
       el,gh,gm
!
      real(kind=kfpt),intent(inout):: ct
!----------------------------------------------------------------------
!***
!***  local variables
!***
      integer(kind=kint):: &
       k,lpblm
!
      real(kind=kfpt):: &
       aden,bden,aubr,bubr,cubry,dthv,dz &
      ,el0,eloq2x,ghl,gml &
      ,qol2st,qol2un,qdzl &
      ,rdz,sq,srel,szq,vkrmz,wcon,wwfc
!
      real(kind=kfpt),dimension(1:lm):: &
       q1
!
      real(kind=kfpt),dimension(1:lm-1):: &
       elm,rel
!
!----------------------------------------------------------------------
!***********************************************************************
!--------1---------2---------3---------4---------5---------6---------7--
      cubry=ubry*1.5  !*2.
      wwfc=0.5 !*0.00
!--------------find the height of the pbl-------------------------------
      lpbl=lm
!
      do k=lm-1,1,-1
        if(q2(k)-epsq2(k)+epsq2(lm).le.epsq2(lm)*fh) then
          lpbl=k
          go to 110
        endif
      enddo
!
      lpbl=1
!
!--------------the height of the pbl------------------------------------
!
 110  hpbl=z(lpbl+1)-z(lm+1)
!
!-----------------------------------------------------------------------
      do k=1,lm
        q1(k)=0.
      enddo
!-----------------------------------------------------------------------
      do k=1,lm-1
        dz=(z(k)-z(k+2))*0.5
        rdz=1./dz
        gml=((u(k)-u(k+1))**2+(v(k)-v(k+1))**2)*rdz*rdz
        gm(k)=max(gml,epsgm)
!
        dthv=thv(k)-thv(k+1)
!----------------------------------------------------------------------
        if(dthv.gt.0.) then
          if(the(k+1).gt.the(k)) then
            if(ps(k+1).gt.p(k)) then
!
              wcon=(p(k+1)-ps(k+1))/(p(k+1)-p(k))
!
              if( &
                  (q2(k).gt.epsq2(k)).and. &
                  (q2(k)*cubry.gt.(dz*wcon*rsqdt)**2) &
                  .or. &
                  ((k.lt.lpbl-1).and. &
                  ((w_tot(k)+w_tot(k+1))*wwfc).gt.(dz*wcon*rdt)) &
                ) then
!
                dthv=(the(k)-the(k+1))+dthv
!
              endif
            endif
          endif
        endif
!--------------------------------------------------------------------------
!
        ghl=dthv*rdz
        if(abs(ghl)<=epsgh)ghl=epsgh
        gh(k)=ghl
      enddo
!
      ct=0.
!
!----------------------------------------------------------------------
!***  find maximum mixing lengths and the level of the pbl top
!----------------------------------------------------------------------
!
      lmxl=lm
!
      do k=1,lm-1
        gml=gm(k)
        ghl=gh(k)
!
        if(ghl>=epsgh)then
          if(gml/ghl<=requ)then
            elm(k)=epsl(k)
            lmxl=k+1
          else
            aubr=(aubm*gml+aubh*ghl)*ghl
            bubr= bubm*gml+bubh*ghl
            qol2st=(-0.5*bubr+sqrt(bubr*bubr*0.25-aubr*cubr))*rcubr
            eloq2x=1./qol2st
            elm(k)=max(sqrt(eloq2x*q2(k)),epsl(k))
          endif
        else
          aden=(adnm*gml+adnh*ghl)*ghl
          bden= bdnm*gml+bdnh*ghl
          qol2un=-0.5*bden+sqrt(bden*bden*0.25-aden)
          eloq2x=1./(qol2un+epsru)       ! repsr1/qol2un
          elm(k)=max(sqrt(eloq2x*q2(k)),epsl(k))
        endif
      enddo
!
      if(elm(lm-1)==epsl(lm-1))lmxl=lm
!
!----------------------------------------------------------------------
!***  the height of the mixed layer
!----------------------------------------------------------------------
      blmx=z(lmxl)-z(lm+1)
!----------------------------------------------------------------------
      do k=lpbl,lm
        q1(k)=sqrt(q2(k))
      enddo
!----------------------------------------------------------------------
      szq=0.
      sq =0.
!
      do k=1,lm-1
        qdzl=(q1(k)+q1(k+1))*(z(k+1)-z(k+2))
        szq=(z(k+1)+z(k+2)-z(lm+1)-z(lm+1))*qdzl+szq
        sq=qdzl+sq
      enddo
!
!----------------------------------------------------------------------
!***  computation of asymptotic l in blackadar formula
!----------------------------------------------------------------------
!
      el0=min(alph*szq*0.5/sq,el0max)
      el0=max(el0            ,el0min)
!
!----------------------------------------------------------------------
!***  above the pbl top
!----------------------------------------------------------------------
!
      lpblm=max(lpbl-1,1)
!
      do k=1,lpblm
        el(k)=min((z(k)-z(k+2))*elfc,elm(k))
        rel(k)=el(k)/elm(k)
      enddo
!
!----------------------------------------------------------------------
!***  inside the pbl
!----------------------------------------------------------------------
!
      if(lpbl<lm)then
        do k=lpbl,lm-1
          vkrmz=(z(k+1)-z(lm+1))*vkarman
          el(k)=min(vkrmz/(vkrmz/el0+1.),elm(k))
          rel(k)=el(k)/elm(k)
        enddo
      endif
!
      do k=lpbl+1,lm-2
        srel=min(((rel(k-1)+rel(k+1))*0.5+rel(k))*0.5,rel(k))
        el(k)=max(srel*elm(k),epsl(k))
      enddo
!
!----------------------------------------------------------------------
      end subroutine mixlen
!----------------------------------------------------------------------
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!----------------------------------------------------------------------
      subroutine prodq2 &
      (lm,epsl,epsq2,dtdif,ustar &
      ,gm,gh,el,q2)
!----------------------------------------------------------------------
!   ******************************************************************
!   *                                                                *
!   *            level 2.5 q2 production/dissipation                 *
!   *                                                                *
!   ******************************************************************
!----------------------------------------------------------------------
!
      implicit none
!
!----------------------------------------------------------------------
      integer(kind=kint),intent(in):: &
       lm
!
      real(kind=kfpt),intent(in):: &
       dtdif,ustar
!
      real(kind=kfpt),dimension(1:lm-1),intent(in):: &
       gh,gm
!
      real(kind=kfpt),dimension(1:lm-1),intent(in):: &
       epsl
!
      real(kind=kfpt),dimension(1:lm),intent(in):: &
       epsq2
!
      real(kind=kfpt),dimension(1:lm-1),intent(inout):: &
       el
!
      real(kind=kfpt),dimension(1:lm),intent(inout):: &
       q2
!----------------------------------------------------------------------
!***
!***  local variables
!***
      integer(kind=kint):: &
       k
!
      real(kind=kfpt):: &
       aden,aequ,anum,arhs,bden,bequ,bnum,brhs,cden,crhs &
      ,dloq1,eloq11,eloq12,eloq13,eloq21,eloq22,eloq31,eloq32 &
      ,eloq41,eloq42,eloq51,eloq52,eloqn,eqol2,ghl,gml &
      ,rden1,rden2,rhs2,rhsp1,rhsp2,rhst2
!
!----------------------------------------------------------------------
!**********************************************************************
!----------------------------------------------------------------------
!
      main_integration: do k=1,lm-1
        gml=gm(k)
        ghl=gh(k)
!
!----------------------------------------------------------------------
!***  coefficients of the equilibrium equation
!----------------------------------------------------------------------
!
        aequ=(aeqm*gml+aeqh*ghl)*ghl
        bequ= beqm*gml+beqh*ghl
!
!----------------------------------------------------------------------
!***  equilibrium solution for l/q
!----------------------------------------------------------------------
!
        eqol2=-0.5*bequ+sqrt(bequ*bequ*0.25-aequ)
!
!----------------------------------------------------------------------
!***  is there production/dissipation ?
!----------------------------------------------------------------------
!
        if((gml+ghl*ghl<=epstrb)                                                &
         .or.(ghl>=epsgh.and.gml/ghl<=requ)                                     &
         .or.(eqol2<=eps2))then
!
!----------------------------------------------------------------------
!***  no turbulence
!----------------------------------------------------------------------
!
          q2(k)=epsq2(k)
          el(k)=epsl(k)
!----------------------------------------------------------------------
!
        else
!
!----------------------------------------------------------------------
!***  turbulence
!----------------------------------------------------------------------
!----------------------------------------------------------------------
!***  coefficients of the terms in the numerator
!----------------------------------------------------------------------
!
          anum=(anmm*gml+anmh*ghl)*ghl
          bnum= bnmm*gml+bnmh*ghl
!
!----------------------------------------------------------------------
!***  coefficients of the terms in the denominator
!----------------------------------------------------------------------
!
          aden=(adnm*gml+adnh*ghl)*ghl
          bden= bdnm*gml+bdnh*ghl
          cden= 1.
!
!----------------------------------------------------------------------
!***  coefficients of the numerator of the linearized eq.
!----------------------------------------------------------------------
!
          arhs=-(anum*bden-bnum*aden)*2.
          brhs=- anum*4.
          crhs=- bnum*2.
!
!----------------------------------------------------------------------
!***  initial value of l/q
!----------------------------------------------------------------------
!
          dloq1=el(k)/sqrt(q2(k))
!
!----------------------------------------------------------------------
!***  first iteration for l/q, rhs=0
!----------------------------------------------------------------------
!
          eloq21=1./eqol2
          eloq11=sqrt(eloq21)
          eloq31=eloq21*eloq11
          eloq41=eloq21*eloq21
          eloq51=eloq21*eloq31
!
!----------------------------------------------------------------------
!***  1./denominator
!----------------------------------------------------------------------
!
          rden1=1./(aden*eloq41+bden*eloq21+cden)
!
!----------------------------------------------------------------------
!***  d(rhs)/d(l/q)
!----------------------------------------------------------------------
!
          rhsp1=(arhs*eloq51+brhs*eloq31+crhs*eloq11)*rden1*rden1
!
!----------------------------------------------------------------------
!***  first-guess solution
!----------------------------------------------------------------------
!
          eloq12=eloq11+(dloq1-eloq11)*exp(rhsp1*dtdif)
          eloq12=max(eloq12,eps1)
!
!----------------------------------------------------------------------
!***  second iteration for l/q
!----------------------------------------------------------------------
!
          eloq22=eloq12*eloq12
          eloq32=eloq22*eloq12
          eloq42=eloq22*eloq22
          eloq52=eloq22*eloq32
!
!----------------------------------------------------------------------
!***  1./denominator
!----------------------------------------------------------------------
!
          rden2=1./(aden*eloq42+bden*eloq22+cden)
          rhs2 =-(anum*eloq42+bnum*eloq22)*rden2+rb1
          rhsp2= (arhs*eloq52+brhs*eloq32+crhs*eloq12)*rden2*rden2
          rhst2=rhs2/rhsp2
!
!----------------------------------------------------------------------
!***  corrected solution
!----------------------------------------------------------------------
!
          eloq13=eloq12-rhst2+(rhst2+dloq1-eloq12)*exp(rhsp2*dtdif)
          eloq13=max(eloq13,eps1)
!
!----------------------------------------------------------------------
!***  two iterations is enough in most cases ...
!----------------------------------------------------------------------
!
          eloqn=eloq13
!
          if(eloqn>eps1)then
            q2(k)=el(k)*el(k)/(eloqn*eloqn)
            q2(k)=max(q2(k),epsq2(k))
            if(q2(k).eq.epsq2(k))then
              el(k)=epsl(k)
            endif
          else
            q2(k)=epsq2(k)
            el(k)=epsl(k)
          endif
!
!----------------------------------------------------------------------
!***  end of turbulent branch
!----------------------------------------------------------------------
!
        endif
!----------------------------------------------------------------------
!***  end of production/dissipation loop
!----------------------------------------------------------------------
!
      enddo main_integration
!
!----------------------------------------------------------------------
!***  lower boundary condition for q2
!----------------------------------------------------------------------
!
      q2(lm)=max(b1**(2./3.)*ustar*ustar,epsq2(lm))
!----------------------------------------------------------------------
!
      end subroutine prodq2
!
!----------------------------------------------------------------------
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine difcof &
      (lm,lmxl,gm,gh,el,t,q2,z &
      ,akm,akh)
!----------------------------------------------------------------------

!   ******************************************************************
!   *                                                                *
!   *                level 2.5 diffusion coefficients                *
!   *                                                                *
!   ******************************************************************
!----------------------------------------------------------------------
      implicit none
!----------------------------------------------------------------------
      integer(kind=kint),intent(in):: &
       lm,lmxl
!
      real(kind=kfpt),dimension(1:lm),intent(in):: &
       q2,t
!
      real(kind=kfpt),dimension(1:lm-1),intent(in):: &
       el,gh,gm
!
      real(kind=kfpt),dimension(1:lm+1),intent(in):: &
       z
!
      real(kind=kfpt),dimension(1:lm-1),intent(out):: &
       akh,akm
!----------------------------------------------------------------------
!***
!***  local variables
!***
      integer(kind=kint):: &
       k,kinv
!
      real(kind=kfpt):: &
       aden,akmin,bden,besh,besm,cden,d2t,ell,eloq2,eloq4,elqdz &
      ,esh,esm,ghl,gml,q1l,rden,rdz
!
!*** begin debugging
!     real(kind=kfpt):: d2tmin
!*** end debugging
!
!----------------------------------------------------------------------
!**********************************************************************
!----------------------------------------------------------------------
!
      do k=1,lm-1
        ell=el(k)
!
        eloq2=ell*ell/q2(k)
        eloq4=eloq2*eloq2
!
        gml=gm(k)
        ghl=gh(k)
!
!----------------------------------------------------------------------
!***  coefficients of the terms in the denominator
!----------------------------------------------------------------------
!
        aden=(adnm*gml+adnh*ghl)*ghl
        bden= bdnm*gml+bdnh*ghl
        cden= 1.
!
!----------------------------------------------------------------------
!***  coefficients for the sm determinant
!----------------------------------------------------------------------
!
        besm=bsmh*ghl
!
!----------------------------------------------------------------------
!***  coefficients for the sh determinant
!----------------------------------------------------------------------
!
        besh=bshm*gml+bshh*ghl
!
!----------------------------------------------------------------------
!***  1./denominator
!----------------------------------------------------------------------
!
        rden=1./(aden*eloq4+bden*eloq2+cden)
!
!----------------------------------------------------------------------
!***  sm and sh
!----------------------------------------------------------------------
!
        esm=(besm*eloq2+cesm)*rden
        esh=(besh*eloq2+cesh)*rden
!
!----------------------------------------------------------------------
!***  diffusion coefficients
!----------------------------------------------------------------------
!
        rdz=2./(z(k)-z(k+2))
        q1l=sqrt(q2(k))
        elqdz=ell*q1l*rdz
        akm(k)=elqdz*esm
        akh(k)=elqdz*esh
!wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
!        akm(k)=max(akm(k),rdz*3.)
!        akh(k)=max(akh(k),rdz*3.)
!mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
!----------------------------------------------------------------------
      enddo
!----------------------------------------------------------------------
!
!----------------------------------------------------------------------
!***  inversions
!----------------------------------------------------------------------
!
!     if(lmxl==lm)then
!       kinv=lm
!       d2tmin=0.
!
!       do k=lm/2,lm-1
!         d2t=t(k-1)-2.*t(k)+t(k+1)
!         if(d2t<d2tmin)then
!           d2tmin=d2t
!           if(d2t<0)kinv=k
!         endif
!       enddo
!
!       if(kinv<lm)then
!         do k=kinv-1,lm-1
!           rdz=2./(z(k)-z(k+2))
!           akmin=0.5*rdz
!           akm(k)=max(akm(k),akmin)
!           akh(k)=max(akh(k),akmin)
!         enddo
!
!*** begin debugging
!         if(print_diag>0)then
!           write(6,"(a,3i3)") '{turb1 lmxl,lm,kinv=',lmxl,lm,kinv
!           write(6,"(a,3i3)") '}turb1 lmxl,lm,kinv=',lmxl,lm,kinv
!           if(print_diag==1)then
!             write(6,"(a)") &
!               '{turb3 k, t, d2t, rdz, z(k), z(k+2), akmin, akh '
!           else
!             write(6,"(a)") &
!               '}turb3 k, t, d2t, rdz, z(k), z(k+2), akmin, akh '
!           endif
!           do k=lm-1,kinv-1,-1
!             d2t=t(k-1)-2.*t(k)+t(k+1)
!             rdz=2./(z(k)-z(k+2))
!             akmin=0.5*rdz
!             if(print_diag==1)then
!               write(6,"(a,i3,f8.3,2e12.5,2f9.2,2e12.5)") '{turb3 ' &
!               ,k,t(k)-273.15,d2t,rdz,z(k),z(k+2),akmin,akh(k)
!             else
!               write(6,"(a,i3,f8.3,2e12.5,2f9.2,2e12.5)") '}turb3 ' &
!               ,k,t(k)-273.15,d2t,rdz,z(k),z(k+2),akmin,akh(k)
!             endif
!           enddo
!         endif     !- if (print_diag > 0) then
!       endif       !- if(kinv<lm)then
!*** end debugging
!
!     endif
!----------------------------------------------------------------------
!
      end subroutine difcof
!
!----------------------------------------------------------------------
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!----------------------------------------------------------------------
      subroutine vdifq &
      (lm,dtdif,q2,el,z)
!   ******************************************************************
!   *                                                                *
!   *               vertical diffusion of q2 (tke)                   *
!   *                                                                *
!   ******************************************************************

!----------------------------------------------------------------------
!
      implicit none
!
!----------------------------------------------------------------------
      integer(kind=kint),intent(in):: &
       lm
!
      real(kind=kfpt),intent(in):: &
       dtdif
!
      real(kind=kfpt),dimension(1:lm-1),intent(in):: &
       el
!
      real(kind=kfpt),dimension(1:lm+1),intent(in):: &
       z
!
      real(kind=kfpt),dimension(1:lm),intent(inout):: &
       q2
!----------------------------------------------------------------------
!***
!***  local variables
!***
      integer(kind=kint):: &
       k
!
      real(kind=kfpt):: &
       aden,akqs,bden,besh,besm,cden,cf,dtozs,ell,eloq2,eloq4 &
      ,elqdz,esh,esm,esqhf,ghl,gml,q1l,rden,rdz
!
      real(kind=kfpt),dimension(1:lm-2):: &
       akq,cm,cr,dtoz,rsq2
!----------------------------------------------------------------------
!**********************************************************************
!----------------------------------------------------------------------
!***
!***  vertical turbulent diffusion
!***
!----------------------------------------------------------------------
      esqhf=0.5*esq
!
      do k=1,lm-2
        dtoz(k)=(dtdif+dtdif)/(z(k)-z(k+2))
        akq(k)=sqrt((q2(k)+q2(k+1))*0.5)*(el(k)+el(k+1))*esqhf                  &
              /(z(k+1)-z(k+2))
        cr(k)=-dtoz(k)*akq(k)
      enddo
!
      cm(1)=dtoz(1)*akq(1)+1.
      rsq2(1)=q2(1)
!
      do k=1+1,lm-2
        cf=-dtoz(k)*akq(k-1)/cm(k-1)
        cm(k)=-cr(k-1)*cf+(akq(k-1)+akq(k))*dtoz(k)+1.
        rsq2(k)=-rsq2(k-1)*cf+q2(k)
      enddo
!
      dtozs=(dtdif+dtdif)/(z(lm-1)-z(lm+1))
      akqs=sqrt((q2(lm-1)+q2(lm))*0.5)*(el(lm-1)+elz0)*esqhf                 &
          /(z(lm)-z(lm+1))
!
      cf=-dtozs*akq(lm-2)/cm(lm-2)
!
      q2(lm-1)=(dtozs*akqs*q2(lm)-rsq2(lm-2)*cf+q2(lm-1))                   &
               /((akq(lm-2)+akqs)*dtozs-cr(lm-2)*cf+1.)
!
      do k=lm-2,1,-1
        q2(k)=(-cr(k)*q2(k+1)+rsq2(k))/cm(k)
      enddo
!----------------------------------------------------------------------
      end subroutine vdifq
!----------------------------------------------------------------------
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!----------------------------------------------------------------------
      subroutine vdifh &
      (lm,dtdif,thz0,qz0,rkhs,chklmq,ct &
      ,th,q,cwm,rkh,z,rho)
!     ***************************************************************
!     *                                                             *
!     *         vertical diffusion of mass variables                *
!     *                                                             *
!     ***************************************************************
!---------------------------------------------------------------------
!
      implicit none
!
!---------------------------------------------------------------------
      integer(kind=kint),intent(in):: &
       lm
!
      real(kind=kfpt),intent(in):: &
       chklmq,ct,dtdif,qz0,rkhs,thz0
!
      real(kind=kfpt),dimension(1:lm-1),intent(in):: &
       rkh
!
      real(kind=kfpt),dimension(1:lm),intent(in):: &
       rho
!
      real(kind=kfpt),dimension(1:lm+1),intent(in):: &
       z
!
      real(kind=kfpt),dimension(1:lm),intent(inout):: &
       cwm,q,th
!
!----------------------------------------------------------------------
!***
!***  local variables
!***
      integer(kind=kint):: &
       k
!
      real(kind=kfpt):: &
       cf,cmb,cmcb,cmqb,cmtb,cthf,dtozl,dtozs &
      ,rcml,rkhh,rkqs,rscb,rsqb,rstb
!
      real(kind=kfpt),dimension(1:lm-1):: &
       cm,cr,dtoz,rkct,rsc,rsq,rst
!
!----------------------------------------------------------------------
!**********************************************************************
!----------------------------------------------------------------------
      cthf=0.5*ct
!
      do k=1,lm-1
        dtoz(k)=dtdif/(z(k)-z(k+1))
        cr(k)=-dtoz(k)*rkh(k)
        rkct(k)=rkh(k)*(z(k)-z(k+2))*cthf
      enddo
!
      cm(1)=dtoz(1)*rkh(1)+rho(1)
!----------------------------------------------------------------------
      rst(1)=th (1)*rho(1)-rkct(1)*dtoz(1)
      rsq(1)=q  (1)*rho(1)
      rsc(1)=cwm(1)*rho(1)
!----------------------------------------------------------------------
      do k=1+1,lm-1
        dtozl=dtoz(k)
        cf=-dtozl*rkh(k-1)/cm(k-1)
        cm(k)=-cr(k-1)*cf+(rkh(k-1)+rkh(k))*dtozl+rho(k)
        rst(k)=-rst(k-1)*cf+(rkct(k-1)-rkct(k))*dtozl+th(k)*rho(k)
        rsq(k)=-rsq(k-1)*cf+q(k)  *rho(k)
        rsc(k)=-rsc(k-1)*cf+cwm(k)*rho(k)
      enddo
!
      dtozs=dtdif/(z(lm)-z(lm+1))
      rkhh=rkh(lm-1)
!
      cf=-dtozs*rkhh/cm(lm-1)
      rkqs=rkhs*chklmq
!
      cmb=cr(lm-1)*cf
      cmtb=-cmb+(rkhh+rkhs)*dtozs+rho(lm)
      cmqb=-cmb+(rkhh+rkqs)*dtozs+rho(lm)
      cmcb=-cmb+(rkhh     )*dtozs+rho(lm)
!
      rstb=-rst(lm-1)*cf+rkct(lm-1)*dtozs+th(lm)*rho(lm)
      rsqb=-rsq(lm-1)*cf+q(lm)  *rho(lm)
      rscb=-rsc(lm-1)*cf+cwm(lm)*rho(lm)
!----------------------------------------------------------------------
      th(lm) =(dtozs*rkhs*thz0+rstb)/cmtb
      q(lm)  =(dtozs*rkqs*qz0 +rsqb)/cmqb
      cwm(lm)=(                rscb)/cmcb
!----------------------------------------------------------------------
      do k=lm-1,1,-1
        rcml=1./cm(k)
        th(k) =(-cr(k)* th(k+1)+rst(k))*rcml
        q(k)  =(-cr(k)*  q(k+1)+rsq(k))*rcml
        cwm(k)=(-cr(k)*cwm(k+1)+rsc(k))*rcml
      enddo
!----------------------------------------------------------------------
!
      end subroutine vdifh
!
!---------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      subroutine vdifv &
      (lm,dtdif,uz0,vz0,rkms,d,u,v,rkm,z,rho)
!     ***************************************************************
!     *                                                             *
!     *        vertical diffusion of velocity components            *
!     *                                                             *
!     ***************************************************************
!---------------------------------------------------------------------
!
      implicit none
!
!---------------------------------------------------------------------
      integer(kind=kint),intent(in):: &
       lm
!
      real(kind=kfpt),intent(in):: &
       rkms,dtdif,uz0,vz0
!
      real(kind=kfpt),dimension(1:lm-1),intent(in):: &
       d,rkm
!
      real(kind=kfpt),dimension(1:lm),intent(in):: &
       rho
!
      real(kind=kfpt),dimension(1:lm+1),intent(in):: &
       z
!
      real(kind=kfpt),dimension(1:lm),intent(inout):: &
       u,v
!----------------------------------------------------------------------
!***
!***  local variables
!***
      integer(kind=kint):: &
       k
!
      real(kind=kfpt):: &
       cf,dtozak,dtozl,dtozs,rcml,rcmvb,rhok,rkmh
!
      real(kind=kfpt),dimension(1:lm-1):: &
       cm,cr,dtoz,rsu,rsv
!----------------------------------------------------------------------
!**********************************************************************
!----------------------------------------------------------------------
      do k=1,lm-1
        dtoz(k)=dtdif/(z(k)-z(k+1))
        cr(k)=-dtoz(k)*rkm(k)
      enddo
!
      rhok=rho(1)
      cm(1)=dtoz(1)*rkm(1)+rhok
      rsu(1)=u(1)*rhok
      rsv(1)=v(1)*rhok
!----------------------------------------------------------------------
      do k=2,lm-1
        dtozl=dtoz(k)
        cf=-dtozl*rkm(k-1)/cm(k-1)
        rhok=rho(k)
        cm(k)=-cr(k-1)*cf+(rkm(k-1)+rkm(k)+d(k)*rkms)*dtozl+rhok
        rsu(k)=-rsu(k-1)*cf+u(k)*rhok
        rsv(k)=-rsv(k-1)*cf+v(k)*rhok
      enddo
!----------------------------------------------------------------------
      dtozs=dtdif/(z(lm)-z(lm+1))
      rkmh=rkm(lm-1)
!
      cf=-dtozs*rkmh/cm(lm-1)
      rhok=rho(lm)
      rcmvb=1./((rkmh+rkms)*dtozs-cr(lm-1)*cf+rhok)
      dtozak=dtozs*rkms
!----------------------------------------------------------------------
      u(lm)=(dtozak*uz0-rsu(lm-1)*cf+u(lm)*rhok)*rcmvb
      v(lm)=(dtozak*vz0-rsv(lm-1)*cf+v(lm)*rhok)*rcmvb
!----------------------------------------------------------------------
      do k=lm-1,1,-1
        rcml=1./cm(k)
        u(k)=(-cr(k)*u(k+1)+rsu(k))*rcml
        v(k)=(-cr(k)*v(k+1)+rsv(k))*rcml
      enddo
!----------------------------------------------------------------------
!
      end subroutine vdifv
!
!-----------------------------------------------------------------------

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!----------------------------------------------------------------------
      subroutine sfcdif &
      (sm,ths,qs &
      ,uz0,vz0,thz0,qz0,hpbl &
      ,z0,ustar,wstar &
      ,ulm,vlm,tlm,thlm,thvlm,qlm,zsl &
      ,rib,rlmo,akms,akhs,ct &
      ,u10,v10,th02,th10,q02,q10 &
      ,kztm,kztm2,ztmax1,ztmax2,ztmin1,ztmin2 &
      ,dzeta1,dzeta2,fh01,fh02 &
      ,psih1,psih2,psim1,psim2 &
      ,i,j,ntsd)
!     ****************************************************************
!     *                                                              *
!     *                       surface layer                          *
!     *                                                              *
!     ****************************************************************
!----------------------------------------------------------------------
      implicit none
!----------------------------------------------------------------------
      integer(kind=kint),parameter:: &
       itrmx=05
      real(kind=kfpt),parameter:: &
       wwst=1.2,wwst2=wwst*wwst,g=9.8,ustfc=0.018/g &
      ,alfm=0.11*1.5e-5,alft=0.40*1.5e-5,alfq=0.62*1.5e-5 &
      ,vkrm=0.40,ric=0.505,excml=0.0001,excms=0.0001 &
!-----------------------------------------------------------------------
      ,epsu2=1.e-6,epsust=1.e-10,smaxu2=35.*35. &
!-----------------------------------------------------------------------
      ,small=0.35, glkbs=30.0,glkbr=9.0,grrs=glkbr/glkbs &
      ,cziv=small*glkbs &                                                       
      ,visc=1.5e-5, tvisc=2.1e-5, qvisc=2.4e-5 &
      ,rvisc=1./visc,rtvisc=1./tvisc,rqvisc=1./qvisc &
      ,sqpr=0.84,sqsc=0.79,zqrzt=sqsc/sqpr &
      ,ustr=0.225,ustc=0.7 &
      ,fzu1=cziv*visc &
      ,fzt1=rvisc    *tvisc*sqpr,fzq1=rtvisc*qvisc*zqrzt &
      ,fzt2=cziv*grrs*tvisc*sqpr,fzq2=rtvisc*qvisc*zqrzt &
!-----------------------------------------------------------------------
      ,ztfc=1. &
      ,czil=0.1,sqvisc=258.2,zilfc=-czil*vkrm*sqvisc,epszt=1.e-28
!-----------------------------------------------------------------------
      integer(kind=kint),intent(in):: &
       i,j,kztm,kztm2,ntsd
!
      real(kind=kfpt),intent(in):: &
       dzeta1,dzeta2,fh01,fh02 &
      ,hpbl,qlm,sm &
      ,thvlm,thlm,ths,tlm,ulm,vlm,zsl &
      ,ztmax1,ztmax2,ztmin1,ztmin2

      real(kind=kfpt),dimension(1:kztm),intent(in):: &
       psih1,psih2,psim1,psim2
!
      real(kind=kfpt),intent(out):: &
       ct,rib,q02,q10,rlmo,th02,th10,u10,v10
!
      real(kind=kfpt),intent(inout):: &
       akhs,akms,qz0,thz0,ustar,uz0,vz0,z0,qs
!----------------------------------------------------------------------
!***  local variables
!
      integer(kind=kint):: &
       itr,k
!
      real(kind=kfpt):: &
       btgh,btgx,cxchl,cxchs,dthv,du2,elfc,fct &
      ,hlflx,hsflx,hv,psh02,psh10,pshz,pshzl,psm10,psmz,psmzl &
      ,rdz,rdzt,rlma,rlmn,rlmp &
      ,rlogt,rlogu,rwgh,rz,rzst,rzsu &
      ,simh,simm &
      ,umflx,ustark,vmflx,wght,wghtt,wghtq,wstar2 &
      ,x,xlt,xlt4,xlu,xlu4,xt,xt4,xu,xu4,zetalt,zetalu &
      ,zetat,zetau,zq,zslt,zslu,zt,zu,zzil,czetmax
!
!***  diagnostics
!
      real(kind=kfpt):: &
       akhs02,akhs10,akms02,akms10,ekms10 &
      ,rlnt02,rlnt10,rlnu10,simh02,simh10,simm10,t02,t10 &
      ,u10e,v10e,wstar,xlt02,xlt024,xlt10 &
      ,xlt104,xlu10,xlu104,xu10,xu104,zt02,zt10,ztat02,ztat10 &
      ,ztau,ztau10,zu10,zuuz
!----------------------------------------------------------------------
!**********************************************************************
!----------------------------------------------------------------------
      rdz=1./zsl
      cxchl=excml*rdz
      cxchs=excms*rdz
!
      btgx=g/thlm
      elfc=vkarman*btgx
!
      if(hpbl>1000.)then
        btgh=btgx*hpbl
      else
        btgh=btgx*1000.
      endif
!
!----------------------------------------------------------------------
!***  sea points
!----------------------------------------------------------------------
      if(sm>0.5)then
!----------------------------------------------------------------------
        do itr=1,itrmx
!----------------------------------------------------------------------
          z0=max(ustfc*ustar*ustar,1.59e-5)
!***  viscous sublayer, janjic mwr 1994
!----------------------------------------------------------------------
          if(ustar<ustc)then
!----------------------------------------------------------------------
            if(ustar<ustr)then
              if(ntsd==0) then
                akms=cxchs
                akhs=cxchs
                qs=qlm
              endif
!
              zu=fzu1*sqrt(sqrt(z0*ustar*rvisc))/ustar
              wght=akms*zu*rvisc
              rwgh=wght/(wght+1.)
              uz0=(ulm*rwgh+uz0)*0.5
              vz0=(vlm*rwgh+vz0)*0.5
!
              zt=fzt1*zu
              zq=fzq1*zt
              wghtt=akhs*zt*rtvisc
              wghtq=akhs*zq*rqvisc
!
              if(ntsd>0)then
                thz0=((wghtt*thlm+ths)/(wghtt+1.)+thz0)*0.5
                qz0=((wghtq*qlm+qs)/(wghtq+1.)+qz0)*0.5
              else
                thz0=(wghtt*thlm+ths)/(wghtt+1.)
                qz0=(wghtq*qlm+qs)/(wghtq+1.)
              endif
!
            endif
!
            if(ustar>=ustr.and.ustar<ustc)then
              zu=z0
              uz0=0.
              vz0=0.
!
              zt=fzt2*sqrt(sqrt(z0*ustar*rvisc))/ustar
              zq=fzq2*zt
              wghtt=akhs*zt*rtvisc
              wghtq=akhs*zq*rqvisc
!
              if(ntsd>0)then
                thz0=((wghtt*thlm+ths)/(wghtt+1.)+thz0)*0.5
                qz0=((wghtq*qlm+qs)/(wghtq+1.)+qz0)*0.5
              else
                thz0=(wghtt*thlm+ths)/(wghtt+1.)
                qz0=(wghtq*qlm+qs)/(wghtq+1.)
              endif
!
            endif
!----------------------------------------------------------------------
          else
!----------------------------------------------------------------------
            zu=z0
            uz0=0.
            vz0=0.
!
            zt=z0
            thz0=ths
!
            zq=z0
            qz0=qs
!----------------------------------------------------------------------
          endif
!----------------------------------------------------------------------
          dthv=thvlm-thz0*(0.608*qz0+1.)
!
          du2=min(max((ulm-uz0)**2+(vlm-vz0)**2,epsu2),smaxu2)
!
          rib=btgx*dthv*zsl/du2
!----------------------------------------------------------------------
!         if(rib>=ric)then
!----------------------------------------------------------------------
!           akms=max( visc*rdz,cxchs)
!           akhs=max(tvisc*rdz,cxchs)
!----------------------------------------------------------------------
!         else  !  turbulent branch
!----------------------------------------------------------------------
            zslu=zsl+zu
            zslt=zsl+zt
!
            rzsu=zslu/zu
            rzst=zslt/zt
!
            rlogu=log(rzsu)
            rlogt=log(rzst)
!
!----------------------------------------------------------------------
!***  1./monin-obukhov length
!----------------------------------------------------------------------
!
            rlmo=elfc*akhs*dthv/ustar**3
!
            zetalu=zslu*rlmo
            zetalt=zslt*rlmo
            zetau=zu*rlmo
            zetat=zt*rlmo
!
            zetalu=min(max(zetalu,ztmin1),ztmax1)
            zetalt=min(max(zetalt,ztmin1),ztmax1)
            zetau=min(max(zetau,ztmin1/rzsu),ztmax1/rzsu)
            zetat=min(max(zetat,ztmin1/rzst),ztmax1/rzst)
!
!----------------------------------------------------------------------
!***   water functions
!----------------------------------------------------------------------
!
            rz=(zetau-ztmin1)/dzeta1
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            psmz=(psim1(k+2)-psim1(k+1))*rdzt+psim1(k+1)
!
            rz=(zetalu-ztmin1)/dzeta1
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            psmzl=(psim1(k+2)-psim1(k+1))*rdzt+psim1(k+1)
!
            simm=psmzl-psmz+rlogu
!
            rz=(zetat-ztmin1)/dzeta1
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            pshz=(psih1(k+2)-psih1(k+1))*rdzt+psih1(k+1)
!
            rz=(zetalt-ztmin1)/dzeta1
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            pshzl=(psih1(k+2)-psih1(k+1))*rdzt+psih1(k+1)
!
            simh=(pshzl-pshz+rlogt)*fh01
!----------------------------------------------------------------------
            ustark=ustar*vkarman
      if(abs(simm)<1.e-10.or.abs(simh)<1.e-10)then
        write(0,*)' simm=',simm,' simh=',simh,' at i=',i,' j=',j
      endif
            akms=max(ustark/simm,cxchs)
            akhs=max(ustark/simh,cxchs)
!
!----------------------------------------------------------------------
!***  beljaars correction for ustar
!----------------------------------------------------------------------
!
            if(dthv<=0.)then                                           !zj
              wstar2=wwst2*abs(btgh*akhs*dthv)**(2./3.)                !zj
            else                                                       !zj
              wstar2=0.                                                !zj
            endif                                                      !zj
            ustar=max(sqrt(akms*sqrt(du2+wstar2)),epsust)
!
!----------------------------------------------------------------------
!         endif  !  end of turbulent branch
!----------------------------------------------------------------------
!
        enddo  !  end of the iteration loop over sea points
!
!----------------------------------------------------------------------
!
!***  land points
!
!----------------------------------------------------------------------
!
      else
!
!----------------------------------------------------------------------
        if(ntsd==0)then
          qs=qlm
        endif
!
        zu=z0
        uz0=0.
        vz0=0.
!
        zt=zu*ztfc
        thz0=ths
!
        zq=zt
        qz0=qs
!----------------------------------------------------------------------
        dthv=thvlm-thz0*(0.608*qz0+1.)
!
        du2=max((ulm-uz0)**2+(vlm-vz0)**2,epsu2)
        rib=btgx*dthv*zsl/du2
!----------------------------------------------------------------------
!       if(rib>=ric)then
!         akms=max( visc*rdz,cxchl)
!         akhs=max(tvisc*rdz,cxchl)
!----------------------------------------------------------------------
!       else  !  turbulent branch
!----------------------------------------------------------------------
          zslu=zsl+zu
!
          rzsu=zslu/zu
!
          rlogu=log(rzsu)

          zslt=zsl+zu ! u,v and t are at the same level
!----------------------------------------------------------------------
!
!
!mp   remove topo modification of zilfc term
!
!
!  rib modification to zilfc term
!  7/29/2009 v wong recommends 5, change pending
!
!zj        czetmax = 10.
!          czetmax = 5.
! stable
!zj       if(dthv>0.)then
!zj         if (rib<ric) then
!zj           zzil=zilfc*(1.0+(rib/ric)*(rib/ric)*czetmax)
!zj         else
!zj           zzil=zilfc*(1.0+czetmax)
!zj         endif
! unstable
!zj       else
            zzil=zilfc
!zj       endif
!
!----------------------------------------------------------------------
!
          land_point_iteration: do itr=1,itrmx
!
!----------------------------------------------------------------------
!***  zilitinkevitch fix for zt
!----------------------------------------------------------------------
!
            zt=max(exp(zzil*sqrt(ustar*zu))*zu,epszt) ! old form
!
            rzst=zslt/zt
            rlogt=log(rzst)
!
!----------------------------------------------------------------------
!***  1./monin-obukhov length-scale
!----------------------------------------------------------------------
!
            rlmo=elfc*akhs*dthv/ustar**3
            zetalu=zslu*rlmo
            zetalt=zslt*rlmo
            zetau=zu*rlmo
            zetat=zt*rlmo
!
            zetalu=min(max(zetalu,ztmin2),ztmax2)
            zetalt=min(max(zetalt,ztmin2),ztmax2)
            zetau=min(max(zetau,ztmin2/rzsu),ztmax2/rzsu)
            zetat=min(max(zetat,ztmin2/rzst),ztmax2/rzst)
!
!----------------------------------------------------------------------
!***  land functions
!----------------------------------------------------------------------
!
            rz=(zetau-ztmin2)/dzeta2
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            psmz=(psim2(k+2)-psim2(k+1))*rdzt+psim2(k+1)
!
            rz=(zetalu-ztmin2)/dzeta2
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            psmzl=(psim2(k+2)-psim2(k+1))*rdzt+psim2(k+1)
!
            simm=psmzl-psmz+rlogu
!
            rz=(zetat-ztmin2)/dzeta2
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            pshz=(psih2(k+2)-psih2(k+1))*rdzt+psih2(k+1)
!
            rz=(zetalt-ztmin2)/dzeta2
            k=int(rz)
            rdzt=rz-real(k)
            k=min(k,kztm2)
            k=max(k,0)
            pshzl=(psih2(k+2)-psih2(k+1))*rdzt+psih2(k+1)
!
            simh=(pshzl-pshz+rlogt)*fh02
!----------------------------------------------------------------------
            ustark=ustar*vkarman
            akms=max(ustark/simm,cxchl)
            akhs=max(ustark/simh,cxchl)
!
!----------------------------------------------------------------------
!***  beljaars correction for ustar
!----------------------------------------------------------------------
!
            if(dthv<=0.)then                                           !zj
              wstar2=wwst2*abs(btgh*akhs*dthv)**(2./3.)                !zj
            else                                                       !zj
              wstar2=0.                                                !zj
            endif                                                      !zj
!
            ustar=max(sqrt(akms*sqrt(du2+wstar2)),epsust)
!
!----------------------------------------------------------------------
          enddo land_point_iteration
!----------------------------------------------------------------------
!
!       endif  !  end of turbulant branch over land
!
!----------------------------------------------------------------------
!
      endif  !  end of land/sea branch
!
!----------------------------------------------------------------------
!***  countergradient fix
!----------------------------------------------------------------------
!     hv=-akhs*dthv
!     if(hv>0.)then
!       fct=-10.*(btgx)**(-1./3.)
!       ct=fct*(hv/(hpbl*hpbl))**(2./3.)
!     else
       ct=0.
!     endif
!----------------------------------------------------------------------
!***  the following diagnostic block produces 2-m and 10-m values
!***  for temperature, moisture, and winds.  it is done here since
!***  the various quantities needed for the computation are lost
!***  upon exit from the rotuine.
!----------------------------------------------------------------------
      wstar=sqrt(wstar2)/wwst
!
      umflx=akms*(ulm -uz0 )
      vmflx=akms*(vlm -vz0 )
      hsflx=akhs*(thlm-thz0)
      hlflx=akhs*(qlm -qz0 )
!----------------------------------------------------------------------
!     if(rib>=ric)then
!----------------------------------------------------------------------
!       if(sm>0.5)then
!         akms10=max( visc/10.,cxchs)
!         akhs02=max(tvisc/02.,cxchs)
!         akhs10=max(tvisc/10.,cxchs)
!       else
!         akms10=max( visc/10.,cxchl)
!         akhs02=max(tvisc/02.,cxchl)
!         akhs10=max(tvisc/10.,cxchl)
!       endif
!----------------------------------------------------------------------
!     else
!----------------------------------------------------------------------
        zu10=zu+10.
        zt02=zt+02.
        zt10=zt+10.
!
        rlnu10=log(zu10/zu)
        rlnt02=log(zt02/zt)
        rlnt10=log(zt10/zt)
!
        ztau10=zu10*rlmo
        ztat02=zt02*rlmo
        ztat10=zt10*rlmo
!
!----------------------------------------------------------------------
!***  sea
!----------------------------------------------------------------------
!
        if(sm>0.5)then
!
!----------------------------------------------------------------------
          ztau10=min(max(ztau10,ztmin1),ztmax1)
          ztat02=min(max(ztat02,ztmin1),ztmax1)
          ztat10=min(max(ztat10,ztmin1),ztmax1)
!----------------------------------------------------------------------
          rz=(ztau10-ztmin1)/dzeta1
          k=int(rz)
          rdzt=rz-real(k)
          k=min(k,kztm2)
          k=max(k,0)
          psm10=(psim1(k+2)-psim1(k+1))*rdzt+psim1(k+1)
!
          simm10=psm10-psmz+rlnu10
!
          rz=(ztat02-ztmin1)/dzeta1
          k=int(rz)
          rdzt=rz-real(k)
          k=min(k,kztm2)
          k=max(k,0)
          psh02=(psih1(k+2)-psih1(k+1))*rdzt+psih1(k+1)
!
          simh02=(psh02-pshz+rlnt02)*fh01
!
          rz=(ztat10-ztmin1)/dzeta1
          k=int(rz)
          rdzt=rz-real(k)
          k=min(k,kztm2)
          k=max(k,0)
          psh10=(psih1(k+2)-psih1(k+1))*rdzt+psih1(k+1)
!
          simh10=(psh10-pshz+rlnt10)*fh01
!
          akms10=max(ustark/simm10,cxchs)
          akhs02=max(ustark/simh02,cxchs)
          akhs10=max(ustark/simh10,cxchs)
!
!----------------------------------------------------------------------
!***  land
!----------------------------------------------------------------------
!
        else
!
!----------------------------------------------------------------------
          ztau10=min(max(ztau10,ztmin2),ztmax2)
          ztat02=min(max(ztat02,ztmin2),ztmax2)
          ztat10=min(max(ztat10,ztmin2),ztmax2)
!----------------------------------------------------------------------
          rz=(ztau10-ztmin2)/dzeta2
          k=int(rz)
          rdzt=rz-real(k)
          k=min(k,kztm2)
          k=max(k,0)
          psm10=(psim2(k+2)-psim2(k+1))*rdzt+psim2(k+1)
!
          simm10=psm10-psmz+rlnu10
!
          rz=(ztat02-ztmin2)/dzeta2
          k=int(rz)
          rdzt=rz-real(k)
          k=min(k,kztm2)
          k=max(k,0)
          psh02=(psih2(k+2)-psih2(k+1))*rdzt+psih2(k+1)
!
          simh02=(psh02-pshz+rlnt02)*fh02
!
          rz=(ztat10-ztmin2)/dzeta2
          k=int(rz)
          rdzt=rz-real(k)
          k=min(k,kztm2)
          k=max(k,0)
          psh10=(psih2(k+2)-psih2(k+1))*rdzt+psih2(k+1)
!
          simh10=(psh10-pshz+rlnt10)*fh02
!
          akms10=ustark/simm10
          akhs02=ustark/simh02
          akhs10=ustark/simh10
!
          if(akms10<=cxchl) akms10=akms
          if(akhs02<=cxchl) akhs02=akhs
          if(akhs10<=cxchl) akhs10=akhs
!
!----------------------------------------------------------------------
        endif
!----------------------------------------------------------------------
!     endif
!----------------------------------------------------------------------
!
      u10 =umflx/akms10+uz0
      v10 =vmflx/akms10+vz0
      th02=hsflx/akhs02+thz0
      th10=hsflx/akhs10+thz0
      q02 =hlflx/akhs02+qz0
      q10 =hlflx/akhs10+qz0
!print*,umflx,vmflx,akms10,uz0,vz0,z0,u10,v10
!
!----------------------------------------------------------------------
!***  compute "equivalent" z0 to approximate local shelter readings.
!     DOESN'T WORK FOR LARGE Z0
!----------------------------------------------------------------------
!
!      u10e=u10
!      v10e=v10
!!
!      if(sm<0.5)then
!
!!1st        zuuz=min(0.5*zu,0.1)
!!1st        zu=max(0.1*zu,zuuz)
!!tst        zuuz=amin1(zu*0.50,0.3)
!!tst        zu=amax1(zu*0.3,zuuz)
!
!        zuuz=amin1(zu*0.50,0.18)
!        zu=amax1(zu*0.35,zuuz)
!!
!        zu10=zu+10.
!        rzsu=zu10/zu
!        rlnu10=log(rzsu)
!
!        zetau=zu*rlmo
!        ztau10=zu10*rlmo
!
!        ztau10=min(max(ztau10,ztmin2),ztmax2)
!        zetau=min(max(zetau,ztmin2/rzsu),ztmax2/rzsu)
!
!        rz=(ztau10-ztmin2)/dzeta2
!        k=int(rz)
!        rdzt=rz-real(k)
!        k=min(k,kztm2)
!        k=max(k,0)
!        psm10=(psim2(k+2)-psim2(k+1))*rdzt+psim2(k+1)
!        simm10=psm10-psmz+rlnu10
!        ekms10=max(ustark/simm10,cxchl)
!
!        u10e=umflx/ekms10+uz0
!        v10e=vmflx/ekms10+vz0
!
!      endif
!!
!      u10=u10e
!      v10=v10e
!
!----------------------------------------------------------------------
      endsubroutine sfcdif
!----------------------------------------------------------------------
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        subroutine herftbl
!     ******************************************************************
!     *                                                                *
!     *               positive half of erf function table              *
!     *               responsible person: z.janjic                     *
!     *                                                                *
!     ******************************************************************
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none
!--local variables------------------------------------------------------
real(kind=kfpt),parameter:: &
 pi=3.141592653589793

integer(kind=kint):: &
 k                           ! index

real(kind=kfpt):: &
 dxe &                       ! argument increment
,dxh &                       !
,rden &                      !
,x &                         ! argument
,xgmin                       !

real(kind=kfpt),dimension(1:kerfm+1):: &
 gauss                       !
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      dxe=(xemax-xemin)/float(kerfm-1)
      dxh=dxe*0.5
      xgmin=xemin-dxh
!
      rdxe=1./dxe
!
      do k=1,kerfm+1
        x=(k-1)*dxe+xgmin
        gauss(k)=exp(-x*x*0.5)
      enddo
!
      rden=1./sqrt(pi+pi)
      herff(1)=0.
      do k=2,kerfm
        herff(k)=((gauss(k)+gauss(k+1))*dxh)*rden+herff(k-1)
      enddo
!
      do k=1,kerfm
        herff(k)=0.5-herff(k)
      enddo

!-----------------------------------------------------------------------
                        endsubroutine herftbl
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        function herf(x)
!     ******************************************************************
!     *                                                                *
!     *               erf function half-table                          *
!     *               responsible person: z.janjic                     *
!     *                                                                *
!     ******************************************************************
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
implicit none

!-----------------------------------------------------------------------
real(kind=kfpt):: &
 herf

!--local variables------------------------------------------------------
integer(kind=kint):: &
 k                           ! index

real(kind=kfpt):: &
 ak &                        ! position in table
,x                           ! argument
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!-----------------------------------------------------------------------
      ak=(x-xemin)*rdxe
      k=int(ak)
      k=max(k,0)
      k=min(k,kerfm2)
!
      herf=max((herff(k+2)-herff(k+1))*(ak-real(k))+herff(k+1),0.)
!-----------------------------------------------------------------------
                        endfunction herf
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                        endmodule module_sfcpbl
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
